@@ -12,6 +12,7 @@ result of DBAPI exceptions are all subclasses of
 
 """
 
+import traceback
 
 class SQLAlchemyError(Exception):
     """Generic error class."""
@@ -78,9 +79,18 @@ class NoReferenceError(InvalidRequestError):
 class NoReferencedTableError(NoReferenceError):
     """Raised by ``ForeignKey`` when the referred ``Table`` cannot be located."""
 
+    def __init__(self, message, tname):
+        NoReferenceError.__init__(self, message)
+        self.table_name = tname
+
 class NoReferencedColumnError(NoReferenceError):
     """Raised by ``ForeignKey`` when the referred ``Column`` cannot be located."""
 
+    def __init__(self, message, tname, cname):
+        NoReferenceError.__init__(self, message)
+        self.table_name = tname
+        self.column_name = cname
+
 class NoSuchTableError(InvalidRequestError):
     """Table does not exist or is not visible to a connection."""
 
@@ -89,10 +99,64 @@ class UnboundExecutionError(InvalidRequestError):
     """SQL was attempted without a database connection to execute it on."""
 
 
-# Moved to orm.exc; compatability definition installed by orm import until 0.6
+class DontWrapMixin(object):
+    """A mixin class which, when applied to a user-defined Exception class,
+    will not be wrapped inside of :class:`.StatementError` if the error is
+    emitted within the process of executing a statement.
+    
+    E.g.::
+        from sqlalchemy.exc import DontWrapMixin
+        
+        class MyCustomException(Exception, DontWrapMixin):
+            pass
+        
+        class MySpecialType(TypeDecorator):
+            impl = String
+            
+            def process_bind_param(self, value, dialect):
+                if value == 'invalid':
+                    raise MyCustomException("invalid!")
+            
+    """
+import sys
+if sys.version_info < (2, 5):
+    class DontWrapMixin:
+        pass
+
+# Moved to orm.exc; compatibility definition installed by orm import until 0.6
 UnmappedColumnError = None
 
-class DBAPIError(SQLAlchemyError):
+class StatementError(SQLAlchemyError):
+    """An error occurred during execution of a SQL statement.
+    
+    :class:`.StatementError` wraps the exception raised
+    during execution, and features :attr:`.statement`
+    and :attr:`.params` attributes which supply context regarding
+    the specifics of the statement which had an issue.
+
+    The wrapped exception object is available in 
+    the :attr:`.orig` attribute.
+    
+    """
+
+    def __init__(self, message, statement, params, orig):
+        SQLAlchemyError.__init__(self, message)
+        self.statement = statement
+        self.params = params
+        self.orig = orig
+
+    def __str__(self):
+        if isinstance(self.params, (list, tuple)) and \
+            len(self.params) > 10 and \
+            isinstance(self.params[0], (list, dict, tuple)):
+            return ' '.join((SQLAlchemyError.__str__(self),
+                             repr(self.statement),
+                             repr(self.params[:2]),
+                             '... and a total of %i bound parameter sets' % len(self.params)))
+        return ' '.join((SQLAlchemyError.__str__(self),
+                         repr(self.statement), repr(self.params)))
+
+class DBAPIError(StatementError):
     """Raised when the execution of a database operation fails.
 
     ``DBAPIError`` wraps exceptions raised by the DB-API underlying the
@@ -103,23 +167,37 @@ class DBAPIError(SQLAlchemyError):
     that there is no guarantee that different DB-API implementations will
     raise the same exception type for any given error condition.
 
-    If the error-raising operation occured in the execution of a SQL
-    statement, that statement and its parameters will be available on
-    the exception object in the ``statement`` and ``params`` attributes.
+    :class:`.DBAPIError` features :attr:`.statement`
+    and :attr:`.params` attributes which supply context regarding
+    the specifics of the statement which had an issue, for the 
+    typical case when the error was raised within the context of
+    emitting a SQL statement.
 
-    The wrapped exception object is available in the ``orig`` attribute.
+    The wrapped exception object is available in the :attr:`.orig` attribute.
     Its type and properties are DB-API implementation specific.
 
     """
 
     @classmethod
-    def instance(cls, statement, params, orig, connection_invalidated=False):
+    def instance(cls, statement, params, 
+                        orig, 
+                        dbapi_base_err,
+                        connection_invalidated=False):
         # Don't ever wrap these, just return them directly as if
         # DBAPIError didn't exist.
-        if isinstance(orig, (KeyboardInterrupt, SystemExit)):
+        if isinstance(orig, (KeyboardInterrupt, SystemExit, DontWrapMixin)):
             return orig
 
         if orig is not None:
+            # not a DBAPI error, statement is present.
+            # raise a StatementError
+            if not isinstance(orig, dbapi_base_err) and statement:
+                return StatementError(
+                            "%s (original cause: %s)" % (
+                                str(orig), 
+                                traceback.format_exception_only(orig.__class__, orig)[-1].strip()
+                            ), statement, params, orig)
+
             name, glob = orig.__class__.__name__, globals()
             if name in glob and issubclass(glob[name], DBAPIError):
                 cls = glob[name]
@@ -133,26 +211,15 @@ class DBAPIError(SQLAlchemyError):
             raise
         except Exception, e:
             text = 'Error in str() of DB-API-generated exception: ' + str(e)
-        SQLAlchemyError.__init__(
-            self, '(%s) %s' % (orig.__class__.__name__, text))
-        self.statement = statement
-        self.params = params
-        self.orig = orig
+        StatementError.__init__(
+                self, 
+                '(%s) %s' % (orig.__class__.__name__, text),
+                statement,
+                params,
+                orig
+        )
         self.connection_invalidated = connection_invalidated
 
-    def __str__(self):
-        if isinstance(self.params, (list, tuple)) and len(self.params) > 10 and isinstance(self.params[0], (list, dict, tuple)):
-            return ' '.join((SQLAlchemyError.__str__(self),
-                             repr(self.statement),
-                             repr(self.params[:2]),
-                             '... and a total of %i bound parameter sets' % len(self.params)))
-        return ' '.join((SQLAlchemyError.__str__(self),
-                         repr(self.statement), repr(self.params)))
-
-
-# As of 0.4, SQLError is now DBAPIError.
-# SQLError alias will be removed in 0.6.
-SQLError = DBAPIError
 
 class InterfaceError(DBAPIError):
     """Wraps a DB-API InterfaceError."""
