@@ -191,67 +191,6 @@ a class, unless the :class:`.relationship` is declared with ``viewonly=True``.
 Otherwise, the unit-of-work system may attempt duplicate INSERT and
 DELETE statements against the underlying table.
 
-.. _declarative_synonyms:
-
-Defining Synonyms
-=================
-
-Synonyms are introduced in :ref:`synonyms`. To define a getter/setter
-which proxies to an underlying attribute, use
-:func:`~.synonym` with the ``descriptor`` argument.  Here we present
-using Python 2.6 style properties::
-
-    class MyClass(Base):
-        __tablename__ = 'sometable'
-
-        id = Column(Integer, primary_key=True)
-
-        _attr = Column('attr', String)
-
-        @property
-        def attr(self):
-            return self._attr
-
-        @attr.setter
-        def attr(self, attr):
-            self._attr = attr
-
-        attr = synonym('_attr', descriptor=attr)
-
-The above synonym is then usable as an instance attribute as well as a
-class-level expression construct::
-
-    x = MyClass()
-    x.attr = "some value"
-    session.query(MyClass).filter(MyClass.attr == 'some other value').all()
-
-For simple getters, the :func:`synonym_for` decorator can be used in
-conjunction with ``@property``::
-
-    class MyClass(Base):
-        __tablename__ = 'sometable'
-
-        id = Column(Integer, primary_key=True)
-        _attr = Column('attr', String)
-
-        @synonym_for('_attr')
-        @property
-        def attr(self):
-            return self._attr
-
-Similarly, :func:`comparable_using` is a front end for the
-:func:`~.comparable_property` ORM function::
-
-    class MyClass(Base):
-        __tablename__ = 'sometable'
-
-        name = Column('name', String)
-
-        @comparable_using(MyUpperCaseComparator)
-        @property
-        def uc_name(self):
-            return self.name.upper()
-
 .. _declarative_sql_expressions:
 
 Defining SQL Expressions
@@ -272,7 +211,7 @@ class declaration can be referenced directly::
             firstname + " " + lastname
         )
 
-Correlated subqueries reference the :class:`Column` objects they
+Correlated subqueries reference the :class:`.Column` objects they
 need either from the local class definition or from remote 
 classes::
 
@@ -308,6 +247,8 @@ of the ``User`` class::
             where(Address.user_id==User.id)
     ) 
 
+.. _declarative_table_args:
+
 Table Configuration
 ===================
 
@@ -323,9 +264,18 @@ dictionary::
         __tablename__ = 'sometable'
         __table_args__ = {'mysql_engine':'InnoDB'}
 
-The other, a tuple of the form
-``(arg1, arg2, ..., {kwarg1:value, ...})``, which allows positional
-arguments to be specified as well (usually constraints):: 
+The other, a tuple, where each argument is positional
+(usually constraints)::
+
+    class MyClass(Base):
+        __tablename__ = 'sometable'
+        __table_args__ = (
+                ForeignKeyConstraint(['id'], ['remote_table.id']),
+                UniqueConstraint('foo'),
+                )
+
+Keyword arguments can be specified with the above form by 
+specifying the last argument as a dictionary::
 
     class MyClass(Base):
         __tablename__ = 'sometable'
@@ -335,9 +285,6 @@ arguments to be specified as well (usually constraints)::
                 {'autoload':True}
                 )
 
-Note that the keyword parameters dictionary is required in the tuple
-form even if empty.
-
 Using a Hybrid Approach with __table__
 =======================================
 
@@ -371,10 +318,41 @@ and simply pass it to declarative classes::
 
 Some configuration schemes may find it more appropriate to use ``__table__``, 
 such as those which already take advantage of the data-driven nature of 
-:class:`.Table` to customize and/or automate schema definition.   See
-the wiki example `NamingConventions <http://www.sqlalchemy.org/trac/wiki/UsageRecipes/NamingConventions>`_
-for one such example.
+:class:`.Table` to customize and/or automate schema definition. 
 
+Note that when the ``__table__`` approach is used, the object is immediately
+usable as a plain :class:`.Table` within the class declaration body itself,
+as a Python class is only another syntactical block.  Below this is illustrated
+by using the ``id`` column in the ``primaryjoin`` condition of a :func:`.relationship`::
+
+    class MyClass(Base):
+        __table__ = Table('my_table', Base.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('name', String(50))
+        )
+
+        widgets = relationship(Widget, 
+                    primaryjoin=Widget.myclass_id==__table__.c.id)
+
+Similarly, mapped attributes which refer to ``__table__`` can be placed inline, 
+as below where we assign the ``name`` column to the attribute ``_name``, generating
+a synonym for ``name``::
+
+    from sqlalchemy.ext.declarative import synonym_for
+    
+    class MyClass(Base):
+        __table__ = Table('my_table', Base.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('name', String(50))
+        )
+
+        _name = __table__.c.name
+
+        @synonym_for("_name")
+        def name(self):
+            return "Name: %s" % _name
+
+        
 Mapper Configuration
 ====================
 
@@ -497,7 +475,9 @@ Concrete is defined as a subclass which has its own table and sets the
         name = Column(String(50))
 
 Usage of an abstract base class is a little less straightforward as it
-requires usage of :func:`~sqlalchemy.orm.util.polymorphic_union`::
+requires usage of :func:`~sqlalchemy.orm.util.polymorphic_union`,
+which needs to be created with the :class:`.Table` objects
+before the class is built::
 
     engineers = Table('engineers', Base.metadata,
                     Column('id', Integer, primary_key=True),
@@ -527,6 +507,10 @@ requires usage of :func:`~sqlalchemy.orm.util.polymorphic_union`::
         __table__ = managers
         __mapper_args__ = {'polymorphic_identity':'manager', 'concrete':True}
 
+There is a recipe which allows the above pattern to be executed
+using the declarative form, via a special base class that defers
+the creation of the mapper.  That recipe is available at
+`DeclarativeAbstractConcreteBase <http://www.sqlalchemy.org/trac/wiki/UsageRecipes/DeclarativeAbstractConcreteBase>`_
 
 Mixin Classes
 ==============
@@ -538,7 +522,7 @@ a base class and have all the other classes subclass this class.
 
 When using :mod:`~sqlalchemy.ext.declarative`, this need is met by
 using a "mixin class". A mixin class is one that isn't mapped to a
-table and doesn't subclass the declarative :class:`Base`. For example::
+table and doesn't subclass the declarative :class:`.Base`. For example::
 
     class MyMixin(object):
 
@@ -584,14 +568,14 @@ and ``b.c.id`` are two distinct Python objects, referencing their
 parent tables ``a`` and ``b`` respectively.
 
 In the case of the mixin column, it seems that only one
-:class:`Column` object is explicitly created, yet the ultimate 
+:class:`.Column` object is explicitly created, yet the ultimate 
 ``created_at`` column above must exist as a distinct Python object
 for each separate destination class.  To accomplish this, the declarative
-extension creates a **copy** of each :class:`Column` object encountered on 
+extension creates a **copy** of each :class:`.Column` object encountered on 
 a class that is detected as a mixin.
 
 This copy mechanism is limited to simple columns that have no foreign
-keys, as a :class:`ForeignKey` itself contains references to columns
+keys, as a :class:`.ForeignKey` itself contains references to columns
 which can't be properly recreated at this level.  For columns that 
 have foreign keys, as well as for the variety of mapper-level constructs
 that require destination-explicit context, the
@@ -612,7 +596,7 @@ patterns common to many classes can be defined as callables::
 
 Where above, the ``address_id`` class-level callable is executed at the 
 point at which the ``User`` class is constructed, and the declarative
-extension can use the resulting :class:`Column` object as returned by
+extension can use the resulting :class:`.Column` object as returned by
 the method without the need to copy it.
 
 Columns generated by :func:`~.declared_attr` can also be
@@ -819,7 +803,7 @@ from multiple collections::
         __tablename__='my_model'
 
         @declared_attr
-        def __table_args__(self):
+        def __table_args__(cls):
             args = dict()
             args.update(MySQLSettings.__table_args__)
             args.update(MyOtherMixin.__table_args__)
@@ -827,6 +811,25 @@ from multiple collections::
 
         id =  Column(Integer, primary_key=True)
 
+Creating Indexes with Mixins
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+To define a named, potentially multicolumn :class:`.Index` that applies to all 
+tables derived from a mixin, use the "inline" form of :class:`.Index` and establish
+it as part of ``__table_args__``::
+
+    class MyMixin(object):
+        a =  Column(Integer)
+        b =  Column(Integer)
+
+        @declared_attr
+        def __table_args__(cls):
+            return (Index('test_idx_%s' % cls.__tablename__, 'a', 'b'),)
+
+    class MyModel(Base,MyMixin):
+        __tablename__ = 'atable'
+        c =  Column(Integer,primary_key=True)
+
 
 Class Constructor
 =================
@@ -856,11 +859,11 @@ Mapped instances then make usage of
 
 """
 
-from sqlalchemy.schema import Table, Column, MetaData
+from sqlalchemy.schema import Table, Column, MetaData, _get_table_key
 from sqlalchemy.orm import synonym as _orm_synonym, mapper,\
                                 comparable_property, class_mapper
 from sqlalchemy.orm.interfaces import MapperProperty
-from sqlalchemy.orm.properties import RelationshipProperty, ColumnProperty
+from sqlalchemy.orm.properties import RelationshipProperty, ColumnProperty, CompositeProperty
 from sqlalchemy.orm.util import _is_mapped_class
 from sqlalchemy import util, exc
 from sqlalchemy.sql import util as sql_util, expression
@@ -941,7 +944,6 @@ def _as_declarative(cls, classname, dict_):
                 continue
             elif base is not cls:
                 # we're a mixin.
-
                 if isinstance(obj, Column):
                     if obj.foreign_keys:
                         raise exc.InvalidRequestError(
@@ -1020,16 +1022,17 @@ def _as_declarative(cls, classname, dict_):
     our_stuff.sort(key=lambda key: our_stuff[key]._creation_order)
 
     # extract columns from the class dict
-    cols = []
+    cols = set()
     for key, c in our_stuff.iteritems():
-        if isinstance(c, ColumnProperty):
+        if isinstance(c, (ColumnProperty, CompositeProperty)):
             for col in c.columns:
-                if isinstance(col, Column) and col.table is None:
+                if isinstance(col, Column) and \
+                    col.table is None:
                     _undefer_column_name(key, col)
-                    cols.append(col)
+                    cols.add(col)
         elif isinstance(c, Column):
             _undefer_column_name(key, c)
-            cols.append(c)
+            cols.add(c)
             # if the column is the same name as the key, 
             # remove it from the explicit properties dict.
             # the normal rules for assigning column-based properties
@@ -1037,7 +1040,7 @@ def _as_declarative(cls, classname, dict_):
             # in multi-column ColumnProperties.
             if key == c.key:
                 del our_stuff[key]
-
+    cols = sorted(cols, key=lambda c:c._creation_order)
     table = None
     if '__table__' not in dict_:
         if tablename is not None:
@@ -1045,14 +1048,10 @@ def _as_declarative(cls, classname, dict_):
             if isinstance(table_args, dict):
                 args, table_kw = (), table_args
             elif isinstance(table_args, tuple):
-                args = table_args[0:-1]
-                table_kw = table_args[-1]
-                if len(table_args) < 2 or not isinstance(table_kw, dict):
-                    raise exc.ArgumentError(
-                        "Tuple form of __table_args__ is "
-                        "(arg1, arg2, arg3, ..., {'kw1':val1, "
-                        "'kw2':val2, ...})"
-                    )
+                if isinstance(table_args[-1], dict):
+                    args, table_kw = table_args[0:-1], table_args[-1]
+                else:
+                    args, table_kw = table_args, {}
             else:
                 args, table_kw = (), {}
 
@@ -1076,8 +1075,7 @@ def _as_declarative(cls, classname, dict_):
     if 'inherits' not in mapper_args:
         for c in cls.__bases__:
             if _is_mapped_class(c):
-                mapper_args['inherits'] = cls._decl_class_registry.get(
-                                                            c.__name__, None)
+                mapper_args['inherits'] = c
                 break
 
     if hasattr(cls, '__mapper_cls__'):
@@ -1193,22 +1191,32 @@ class DeclarativeMeta(type):
 class _GetColumns(object):
     def __init__(self, cls):
         self.cls = cls
-    def __getattr__(self, key):
 
+    def __getattr__(self, key):
         mapper = class_mapper(self.cls, compile=False)
         if mapper:
-            prop = mapper.get_property(key, raiseerr=False)
-            if prop is None:
+            if not mapper.has_property(key):
                 raise exc.InvalidRequestError(
                             "Class %r does not have a mapped column named %r"
                             % (self.cls, key))
-            elif not isinstance(prop, ColumnProperty):
+
+            prop = mapper.get_property(key)
+            if not isinstance(prop, ColumnProperty):
                 raise exc.InvalidRequestError(
                             "Property %r is not an instance of"
                             " ColumnProperty (i.e. does not correspond"
                             " directly to a Column)." % key)
         return getattr(self.cls, key)
 
+class _GetTable(object):
+    def __init__(self, key, metadata):
+        self.key = key
+        self.metadata = metadata
+
+    def __getattr__(self, key):
+        return self.metadata.tables[
+                _get_table_key(key, self.key)
+            ]
 
 def _deferred_relationship(cls, prop):
     def resolve_arg(arg):
@@ -1219,6 +1227,8 @@ def _deferred_relationship(cls, prop):
                 return _GetColumns(cls._decl_class_registry[key])
             elif key in cls.metadata.tables:
                 return cls.metadata.tables[key]
+            elif key in cls.metadata._schemas:
+                return _GetTable(key, cls.metadata)
             else:
                 return sqlalchemy.__dict__[key]
 
@@ -1424,7 +1434,7 @@ def declarative_base(bind=None, metadata=None, mapper=None, cls=object,
       cls.__init__ by way of the normal Python semantics.
 
     :param metaclass:
-      Defaults to :class:`DeclarativeMeta`.  A metaclass or __metaclass__
+      Defaults to :class:`.DeclarativeMeta`.  A metaclass or __metaclass__
       compatible callable to use as the meta type of the generated
       declarative base class.
 
