@@ -1,22 +1,23 @@
 # -*- encoding: utf-8
-from sqlalchemy.test.testing import eq_
+from test.lib.testing import eq_
 import datetime
 import os
 import re
 import warnings
 from sqlalchemy import *
-from sqlalchemy import types, exc, schema
+from sqlalchemy import types, exc, schema, event
 from sqlalchemy.orm import *
 from sqlalchemy.sql import table, column
 from sqlalchemy.databases import mssql
 from sqlalchemy.dialects.mssql import pyodbc, mxodbc, pymssql
 from sqlalchemy.engine import url
-from sqlalchemy.test import *
-from sqlalchemy.test.testing import eq_, emits_warning_on, \
+from test.lib import *
+from test.lib.testing import eq_, emits_warning_on, \
     assert_raises_message
+from sqlalchemy.util.compat import decimal
 from sqlalchemy.engine.reflection import Inspector
 
-class CompileTest(TestBase, AssertsCompiledSQL):
+class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = mssql.dialect()
 
     def test_insert(self):
@@ -43,7 +44,7 @@ class CompileTest(TestBase, AssertsCompiledSQL):
         for expr, compile in [
             (
                 select([literal("x"), literal("y")]), 
-                "SELECT 'x', 'y'",
+                "SELECT 'x' AS anon_1, 'y' AS anon_2",
             ),
             (
                 select([t]).where(t.c.foo.in_(['x', 'y', 'z'])),
@@ -278,60 +279,49 @@ class CompileTest(TestBase, AssertsCompiledSQL):
                             'LEN(inserted.name) AS length_1 VALUES '
                             '(:name)')
 
+    def test_limit_using_top(self):
+        t = table('t', column('x', Integer), column('y', Integer))
 
-class IdentityInsertTest(TestBase, AssertsCompiledSQL):
-    __only_on__ = 'mssql'
-    __dialect__ = mssql.MSDialect()
+        s = select([t]).where(t.c.x==5).order_by(t.c.y).limit(10)
 
-    @classmethod
-    def setup_class(cls):
-        global metadata, cattable
-        metadata = MetaData(testing.db)
-
-        cattable = Table('cattable', metadata,
-            Column('id', Integer),
-            Column('description', String(50)),
-            PrimaryKeyConstraint('id', name='PK_cattable'),
+        self.assert_compile(
+            s,
+            "SELECT TOP 10 t.x, t.y FROM t WHERE t.x = :x_1 ORDER BY t.y",
+            {u'x_1': 5}
         )
 
-    def setup(self):
-        metadata.create_all()
+    def test_offset_using_window(self):
+        t = table('t', column('x', Integer), column('y', Integer))
 
-    def teardown(self):
-        metadata.drop_all()
+        s = select([t]).where(t.c.x==5).order_by(t.c.y).offset(20)
 
-    def test_compiled(self):
-        self.assert_compile(cattable.insert().values(id=9,
-                            description='Python'),
-                            'INSERT INTO cattable (id, description) '
-                            'VALUES (:id, :description)')
+        self.assert_compile(
+            s,
+            "SELECT anon_1.x, anon_1.y FROM (SELECT t.x AS x, t.y "
+            "AS y, ROW_NUMBER() OVER (ORDER BY t.y) AS "
+            "mssql_rn FROM t WHERE t.x = :x_1) AS "
+            "anon_1 WHERE mssql_rn > :mssql_rn_1",
+            {u'mssql_rn_1': 20, u'x_1': 5}
+        )
 
-    def test_execute(self):
-        cattable.insert().values(id=9, description='Python').execute()
+    def test_limit_offset_using_window(self):
+        t = table('t', column('x', Integer), column('y', Integer))
 
-        cats = cattable.select().order_by(cattable.c.id).execute()
-        eq_([(9, 'Python')], list(cats))
+        s = select([t]).where(t.c.x==5).order_by(t.c.y).limit(10).offset(20)
 
-        result = cattable.insert().values(description='PHP').execute()
-        eq_([10], result.inserted_primary_key)
-        lastcat = cattable.select().order_by(desc(cattable.c.id)).execute()
-        eq_((10, 'PHP'), lastcat.first())
+        self.assert_compile(
+            s,
+            "SELECT anon_1.x, anon_1.y "
+            "FROM (SELECT t.x AS x, t.y AS y, "
+            "ROW_NUMBER() OVER (ORDER BY t.y) AS mssql_rn "
+            "FROM t "
+            "WHERE t.x = :x_1) AS anon_1 "
+            "WHERE mssql_rn > :mssql_rn_1 AND mssql_rn <= :mssql_rn_2",
+            {u'mssql_rn_1': 20, u'mssql_rn_2': 30, u'x_1': 5}
+        )
 
-    def test_executemany(self):
-        cattable.insert().execute([{'id': 89, 'description': 'Python'},
-                                  {'id': 8, 'description': 'Ruby'},
-                                  {'id': 3, 'description': 'Perl'},
-                                  {'id': 1, 'description': 'Java'}])
-        cats = cattable.select().order_by(cattable.c.id).execute()
-        eq_([(1, 'Java'), (3, 'Perl'), (8, 'Ruby'), (89, 'Python')],
-            list(cats))
-        cattable.insert().execute([{'description': 'PHP'},
-                                  {'description': 'Smalltalk'}])
-        lastcats = \
-            cattable.select().order_by(desc(cattable.c.id)).limit(2).execute()
-        eq_([(91, 'Smalltalk'), (90, 'PHP')], list(lastcats))
 
-class SchemaAliasingTest(TestBase, AssertsCompiledSQL):
+class SchemaAliasingTest(fixtures.TestBase, AssertsCompiledSQL):
     """SQL server cannot reference schema-qualified tables in a SELECT statement, they
     must be aliased.
     """
@@ -406,7 +396,60 @@ class SchemaAliasingTest(TestBase, AssertsCompiledSQL):
 
         )
 
-class ReflectionTest(TestBase, ComparesTables):
+class IdentityInsertTest(fixtures.TestBase, AssertsCompiledSQL):
+    __only_on__ = 'mssql'
+    __dialect__ = mssql.MSDialect()
+
+    @classmethod
+    def setup_class(cls):
+        global metadata, cattable
+        metadata = MetaData(testing.db)
+
+        cattable = Table('cattable', metadata,
+            Column('id', Integer),
+            Column('description', String(50)),
+            PrimaryKeyConstraint('id', name='PK_cattable'),
+        )
+
+    def setup(self):
+        metadata.create_all()
+
+    def teardown(self):
+        metadata.drop_all()
+
+    def test_compiled(self):
+        self.assert_compile(cattable.insert().values(id=9,
+                            description='Python'),
+                            'INSERT INTO cattable (id, description) '
+                            'VALUES (:id, :description)')
+
+    def test_execute(self):
+        cattable.insert().values(id=9, description='Python').execute()
+
+        cats = cattable.select().order_by(cattable.c.id).execute()
+        eq_([(9, 'Python')], list(cats))
+
+        result = cattable.insert().values(description='PHP').execute()
+        eq_([10], result.inserted_primary_key)
+        lastcat = cattable.select().order_by(desc(cattable.c.id)).execute()
+        eq_((10, 'PHP'), lastcat.first())
+
+    def test_executemany(self):
+        cattable.insert().execute([{'id': 89, 'description': 'Python'},
+                                  {'id': 8, 'description': 'Ruby'},
+                                  {'id': 3, 'description': 'Perl'},
+                                  {'id': 1, 'description': 'Java'}])
+        cats = cattable.select().order_by(cattable.c.id).execute()
+        eq_([(1, 'Java'), (3, 'Perl'), (8, 'Ruby'), (89, 'Python')],
+            list(cats))
+        cattable.insert().execute([{'description': 'PHP'},
+                                  {'description': 'Smalltalk'}])
+        lastcats = \
+            cattable.select().order_by(desc(cattable.c.id)).limit(2).execute()
+        eq_([(91, 'Smalltalk'), (90, 'PHP')], list(lastcats))
+
+
+class ReflectionTest(fixtures.TestBase, ComparesTables):
     __only_on__ = 'mssql'
 
     def test_basic_reflection(self):
@@ -459,6 +502,7 @@ class ReflectionTest(TestBase, ComparesTables):
 
     @testing.provide_metadata
     def test_identity(self):
+        metadata = self.metadata
         table = Table(
             'identity_test', metadata,
             Column('col1', Integer, Sequence('fred', 2, 3), primary_key=True)
@@ -475,6 +519,7 @@ class ReflectionTest(TestBase, ComparesTables):
     @testing.emits_warning("Did not recognize")
     @testing.provide_metadata
     def test_skip_types(self):
+        metadata = self.metadata
         testing.db.execute("""
             create table foo (id integer primary key, data xml)
         """)
@@ -484,6 +529,7 @@ class ReflectionTest(TestBase, ComparesTables):
 
     @testing.provide_metadata
     def test_indexes_cols(self):
+        metadata = self.metadata
 
         t1 = Table('t', metadata, Column('x', Integer), Column('y', Integer))
         Index('foo', t1.c.x, t1.c.y)
@@ -499,6 +545,7 @@ class ReflectionTest(TestBase, ComparesTables):
 
     @testing.provide_metadata
     def test_indexes_cols_with_commas(self):
+        metadata = self.metadata
 
         t1 = Table('t', metadata, 
                         Column('x, col', Integer, key='x'), 
@@ -517,6 +564,7 @@ class ReflectionTest(TestBase, ComparesTables):
 
     @testing.provide_metadata
     def test_indexes_cols_with_spaces(self):
+        metadata = self.metadata
 
         t1 = Table('t', metadata, Column('x col', Integer, key='x'), 
                                     Column('y', Integer))
@@ -531,7 +579,7 @@ class ReflectionTest(TestBase, ComparesTables):
             set([t2.c['x col'], t2.c.y])
         )
 
-class QueryUnicodeTest(TestBase):
+class QueryUnicodeTest(fixtures.TestBase):
 
     __only_on__ = 'mssql'
 
@@ -556,7 +604,7 @@ class QueryUnicodeTest(TestBase):
         finally:
             meta.drop_all()
 
-class QueryTest(TestBase):
+class QueryTest(fixtures.TestBase):
     __only_on__ = 'mssql'
 
     def test_fetchid_trigger(self):
@@ -688,7 +736,7 @@ class Foo(object):
         for k in kw:
             setattr(self, k, kw[k])
 
-class GenerativeQueryTest(TestBase):
+class GenerativeQueryTest(fixtures.TestBase):
     __only_on__ = 'mssql'
 
     @classmethod
@@ -722,7 +770,7 @@ class GenerativeQueryTest(TestBase):
         assert list(query[:10]) == orig[:10]
 
 
-class SchemaTest(TestBase):
+class SchemaTest(fixtures.TestBase):
 
     def setup(self):
         t = Table('sometable', MetaData(),
@@ -739,19 +787,19 @@ class SchemaTest(TestBase):
         return self.ddl_compiler.get_column_specification(self.column)
 
     def test_that_mssql_default_nullability_emits_null(self):
-        eq_("test_column VARCHAR NULL", self._column_spec())
+        eq_("test_column VARCHAR(max) NULL", self._column_spec())
 
     def test_that_mssql_none_nullability_does_not_emit_nullability(self):
         self.column.nullable = None
-        eq_("test_column VARCHAR", self._column_spec())
+        eq_("test_column VARCHAR(max)", self._column_spec())
 
     def test_that_mssql_specified_nullable_emits_null(self):
         self.column.nullable = True
-        eq_("test_column VARCHAR NULL", self._column_spec())
+        eq_("test_column VARCHAR(max) NULL", self._column_spec())
 
     def test_that_mssql_specified_not_nullable_emits_not_null(self):
         self.column.nullable = False
-        eq_("test_column VARCHAR NOT NULL", self._column_spec())
+        eq_("test_column VARCHAR(max) NOT NULL", self._column_spec())
 
 
 def full_text_search_missing():
@@ -769,7 +817,7 @@ def full_text_search_missing():
     finally:
         connection.close()
 
-class MatchTest(TestBase, AssertsCompiledSQL):
+class MatchTest(fixtures.TestBase, AssertsCompiledSQL):
 
     __only_on__ = 'mssql'
     __skip_if__ = full_text_search_missing,
@@ -883,7 +931,7 @@ class MatchTest(TestBase, AssertsCompiledSQL):
         eq_([1, 3, 5], [r.id for r in results])
 
 
-class ParseConnectTest(TestBase, AssertsCompiledSQL):
+class ParseConnectTest(fixtures.TestBase, AssertsCompiledSQL):
     @classmethod
     def setup_class(cls):
         global dialect
@@ -1005,7 +1053,7 @@ class ParseConnectTest(TestBase, AssertsCompiledSQL):
                               'Unrecognized server version info',
                               engine.connect)
 
-class TypesTest(TestBase, AssertsExecutionResults, ComparesTables):
+class TypesTest(fixtures.TestBase, AssertsExecutionResults, ComparesTables):
     __only_on__ = 'mssql'
 
     @classmethod
@@ -1019,7 +1067,6 @@ class TypesTest(TestBase, AssertsExecutionResults, ComparesTables):
     @testing.fails_on_everything_except('mssql+pyodbc',
             'this is some pyodbc-specific feature')
     def test_decimal_notation(self):
-        import decimal
         numeric_table = Table('numeric_table', metadata, Column('id',
                               Integer, Sequence('numeric_id_seq',
                               optional=True), primary_key=True),
@@ -1204,7 +1251,6 @@ class TypesTest(TestBase, AssertsExecutionResults, ComparesTables):
                                   **kw), nullable=None)
                 testing.db.dialect.type_descriptor(c.type)
                 table_args.append(c)
-
         dates_table = Table(*table_args)
         gen = testing.db.dialect.ddl_compiler(testing.db.dialect,
                 schema.CreateTable(dates_table))
@@ -1263,14 +1309,14 @@ class TypesTest(TestBase, AssertsExecutionResults, ComparesTables):
              'BINARY(10)'),
 
             (mssql.MSVarBinary, [], {},
-             'VARBINARY'),
+             'VARBINARY(max)'),
             (mssql.MSVarBinary, [10], {},
              'VARBINARY(10)'),
 
-            (types.VARBINARY, [], {},
-             'VARBINARY'),
             (types.VARBINARY, [10], {},
              'VARBINARY(10)'),
+            (types.VARBINARY, [], {},
+             'VARBINARY(max)'),
 
             (mssql.MSImage, [], {},
              'IMAGE'),
@@ -1405,14 +1451,14 @@ class TypesTest(TestBase, AssertsExecutionResults, ComparesTables):
              'NCHAR(1) COLLATE Latin1_General_CI_AS'),
 
             (mssql.MSString, [], {},
-             'VARCHAR'),
+             'VARCHAR(max)'),
             (mssql.MSString, [1], {},
              'VARCHAR(1)'),
             (mssql.MSString, [1], {'collation': 'Latin1_General_CI_AS'},
              'VARCHAR(1) COLLATE Latin1_General_CI_AS'),
 
             (mssql.MSNVarchar, [], {},
-             'NVARCHAR'),
+             'NVARCHAR(max)'),
             (mssql.MSNVarchar, [1], {},
              'NVARCHAR(1)'),
             (mssql.MSNVarchar, [1], {'collation': 'Latin1_General_CI_AS'},
@@ -1540,7 +1586,7 @@ class TypesTest(TestBase, AssertsExecutionResults, ComparesTables):
                         not in list(engine.execute(tbl.select()).first())
                 engine.execute(tbl.delete())
 
-class BinaryTest(TestBase, AssertsExecutionResults):
+class BinaryTest(fixtures.TestBase, AssertsExecutionResults):
     """Test the Binary and VarBinary types"""
 
     __only_on__ = 'mssql'
@@ -1645,7 +1691,7 @@ class BinaryTest(TestBase, AssertsExecutionResults):
         return stream
 
 
-class ReflectHugeViewTest(TestBase):
+class ReflectHugeViewTest(fixtures.TestBase):
     __only_on__ = 'mssql'
 
     def setup(self):
@@ -1665,8 +1711,8 @@ class ReflectHugeViewTest(TestBase):
             )
         assert len(view_str) > 4000
 
-        DDL(view_str).execute_at('after-create', t)
-        DDL("DROP VIEW huge_named_view").execute_at('before-drop', t)
+        event.listen(t, 'after_create', DDL(view_str) )
+        event.listen(t, 'before_drop', DDL("DROP VIEW huge_named_view") )
 
         self.metadata.create_all()
 
