@@ -1,21 +1,20 @@
 # coding: utf-8
-from sqlalchemy.test.testing import eq_, assert_raises, assert_raises_message
-from sqlalchemy.test import  engines
+from test.lib.testing import eq_, assert_raises, assert_raises_message
+from test.lib import  engines
 import datetime
-import decimal
 from sqlalchemy import *
 from sqlalchemy.orm import *
 from sqlalchemy import exc, schema, types
 from sqlalchemy.dialects.postgresql import base as postgresql
 from sqlalchemy.engine.strategies import MockEngineStrategy
-from sqlalchemy.test import *
-from sqlalchemy.test.util import round_decimal
+from sqlalchemy.util.compat import decimal
+from test.lib import *
+from test.lib.util import round_decimal
 from sqlalchemy.sql import table, column
-from sqlalchemy.test.testing import eq_
-from test.engine._base import TablesTest
+from test.lib.testing import eq_
 import logging
 
-class SequenceTest(TestBase, AssertsCompiledSQL):
+class SequenceTest(fixtures.TestBase, AssertsCompiledSQL):
 
     def test_format(self):
         seq = Sequence('my_seq_no_schema')
@@ -32,6 +31,7 @@ class SequenceTest(TestBase, AssertsCompiledSQL):
     @testing.only_on('postgresql', 'foo')
     @testing.provide_metadata
     def test_reverse_eng_name(self):
+        metadata = self.metadata
         engine = engines.testing_engine(options=dict(implicit_returning=False))
         for tname, cname in [
             ('tb1' * 30, 'abc'),
@@ -47,7 +47,7 @@ class SequenceTest(TestBase, AssertsCompiledSQL):
             r = engine.execute(t.insert())
             assert r.inserted_primary_key == [1]
 
-class CompileTest(TestBase, AssertsCompiledSQL):
+class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
 
     __dialect__ = postgresql.dialect()
 
@@ -152,6 +152,28 @@ class CompileTest(TestBase, AssertsCompiledSQL):
                             "WHERE data > 'a' AND data < 'b''s'",
                             dialect=postgresql.dialect())
 
+    def test_create_index_with_ops(self):
+        m = MetaData()
+        tbl = Table('testtbl', m,
+                    Column('data', String), 
+                    Column('data2', Integer, key='d2'))
+
+        idx = Index('test_idx1', tbl.c.data,
+                    postgresql_ops={'data': 'text_pattern_ops'})
+
+        idx2 = Index('test_idx2', tbl.c.data, tbl.c.d2,
+                    postgresql_ops={'data': 'text_pattern_ops',
+                                    'd2': 'int4_ops'})
+
+        self.assert_compile(schema.CreateIndex(idx),
+                            'CREATE INDEX test_idx1 ON testtbl '
+                            '(data text_pattern_ops)',
+                            dialect=postgresql.dialect())
+        self.assert_compile(schema.CreateIndex(idx2),
+                            'CREATE INDEX test_idx2 ON testtbl '
+                            '(data text_pattern_ops, data2 int4_ops)',
+                            dialect=postgresql.dialect())
+
     @testing.uses_deprecated(r".*'postgres_where' argument has been "
                              "renamed.*")
     def test_old_create_partial_index(self):
@@ -230,7 +252,7 @@ class CompileTest(TestBase, AssertsCompiledSQL):
             '''SELECT pg_table.col1, pg_table."variadic" FROM pg_table''')
 
 
-class FloatCoercionTest(TablesTest, AssertsExecutionResults):
+class FloatCoercionTest(fixtures.TablesTest, AssertsExecutionResults):
     __only_on__ = 'postgresql'
     __dialect__ = postgresql.dialect()
 
@@ -242,8 +264,9 @@ class FloatCoercionTest(TablesTest, AssertsExecutionResults):
         )
 
     @classmethod
-    @testing.resolve_artifact_names
     def insert_data(cls):
+        data_table = cls.tables.data_table
+
         data_table.insert().execute(
             {'data':3},
             {'data':5},
@@ -257,8 +280,11 @@ class FloatCoercionTest(TablesTest, AssertsExecutionResults):
             {'data':9},
         )
 
-    @testing.resolve_artifact_names
+    @testing.fails_on('postgresql+zxjdbc',
+                      'XXX: postgresql+zxjdbc currently returns a Decimal result for Float')
     def test_float_coercion(self):
+        data_table = self.tables.data_table
+
         for type_, result in [
             (Numeric, decimal.Decimal('140.381230939')),
             (Float, 140.381230939),
@@ -280,11 +306,14 @@ class FloatCoercionTest(TablesTest, AssertsExecutionResults):
             ).scalar()
             eq_(round_decimal(ret, 9), result)
 
+    @testing.fails_on('postgresql+zxjdbc',
+                      'zxjdbc has no support for PG arrays')
     @testing.provide_metadata
     def test_arrays(self):
+        metadata = self.metadata
         t1 = Table('t', metadata, 
             Column('x', postgresql.ARRAY(Float)),
-            Column('y', postgresql.ARRAY(postgresql.REAL)),
+            Column('y', postgresql.ARRAY(REAL)),
             Column('z', postgresql.ARRAY(postgresql.DOUBLE_PRECISION)),
             Column('q', postgresql.ARRAY(Numeric))
         )
@@ -296,7 +325,7 @@ class FloatCoercionTest(TablesTest, AssertsExecutionResults):
             ([5], [5], [6], [decimal.Decimal("6.4")])
         )
 
-class EnumTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
+class EnumTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
 
     __only_on__ = 'postgresql'
     __dialect__ = postgresql.dialect()
@@ -503,12 +532,12 @@ class EnumTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         finally:
             metadata.drop_all()
 
-class NumericInterpretationTest(TestBase):
-
+class NumericInterpretationTest(fixtures.TestBase):
+    __only_on__ = 'postgresql'
 
     def test_numeric_codes(self):
         from sqlalchemy.dialects.postgresql import pg8000, psycopg2, base
-        from decimal import Decimal
+        from sqlalchemy.util.compat import decimal
 
         for dialect in (pg8000.dialect(), psycopg2.dialect()):
 
@@ -519,9 +548,34 @@ class NumericInterpretationTest(TestBase):
                 val = 23.7
                 if proc is not None:
                     val = proc(val)
-                assert val in (23.7, Decimal("23.7"))
+                assert val in (23.7, decimal.Decimal("23.7"))
 
-class InsertTest(TestBase, AssertsExecutionResults):
+    @testing.provide_metadata
+    def test_numeric_default(self):
+        metadata = self.metadata
+        # pg8000 appears to fail when the value is 0, 
+        # returns an int instead of decimal.
+        t =Table('t', metadata, 
+            Column('id', Integer, primary_key=True),
+            Column('nd', Numeric(asdecimal=True), default=1),
+            Column('nf', Numeric(asdecimal=False), default=1),
+            Column('fd', Float(asdecimal=True), default=1),
+            Column('ff', Float(asdecimal=False), default=1),
+        )
+        metadata.create_all()
+        r = t.insert().execute()
+
+        row = t.select().execute().first()
+        assert isinstance(row[1], decimal.Decimal)
+        assert isinstance(row[2], float)
+        assert isinstance(row[3], decimal.Decimal)
+        assert isinstance(row[4], float)
+        eq_(
+            row,
+            (1, decimal.Decimal("1"), 1, decimal.Decimal("1"), 1)
+        )
+
+class InsertTest(fixtures.TestBase, AssertsExecutionResults):
 
     __only_on__ = 'postgresql'
 
@@ -533,7 +587,7 @@ class InsertTest(TestBase, AssertsExecutionResults):
 
     def teardown(self):
         metadata.drop_all()
-        metadata.tables.clear()
+        metadata.clear()
         if self.engine is not testing.db:
             self.engine.dispose()
 
@@ -962,7 +1016,7 @@ class InsertTest(TestBase, AssertsExecutionResults):
         assert table.select().execute().fetchall() == [(30, 'd1'), (31,
                 'd2'), (32, 'd3'), (33, 'd4')]
 
-class DomainReflectionTest(TestBase, AssertsExecutionResults):
+class DomainReflectionTest(fixtures.TestBase, AssertsExecutionResults):
 
     """Test PostgreSQL domains"""
 
@@ -979,7 +1033,7 @@ class DomainReflectionTest(TestBase, AssertsExecutionResults):
             :
             try:
                 con.execute(ddl)
-            except exc.SQLError, e:
+            except exc.DBAPIError, e:
                 if not 'already exists' in str(e):
                     raise e
         con.execute('CREATE TABLE testtable (question integer, answer '
@@ -1070,24 +1124,153 @@ class DomainReflectionTest(TestBase, AssertsExecutionResults):
         finally:
             postgresql.PGDialect.ischema_names = ischema_names
 
+class DistinctOnTest(fixtures.TestBase, AssertsCompiledSQL):
+    """Test 'DISTINCT' with SQL expression language and orm.Query with
+    an emphasis on PG's 'DISTINCT ON' syntax.
 
-class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
+    """
+    __dialect__ = postgresql.dialect()
+
+    def setup(self):
+        self.table = Table('t', MetaData(), 
+                Column('id',Integer, primary_key=True), 
+                Column('a', String),
+                Column('b', String),
+            )
+
+    def test_plain_generative(self):
+        self.assert_compile(
+            select([self.table]).distinct(),
+            "SELECT DISTINCT t.id, t.a, t.b FROM t"
+        )
+
+    def test_on_columns_generative(self):
+        self.assert_compile(
+            select([self.table]).distinct(self.table.c.a),
+            "SELECT DISTINCT ON (t.a) t.id, t.a, t.b FROM t"
+        )
+
+    def test_on_columns_generative_multi_call(self):
+        self.assert_compile(
+            select([self.table]).distinct(self.table.c.a).
+                distinct(self.table.c.b),
+            "SELECT DISTINCT ON (t.a, t.b) t.id, t.a, t.b FROM t"
+        )
+
+    def test_plain_inline(self):
+        self.assert_compile(
+            select([self.table], distinct=True),
+            "SELECT DISTINCT t.id, t.a, t.b FROM t"
+        )
+
+    def test_on_columns_inline_list(self):
+        self.assert_compile(
+            select([self.table], 
+                    distinct=[self.table.c.a, self.table.c.b]).
+                    order_by(self.table.c.a, self.table.c.b),
+            "SELECT DISTINCT ON (t.a, t.b) t.id, "
+            "t.a, t.b FROM t ORDER BY t.a, t.b"
+        )
+
+    def test_on_columns_inline_scalar(self):
+        self.assert_compile(
+            select([self.table], distinct=self.table.c.a),
+            "SELECT DISTINCT ON (t.a) t.id, t.a, t.b FROM t"
+        )
+
+    def test_query_plain(self):
+        sess = Session()
+        self.assert_compile(
+            sess.query(self.table).distinct(),
+            "SELECT DISTINCT t.id AS t_id, t.a AS t_a, "
+            "t.b AS t_b FROM t"
+        )
+
+    def test_query_on_columns(self):
+        sess = Session()
+        self.assert_compile(
+            sess.query(self.table).distinct(self.table.c.a),
+            "SELECT DISTINCT ON (t.a) t.id AS t_id, t.a AS t_a, "
+            "t.b AS t_b FROM t"
+        )
+
+    def test_query_on_columns_multi_call(self):
+        sess = Session()
+        self.assert_compile(
+            sess.query(self.table).distinct(self.table.c.a).
+                    distinct(self.table.c.b),
+            "SELECT DISTINCT ON (t.a, t.b) t.id AS t_id, t.a AS t_a, "
+            "t.b AS t_b FROM t"
+        )
+
+    def test_query_on_columns_subquery(self):
+        sess = Session()
+        class Foo(object):
+            pass
+        mapper(Foo, self.table)
+        sess = Session()
+        self.assert_compile(
+            sess.query(Foo).from_self().distinct(Foo.a, Foo.b),
+            "SELECT DISTINCT ON (anon_1.t_a, anon_1.t_b) anon_1.t_id "
+            "AS anon_1_t_id, anon_1.t_a AS anon_1_t_a, anon_1.t_b "
+            "AS anon_1_t_b FROM (SELECT t.id AS t_id, t.a AS t_a, "
+            "t.b AS t_b FROM t) AS anon_1"
+        )
+
+    def test_query_distinct_on_aliased(self):
+        class Foo(object):
+            pass
+        mapper(Foo, self.table)
+        a1 = aliased(Foo)
+        sess = Session()
+        self.assert_compile(
+            sess.query(a1).distinct(a1.a),
+            "SELECT DISTINCT ON (t_1.a) t_1.id AS t_1_id, "
+            "t_1.a AS t_1_a, t_1.b AS t_1_b FROM t AS t_1"
+        )
+
+    def test_distinct_on_subquery_anon(self):
+
+        sq = select([self.table]).alias()
+        q = select([self.table.c.id,sq.c.id]).\
+                    distinct(sq.c.id).\
+                    where(self.table.c.id==sq.c.id)
+
+        self.assert_compile(
+            q,
+            "SELECT DISTINCT ON (anon_1.id) t.id, anon_1.id "
+            "FROM t, (SELECT t.id AS id, t.a AS a, t.b "
+            "AS b FROM t) AS anon_1 WHERE t.id = anon_1.id"
+            )
+
+    def test_distinct_on_subquery_named(self):
+        sq = select([self.table]).alias('sq')
+        q = select([self.table.c.id,sq.c.id]).\
+                    distinct(sq.c.id).\
+                    where(self.table.c.id==sq.c.id)
+        self.assert_compile(
+            q,
+            "SELECT DISTINCT ON (sq.id) t.id, sq.id "
+            "FROM t, (SELECT t.id AS id, t.a AS a, "
+            "t.b AS b FROM t) AS sq WHERE t.id = sq.id"
+            )
+
+
+class MiscTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
 
     __only_on__ = 'postgresql'
 
+    @testing.provide_metadata
     def test_date_reflection(self):
-        m1 = MetaData(testing.db)
-        t1 = Table('pgdate', m1, Column('date1',
+        metadata = self.metadata
+        t1 = Table('pgdate', metadata, Column('date1',
                    DateTime(timezone=True)), Column('date2',
                    DateTime(timezone=False)))
-        m1.create_all()
-        try:
-            m2 = MetaData(testing.db)
-            t2 = Table('pgdate', m2, autoload=True)
-            assert t2.c.date1.type.timezone is True
-            assert t2.c.date2.type.timezone is False
-        finally:
-            m1.drop_all()
+        metadata.create_all()
+        m2 = MetaData(testing.db)
+        t2 = Table('pgdate', m2, autoload=True)
+        assert t2.c.date1.type.timezone is True
+        assert t2.c.date2.type.timezone is False
 
     @testing.fails_on('+zxjdbc',
                       'The JDBC driver handles the version parsing')
@@ -1115,6 +1298,12 @@ class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             eq_(testing.db.dialect._get_server_version_info(MockConn(string)),
                 version)
 
+    @testing.only_on('postgresql+psycopg2', 'psycopg2-specific feature')
+    def test_psycopg2_version(self):
+        v = testing.db.dialect.psycopg2_version
+        assert testing.db.dialect.dbapi.__version__.\
+                    startswith(".".join(str(x) for x in v))
+
     @testing.only_on('postgresql+psycopg2', 'psycopg2-specific feature')
     def test_notice_logging(self):
         log = logging.getLogger('sqlalchemy.dialects.postgresql')
@@ -1170,76 +1359,36 @@ class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
 
     def test_checksfor_sequence(self):
         meta1 = MetaData(testing.db)
+        seq = Sequence('fooseq')
         t = Table('mytable', meta1, Column('col1', Integer,
-                  Sequence('fooseq')))
+                  seq))
+        seq.drop()
         try:
             testing.db.execute('CREATE SEQUENCE fooseq')
             t.create(checkfirst=True)
         finally:
             t.drop(checkfirst=True)
 
+    @testing.provide_metadata
     def test_renamed_sequence_reflection(self):
-        m1 = MetaData(testing.db)
-        t = Table('t', m1, Column('id', Integer, primary_key=True))
-        m1.create_all()
-        try:
-            m2 = MetaData(testing.db)
-            t2 = Table('t', m2, autoload=True, implicit_returning=False)
-            eq_(t2.c.id.server_default.arg.text,
-                "nextval('t_id_seq'::regclass)")
-            r = t2.insert().execute()
-            eq_(r.inserted_primary_key, [1])
-            testing.db.connect().execution_options(autocommit=True).\
-                    execute('alter table t_id_seq rename to foobar_id_seq'
-                    )
-            m3 = MetaData(testing.db)
-            t3 = Table('t', m3, autoload=True, implicit_returning=False)
-            eq_(t3.c.id.server_default.arg.text,
-                "nextval('foobar_id_seq'::regclass)")
-            r = t3.insert().execute()
-            eq_(r.inserted_primary_key, [2])
-        finally:
-            m1.drop_all()
-
-    def test_distinct_on(self):
-        t = Table('mytable', MetaData(testing.db), Column('id',
-                  Integer, primary_key=True), Column('a', String(8)))
-        eq_(str(t.select(distinct=t.c.a)),
-            'SELECT DISTINCT ON (mytable.a) mytable.id, mytable.a '
-            '\nFROM mytable')
-        eq_(str(t.select(distinct=['id', 'a'])),
-            'SELECT DISTINCT ON (id, a) mytable.id, mytable.a \nFROM '
-            'mytable')
-        eq_(str(t.select(distinct=[t.c.id, t.c.a])),
-            'SELECT DISTINCT ON (mytable.id, mytable.a) mytable.id, '
-            'mytable.a \nFROM mytable')
-
-    @testing.fails_if(lambda: True, "[ticket:2142]")
-    def test_distinct_on_subquery(self):
-        t1 = Table('mytable1', MetaData(testing.db), Column('id',
-                  Integer, primary_key=True), Column('a', String(8)))
-        t2 = Table('mytable2', MetaData(testing.db), Column('id',
-                  Integer, primary_key=True), Column('a', String(8)))
-
-        sq = select([t1]).alias()
-        q = select([t2.c.id,sq.c.id], distinct=sq.c.id).where(t2.c.id==sq.c.id)
-        self.assert_compile(
-            q,
-            "SELECT DISTINCT ON (anon_1.id) mytable2.id, anon_1.id "
-            "FROM mytable2, (SELECT mytable1.id AS id, mytable1.a AS a "
-            "FROM mytable1) AS anon_1 "
-            "WHERE mytable2.id = anon_1.id"
-            )
-
-        sq = select([t1]).alias('sq')
-        q = select([t2.c.id,sq.c.id], distinct=sq.c.id).where(t2.c.id==sq.c.id)
-        self.assert_compile(
-            q,
-            "SELECT DISTINCT ON (sq.id) mytable2.id, sq.id "
-            "FROM mytable2, (SELECT mytable1.id AS id, mytable1.a AS a "
-            "FROM mytable1) AS sq "
-            "WHERE mytable2.id = sq.id"
-            )
+        metadata = self.metadata
+        t = Table('t', metadata, Column('id', Integer, primary_key=True))
+        metadata.create_all()
+        m2 = MetaData(testing.db)
+        t2 = Table('t', m2, autoload=True, implicit_returning=False)
+        eq_(t2.c.id.server_default.arg.text,
+            "nextval('t_id_seq'::regclass)")
+        r = t2.insert().execute()
+        eq_(r.inserted_primary_key, [1])
+        testing.db.connect().execution_options(autocommit=True).\
+                execute('alter table t_id_seq rename to foobar_id_seq'
+                )
+        m3 = MetaData(testing.db)
+        t3 = Table('t', m3, autoload=True, implicit_returning=False)
+        eq_(t3.c.id.server_default.arg.text,
+            "nextval('foobar_id_seq'::regclass)")
+        r = t3.insert().execute()
+        eq_(r.inserted_primary_key, [2])
 
     def test_schema_reflection(self):
         """note: this test requires that the 'test_schema' schema be
@@ -1366,24 +1515,17 @@ class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         finally:
             testing.db.execute('drop table speedy_users')
 
-    @testing.emits_warning()
+    @testing.provide_metadata
     def test_index_reflection(self):
         """ Reflecting partial & expression-based indexes should warn
         """
 
-        import warnings
+        metadata = self.metadata
 
-        def capture_warnings(*args, **kw):
-            capture_warnings._orig_showwarning(*args, **kw)
-            capture_warnings.warnings.append(args)
-
-        capture_warnings._orig_showwarning = warnings.warn
-        capture_warnings.warnings = []
-        m1 = MetaData(testing.db)
-        t1 = Table('party', m1, Column('id', String(10),
+        t1 = Table('party', metadata, Column('id', String(10),
                    nullable=False), Column('name', String(20),
                    index=True), Column('aname', String(20)))
-        m1.create_all()
+        metadata.create_all()
         testing.db.execute("""
           create index idx1 on party ((id || name))
         """)
@@ -1394,17 +1536,10 @@ class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             create index idx3 on party using btree
                 (lower(name::text), lower(aname::text))
         """)
-        try:
+
+        def go():
             m2 = MetaData(testing.db)
-            warnings.warn = capture_warnings
             t2 = Table('party', m2, autoload=True)
-            wrn = capture_warnings.warnings
-            assert str(wrn[0][0]) \
-                == 'Skipped unsupported reflection of '\
-                'expression-based index idx1'
-            assert str(wrn[1][0]) \
-                == 'Predicate of partial index idx2 ignored during '\
-                'reflection'
             assert len(t2.indexes) == 2
 
             # Make sure indexes are in the order we expect them in
@@ -1417,9 +1552,16 @@ class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             assert r2.unique == False
             assert [t2.c.id] == r1.columns
             assert [t2.c.name] == r2.columns
-        finally:
-            warnings.warn = capture_warnings._orig_showwarning
-            m1.drop_all()
+
+        testing.assert_warnings(go,
+            [
+                'Skipped unsupported reflection of '
+                'expression-based index idx1',
+                'Predicate of partial index idx2 ignored during '
+                'reflection',
+                'Skipped unsupported reflection of '
+                'expression-based index idx3'
+            ])
 
     @testing.provide_metadata
     def test_index_reflection_modified(self):
@@ -1429,6 +1571,8 @@ class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
 
         """
 
+        metadata = self.metadata
+
         t1 = Table('t', metadata,
             Column('id', Integer, primary_key=True),
             Column('x', Integer)
@@ -1442,40 +1586,6 @@ class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         eq_(ind, [{'unique': False, 'column_names': [u'y'], 'name': u'idx1'}])
         conn.close()
 
-    @testing.fails_on('postgresql+pypostgresql',
-                      'pypostgresql bombs on multiple calls')
-    def test_set_isolation_level(self):
-        """Test setting the isolation level with create_engine"""
-
-        eng = create_engine(testing.db.url)
-        eq_(eng.execute('show transaction isolation level').scalar(),
-            'read committed')
-        eng = create_engine(testing.db.url,
-                            isolation_level='SERIALIZABLE')
-        eq_(eng.execute('show transaction isolation level').scalar(),
-            'serializable')
-
-        # check that it stays
-        conn = eng.connect()
-        eq_(conn.execute('show transaction isolation level').scalar(),
-            'serializable')
-        conn.close()
-
-        conn = eng.connect()
-        eq_(conn.execute('show transaction isolation level').scalar(),
-            'serializable')
-        conn.close()
-
-        eng = create_engine(testing.db.url, isolation_level='FOO')
-        if testing.db.driver == 'zxjdbc':
-            exception_cls = eng.dialect.dbapi.Error
-        elif testing.db.driver == 'psycopg2':
-            exception_cls = exc.InvalidRequestError
-        else:
-            exception_cls = eng.dialect.dbapi.ProgrammingError
-        assert_raises(exception_cls, eng.execute,
-                      'show transaction isolation level')
-
     @testing.fails_on('+zxjdbc', 'psycopg2/pg8000 specific assertion')
     @testing.fails_on('pypostgresql',
                       'psycopg2/pg8000 specific assertion')
@@ -1484,7 +1594,24 @@ class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
                     : Numeric})
         assert_raises(exc.InvalidRequestError, testing.db.execute, stmt)
 
-class TimezoneTest(TestBase):
+    def test_serial_integer(self):
+        for type_, expected in [
+            (Integer, 'SERIAL'),
+            (BigInteger, 'BIGSERIAL'),
+            (SmallInteger, 'SMALLINT'),
+            (postgresql.INTEGER, 'SERIAL'),
+            (postgresql.BIGINT, 'BIGSERIAL'),
+        ]:
+            m = MetaData()
+
+            t = Table('t', m, Column('c', type_, primary_key=True))
+            ddl_compiler = testing.db.dialect.ddl_compiler(testing.db.dialect, schema.CreateTable(t))
+            eq_(
+                ddl_compiler.get_column_specification(t.c.c),
+                "c %s NOT NULL" % expected
+            )
+
+class TimezoneTest(fixtures.TestBase):
 
     """Test timezone-aware datetimes.
 
@@ -1521,6 +1648,8 @@ class TimezoneTest(TestBase):
     def teardown_class(cls):
         metadata.drop_all()
 
+    @testing.fails_on('postgresql+zxjdbc',
+                      "XXX: postgresql+zxjdbc doesn't give a tzinfo back")
     def test_with_timezone(self):
 
         # get a date with a tzinfo
@@ -1559,7 +1688,7 @@ class TimezoneTest(TestBase):
         row = result.first()
         assert row[0] >= somedate
 
-class TimePrecisionTest(TestBase, AssertsCompiledSQL):
+class TimePrecisionTest(fixtures.TestBase, AssertsCompiledSQL):
 
     __dialect__ = postgresql.dialect()
 
@@ -1581,6 +1710,7 @@ class TimePrecisionTest(TestBase, AssertsCompiledSQL):
     @testing.only_on('postgresql', 'DB specific feature')
     @testing.provide_metadata
     def test_reflection(self):
+        metadata = self.metadata
         t1 = Table(
             't1',
             metadata,
@@ -1608,7 +1738,7 @@ class TimePrecisionTest(TestBase, AssertsCompiledSQL):
         eq_(t2.c.c5.type.timezone, False)
         eq_(t2.c.c6.type.timezone, True)
 
-class ArrayTest(TestBase, AssertsExecutionResults):
+class ArrayTest(fixtures.TestBase, AssertsExecutionResults):
 
     __only_on__ = 'postgresql'
 
@@ -1699,9 +1829,11 @@ class ArrayTest(TestBase, AssertsExecutionResults):
         class Foo(object):
             pass
 
-        footable = Table('foo', metadata, Column('id', Integer,
-                         primary_key=True), Column('intarr',
-                         postgresql.ARRAY(Integer), nullable=True))
+        footable = Table('foo', metadata, 
+                        Column('id', Integer,primary_key=True), 
+                        Column('intarr', 
+                            postgresql.ARRAY(Integer, mutable=True), 
+                            nullable=True))
         mapper(Foo, footable)
         metadata.create_all()
         sess = create_session()
@@ -1734,12 +1866,17 @@ class ArrayTest(TestBase, AssertsExecutionResults):
         sess.add(foo)
         sess.flush()
 
+    @testing.fails_on('+zxjdbc',
+                      "Can't infer the SQL type to use for an instance "
+                      "of org.python.core.PyList.")
     @testing.provide_metadata
     def test_tuple_flag(self):
+        metadata = self.metadata
         assert_raises_message(
             exc.ArgumentError, 
             "mutable must be set to False if as_tuple is True.",
-            postgresql.ARRAY, Integer, as_tuple=True)
+            postgresql.ARRAY, Integer, mutable=True, 
+                as_tuple=True)
 
         t1 = Table('t1', metadata,
             Column('id', Integer, primary_key=True),
@@ -1766,7 +1903,9 @@ class ArrayTest(TestBase, AssertsExecutionResults):
             set([('1', '2', '3'), ('4', '5', '6'), (('4', '5'), ('6', '7'))])
         )
 
-class TimestampTest(TestBase, AssertsExecutionResults):
+
+
+class TimestampTest(fixtures.TestBase, AssertsExecutionResults):
     __only_on__ = 'postgresql'
 
     def test_timestamp(self):
@@ -1777,37 +1916,46 @@ class TimestampTest(TestBase, AssertsExecutionResults):
         result = connection.execute(s).first()
         eq_(result[0], datetime.datetime(2007, 12, 25, 0, 0))
 
-class ServerSideCursorsTest(TestBase, AssertsExecutionResults):
+class ServerSideCursorsTest(fixtures.TestBase, AssertsExecutionResults):
 
     __only_on__ = 'postgresql+psycopg2'
 
-    @classmethod
-    def setup_class(cls):
-        global ss_engine
-        ss_engine = \
-            engines.testing_engine(options={'server_side_cursors'
-                                   : True})
+    def _fixture(self, server_side_cursors):
+        self.engine = engines.testing_engine(
+                        options={'server_side_cursors':server_side_cursors}
+                    )
+        return self.engine
 
-    @classmethod
-    def teardown_class(cls):
-        ss_engine.dispose()
+    def tearDown(self):
+        engines.testing_reaper.close_all()
+        self.engine.dispose()
 
-    def test_uses_ss(self):
-        result = ss_engine.execute('select 1')
-        assert result.cursor.name
-        result = ss_engine.execute(text('select 1'))
-        assert result.cursor.name
-        result = ss_engine.execute(select([1]))
+    def test_global_string(self):
+        engine = self._fixture(True)
+        result = engine.execute('select 1')
         assert result.cursor.name
 
-    def test_uses_ss_when_explicitly_enabled(self):
-        engine = engines.testing_engine(options={'server_side_cursors'
-                : False})
+    def test_global_text(self):
+        engine = self._fixture(True)
+        result = engine.execute(text('select 1'))
+        assert result.cursor.name
+
+    def test_global_expr(self):
+        engine = self._fixture(True)
+        result = engine.execute(select([1]))
+        assert result.cursor.name
+
+    def test_global_off_explicit(self):
+        engine = self._fixture(False)
         result = engine.execute(text('select 1'))
 
         # It should be off globally ...
 
         assert not result.cursor.name
+
+    def test_stmt_option(self):
+        engine = self._fixture(False)
+
         s = select([1]).execution_options(stream_results=True)
         result = engine.execute(s)
 
@@ -1815,98 +1963,78 @@ class ServerSideCursorsTest(TestBase, AssertsExecutionResults):
 
         assert result.cursor.name
 
-        # and this one
 
+    def test_conn_option(self):
+        engine = self._fixture(False)
+
+        # and this one
         result = \
             engine.connect().execution_options(stream_results=True).\
                 execute('select 1'
                 )
         assert result.cursor.name
 
-        # not this one
+    def test_stmt_enabled_conn_option_disabled(self):
+        engine = self._fixture(False)
 
+        s = select([1]).execution_options(stream_results=True)
+
+        # not this one
         result = \
             engine.connect().execution_options(stream_results=False).\
                 execute(s)
         assert not result.cursor.name
 
-    def test_ss_explicitly_disabled(self):
+    def test_stmt_option_disabled(self):
+        engine = self._fixture(True)
         s = select([1]).execution_options(stream_results=False)
-        result = ss_engine.execute(s)
+        result = engine.execute(s)
         assert not result.cursor.name
 
     def test_aliases_and_ss(self):
-        engine = engines.testing_engine(options={'server_side_cursors'
-                : False})
+        engine = self._fixture(False)
         s1 = select([1]).execution_options(stream_results=True).alias()
         result = engine.execute(s1)
         assert result.cursor.name
 
         # s1's options shouldn't affect s2 when s2 is used as a
         # from_obj.
-
         s2 = select([1], from_obj=s1)
         result = engine.execute(s2)
         assert not result.cursor.name
 
-    def test_for_update_and_ss(self):
+    def test_for_update_expr(self):
+        engine = self._fixture(True)
         s1 = select([1], for_update=True)
-        result = ss_engine.execute(s1)
-        assert result.cursor.name
-        result = ss_engine.execute('SELECT 1 FOR UPDATE')
+        result = engine.execute(s1)
         assert result.cursor.name
 
-    def test_orm_queries_with_ss(self):
-        metadata = MetaData(testing.db)
+    def test_for_update_string(self):
+        engine = self._fixture(True)
+        result = engine.execute('SELECT 1 FOR UPDATE')
+        assert result.cursor.name
 
-
-        class Foo(object):
-
-            pass
-
-
-        footable = Table('foobar', metadata, Column('id', Integer,
-                         primary_key=True))
-        mapper(Foo, footable)
-        metadata.create_all()
-        try:
-            sess = create_session()
-            engine = \
-                engines.testing_engine(options={'server_side_cursors'
-                    : False})
-            result = engine.execute(sess.query(Foo).statement)
-            assert not result.cursor.name, result.cursor.name
-            result.close()
-            q = sess.query(Foo).execution_options(stream_results=True)
-            result = engine.execute(q.statement)
-            assert result.cursor.name
-            result.close()
-            result = \
-                sess.query(Foo).execution_options(stream_results=True).\
-                    statement.execute()
-            assert result.cursor.name
-            result.close()
-        finally:
-            metadata.drop_all()
-
-    def test_text_with_ss(self):
-        engine = engines.testing_engine(options={'server_side_cursors'
-                : False})
+    def test_text_no_ss(self):
+        engine = self._fixture(False)
         s = text('select 42')
         result = engine.execute(s)
         assert not result.cursor.name
+
+    def test_text_ss_option(self):
+        engine = self._fixture(False)
         s = text('select 42').execution_options(stream_results=True)
         result = engine.execute(s)
         assert result.cursor.name
 
     def test_roundtrip(self):
-        test_table = Table('test_table', MetaData(ss_engine),
+        engine = self._fixture(True)
+        test_table = Table('test_table', MetaData(engine),
                            Column('id', Integer, primary_key=True),
                            Column('data', String(50)))
         test_table.create(checkfirst=True)
         try:
             test_table.insert().execute(data='data1')
-            nextid = ss_engine.execute(Sequence('test_table_id_seq'))
+            nextid = engine.execute(Sequence('test_table_id_seq'))
             test_table.insert().execute(id=nextid, data='data2')
             eq_(test_table.select().execute().fetchall(), [(1, 'data1'
                 ), (2, 'data2')])
@@ -1920,7 +2048,7 @@ class ServerSideCursorsTest(TestBase, AssertsExecutionResults):
         finally:
             test_table.drop(checkfirst=True)
 
-class SpecialTypesTest(TestBase, ComparesTables, AssertsCompiledSQL):
+class SpecialTypesTest(fixtures.TestBase, ComparesTables, AssertsCompiledSQL):
     """test DDL and reflection of PG-specific types """
 
     __only_on__ = 'postgresql'
@@ -1986,6 +2114,7 @@ class SpecialTypesTest(TestBase, ComparesTables, AssertsCompiledSQL):
 
     @testing.provide_metadata
     def test_bit_reflection(self):
+        metadata = self.metadata
         t1 = Table('t1', metadata,
         Column('bit1', postgresql.BIT()),
         Column('bit5', postgresql.BIT(5)),
@@ -2004,12 +2133,14 @@ class SpecialTypesTest(TestBase, ComparesTables, AssertsCompiledSQL):
         eq_(t2.c.bitvarying5.type.length, 5)
         eq_(t2.c.bitvarying5.type.varying, True)
 
-class UUIDTest(TestBase):
+class UUIDTest(fixtures.TestBase):
     """Test the bind/return values of the UUID type."""
 
     __only_on__ = 'postgresql'
 
     @testing.requires.python25
+    @testing.fails_on('postgresql+zxjdbc',
+                      'column "data" is of type uuid but expression is of type character varying')
     @testing.fails_on('postgresql+pg8000', 'No support for UUID type')
     def test_uuid_string(self):
         import uuid
@@ -2022,6 +2153,8 @@ class UUIDTest(TestBase):
         )
 
     @testing.requires.python25
+    @testing.fails_on('postgresql+zxjdbc',
+                      'column "data" is of type uuid but expression is of type character varying')
     @testing.fails_on('postgresql+pg8000', 'No support for UUID type')
     def test_uuid_uuid(self):
         import uuid
@@ -2064,7 +2197,7 @@ class UUIDTest(TestBase):
         eq_(r.fetchone(), None)
 
 
-class MatchTest(TestBase, AssertsCompiledSQL):
+class MatchTest(fixtures.TestBase, AssertsCompiledSQL):
 
     __only_on__ = 'postgresql'
     __excluded_on__ = ('postgresql', '<', (8, 3, 0)),
@@ -2172,7 +2305,7 @@ class MatchTest(TestBase, AssertsCompiledSQL):
         eq_([1, 3, 5], [r.id for r in results])
 
 
-class TupleTest(TestBase):
+class TupleTest(fixtures.TestBase):
     __only_on__ = 'postgresql'
 
     def test_tuple_containment(self):
