@@ -1,19 +1,20 @@
-from sqlalchemy.test.testing import eq_, assert_raises, assert_raises_message
+from test.lib.testing import eq_, assert_raises, assert_raises_message
 import StringIO, unicodedata
 from sqlalchemy import types as sql_types
-from sqlalchemy import schema
+from sqlalchemy import schema, events, event
 from sqlalchemy.engine.reflection import Inspector
-from sqlalchemy import MetaData
-from sqlalchemy.test.schema import Table, Column
+from sqlalchemy import MetaData, Integer
+from test.lib.schema import Table, Column
 import sqlalchemy as sa
-from sqlalchemy.test import TestBase, ComparesTables, \
+from test.lib import ComparesTables, \
                             testing, engines, AssertsCompiledSQL
+from test.lib import fixtures
 
 create_inspector = Inspector.from_engine
 
 metadata, users = None, None
 
-class ReflectionTest(TestBase, ComparesTables):
+class ReflectionTest(fixtures.TestBase, ComparesTables):
 
     @testing.exclude('mssql', '<', (10, 0, 0),
                      'Date is only supported on MSSQL 2008+')
@@ -114,7 +115,7 @@ class ReflectionTest(TestBase, ComparesTables):
             meta3 = MetaData(testing.db)
             foo = Table('foo', meta3, autoload=True)
             foo = Table('foo', meta3, include_columns=['b', 'f', 'e'],
-                        useexisting=True)
+                        extend_existing=True)
             eq_([c.name for c in foo.c], ['b', 'e', 'f'])
             for c in ('b', 'f', 'e'):
                 assert c in foo.c
@@ -331,6 +332,8 @@ class ReflectionTest(TestBase, ComparesTables):
     def test_override_composite_fk(self):
         """Test double-remove of composite foreign key, when replaced."""
 
+        metadata = self.metadata
+
         a = Table('a',
             metadata,
             Column('x', sa.Integer, primary_key=True),
@@ -391,13 +394,15 @@ class ReflectionTest(TestBase, ComparesTables):
         finally:
             meta.drop_all()
 
+    @testing.provide_metadata
     def test_nonreflected_fk_raises(self):
         """test that a NoReferencedColumnError is raised when reflecting
         a table with an FK to another table which has not included the target
         column in its reflection.
 
         """
-        meta = MetaData(testing.db)
+
+        meta = self.metadata
         a1 = Table('a', meta,
             Column('x', sa.Integer, primary_key=True),
             Column('z', sa.Integer),
@@ -408,15 +413,11 @@ class ReflectionTest(TestBase, ComparesTables):
             test_needs_fk=True
         )
         meta.create_all()
-        try:
-            m2 = MetaData(testing.db)
-            a2 = Table('a', m2, include_columns=['z'], autoload=True)
-            b2 = Table('b', m2, autoload=True)
-
-            assert_raises(sa.exc.NoReferencedColumnError, a2.join, b2)
-        finally:
-            meta.drop_all()
+        m2 = MetaData(testing.db)
+        a2 = Table('a', m2, include_columns=['z'], autoload=True)
+        b2 = Table('b', m2, autoload=True)
 
+        assert_raises(sa.exc.NoReferencedColumnError, a2.join, b2)
 
     @testing.exclude('mysql', '<', (4, 1, 1), 'innodb funkiness')
     def test_override_existing_fk(self):
@@ -476,46 +477,6 @@ class ReflectionTest(TestBase, ComparesTables):
         finally:
             meta.drop_all()
 
-    @testing.exclude('mysql', '<', (4, 1, 1), 'innodb funkiness')
-    def test_use_existing(self):
-        meta = MetaData(testing.db)
-        users = Table('users', meta, 
-                    Column('id', sa.Integer, primary_key=True), 
-                    Column('name', sa.String(30)),
-                      test_needs_fk=True)
-        addresses = Table(
-            'addresses',
-            meta,
-            Column('id', sa.Integer, primary_key=True),
-            Column('user_id', sa.Integer, sa.ForeignKey('users.id')),
-            Column('data', sa.String(100)),
-            test_needs_fk=True,
-            )
-        meta.create_all()
-        try:
-            meta2 = MetaData(testing.db)
-            addresses = Table('addresses', meta2, Column('data',
-                              sa.Unicode), autoload=True)
-            try:
-                users = Table('users', meta2, Column('name',
-                              sa.Unicode), autoload=True)
-                assert False
-            except sa.exc.InvalidRequestError, err:
-                assert str(err) \
-                    == "Table 'users' is already defined for this "\
-                    "MetaData instance.  Specify 'useexisting=True' "\
-                    "to redefine options and columns on an existing "\
-                    "Table object."
-            users = Table('users', meta2, Column('name', sa.Unicode),
-                          autoload=True, useexisting=True)
-            assert isinstance(users.c.name.type, sa.Unicode)
-            assert not users.quote
-            users = Table('users', meta2, quote=True, autoload=True,
-                          useexisting=True)
-            assert users.quote
-        finally:
-            meta.drop_all()
-
     def test_pks_not_uniques(self):
         """test that primary key reflection not tripped up by unique
         indexes"""
@@ -547,7 +508,7 @@ class ReflectionTest(TestBase, ComparesTables):
             )
 
         assert_raises_message(sa.exc.InvalidRequestError,
-                            "Foreign key assocated with column 'slots.pkg_id' "
+                            "Foreign key associated with column 'slots.pkg_id' "
                             "could not find table 'pkgs' with which to generate "
                             "a foreign key to target column 'pkg_id'",
                               metadata.create_all)
@@ -611,6 +572,7 @@ class ReflectionTest(TestBase, ComparesTables):
             self.assert_tables_equal(multi, table)
             self.assert_tables_equal(multi2, table2)
             j = sa.join(table, table2)
+
             self.assert_(sa.and_(table.c.multi_id == table2.c.foo,
                          table.c.multi_rev == table2.c.bar,
                          table.c.multi_hoho
@@ -620,7 +582,10 @@ class ReflectionTest(TestBase, ComparesTables):
 
 
     @testing.crashes('oracle', 'FIXME: unknown, confirm not fails_on')
-    @testing.fails_on('+informixdb', 'FIXME: should be supported via the DELIMITED env var but that breaks everything else for now')
+    @testing.fails_on('+informixdb', 
+                        "FIXME: should be supported via the "
+                        "DELIMITED env var but that breaks "
+                        "everything else for now")
     def test_reserved(self):
 
         # check a table that uses an SQL reserved name doesn't cause an
@@ -730,6 +695,18 @@ class ReflectionTest(TestBase, ComparesTables):
             m9.reflect()
             self.assert_(not m9.tables)
 
+    def test_reflect_all_conn_closing(self):
+        m1 = MetaData()
+        c = testing.db.connect()
+        m1.reflect(bind=c)
+        assert not c.closed
+
+    def test_inspector_conn_closing(self):
+        m1 = MetaData()
+        c = testing.db.connect()
+        i = Inspector.from_engine(testing.db)
+        assert not c.closed
+
     def test_index_reflection(self):
         m1 = MetaData(testing.db)
         t1 = Table('party', m1,
@@ -760,12 +737,14 @@ class ReflectionTest(TestBase, ComparesTables):
         finally:
             m1.drop_all()
 
+    @testing.requires.views
     @testing.provide_metadata
     def test_views(self):
+        metadata = self.metadata
         users, addresses = createTables(metadata, None)
         try:
             metadata.create_all()
-            createViews(metadata.bind, None)
+            _create_views(metadata.bind, None)
             m2 = MetaData(testing.db)
             users_v = Table("users_v", m2, autoload=True)
             addresses_v = Table("email_addresses_v", m2, autoload=True)
@@ -778,14 +757,16 @@ class ReflectionTest(TestBase, ComparesTables):
                 eq_(c1.name, c2.name)
                 self.assert_types_base(c1, c2)
         finally:
-            dropViews(metadata.bind)
+            _drop_views(metadata.bind)
 
+    @testing.requires.views
     @testing.provide_metadata
     def test_reflect_all_with_views(self):
+        metadata = self.metadata
         users, addresses = createTables(metadata, None)
         try:
             metadata.create_all()
-            createViews(metadata.bind, None)
+            _create_views(metadata.bind, None)
             m2 = MetaData(testing.db)
 
             m2.reflect(views=False)
@@ -802,9 +783,9 @@ class ReflectionTest(TestBase, ComparesTables):
                             u'users', u'email_addresses'])
             )
         finally:
-            dropViews(metadata.bind)
+            _drop_views(metadata.bind)
 
-class CreateDropTest(TestBase):
+class CreateDropTest(fixtures.TestBase):
 
     @classmethod
     def setup_class(cls):
@@ -889,7 +870,7 @@ class CreateDropTest(TestBase):
                      - set(testing.db.table_names()))
         metadata.drop_all(bind=testing.db)
 
-class SchemaManipulationTest(TestBase):
+class SchemaManipulationTest(fixtures.TestBase):
     def test_append_constraint_unique(self):
         meta = MetaData()
 
@@ -905,7 +886,7 @@ class SchemaManipulationTest(TestBase):
         assert len(addresses.c.user_id.foreign_keys) == 1
         assert addresses.constraints == set([addresses.primary_key, fk])
 
-class UnicodeReflectionTest(TestBase):
+class UnicodeReflectionTest(fixtures.TestBase):
 
     @testing.requires.unicode_connections
     def test_basic(self):
@@ -955,7 +936,7 @@ class UnicodeReflectionTest(TestBase):
             bind.dispose()
 
 
-class SchemaTest(TestBase):
+class SchemaTest(fixtures.TestBase):
 
     def test_iteration(self):
         metadata = MetaData()
@@ -1017,7 +998,7 @@ class SchemaTest(TestBase):
             metadata.drop_all()
 
 
-class HasSequenceTest(TestBase):
+class HasSequenceTest(fixtures.TestBase):
 
     @testing.requires.sequences
     def test_has_sequence(self):
@@ -1107,7 +1088,8 @@ def createIndexes(con, schema=None):
     query = "CREATE INDEX users_t_idx ON %s (test1, test2)" % fullname
     con.execute(sa.sql.text(query))
 
-def createViews(con, schema=None):
+@testing.requires.views
+def _create_views(con, schema=None):
     for table_name in ('users', 'email_addresses'):
         fullname = table_name
         if schema:
@@ -1117,7 +1099,8 @@ def createViews(con, schema=None):
                                                                    fullname)
         con.execute(sa.sql.text(query))
 
-def dropViews(con, schema=None):
+@testing.requires.views
+def _drop_views(con, schema=None):
     for table_name in ('email_addresses', 'users'):
         fullname = table_name
         if schema:
@@ -1127,7 +1110,8 @@ def dropViews(con, schema=None):
         con.execute(sa.sql.text(query))
 
 
-class ReverseCasingReflectTest(TestBase, AssertsCompiledSQL):
+class ReverseCasingReflectTest(fixtures.TestBase, AssertsCompiledSQL):
+    __dialect__ = 'default'
 
     @testing.requires.denormalized_names
     def setup(self):
@@ -1152,7 +1136,52 @@ class ReverseCasingReflectTest(TestBase, AssertsCompiledSQL):
                             'weird_casing."Col2", weird_casing."col3" '
                             'FROM weird_casing')
 
-class ComponentReflectionTest(TestBase):
+class CaseSensitiveTest(fixtures.TablesTest):
+    """Nail down case sensitive behaviors, mostly on MySQL."""
+
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('SomeTable', metadata, 
+            Column('x', Integer, primary_key=True),
+            test_needs_fk=True
+        )
+        Table('SomeOtherTable', metadata, 
+            Column('x', Integer, primary_key=True),
+            Column('y', Integer, sa.ForeignKey("SomeTable.x")),
+            test_needs_fk=True
+        )
+
+    @testing.fails_if(testing.requires._has_mysql_on_windows)
+    def test_table_names(self):
+        x = testing.db.run_callable(
+            testing.db.dialect.get_table_names
+        )
+        assert set(["SomeTable", "SomeOtherTable"]).issubset(x)
+
+    def test_reflect_exact_name(self):
+        m = MetaData()
+        t1 = Table("SomeTable", m, autoload=True, autoload_with=testing.db)
+        eq_(t1.name, "SomeTable")
+        assert t1.c.x is not None
+
+    @testing.fails_if(lambda: 
+            testing.against('mysql') and 
+            not testing.requires._has_mysql_fully_case_sensitive())
+    def test_reflect_via_fk(self):
+        m = MetaData()
+        t2 = Table("SomeOtherTable", m, autoload=True, autoload_with=testing.db)
+        eq_(t2.name, "SomeOtherTable")
+        assert "SomeTable" in m.tables
+
+    @testing.fails_if(testing.requires._has_mysql_fully_case_sensitive)
+    @testing.fails_on_everything_except('sqlite', 'mysql')
+    def test_reflect_case_insensitive(self):
+        m = MetaData()
+        t2 = Table("sOmEtAbLe", m, autoload=True, autoload_with=testing.db)
+        eq_(t2.name, "sOmEtAbLe")
+
+
+class ComponentReflectionTest(fixtures.TestBase):
 
     @testing.requires.schemas
     def test_get_schema_names(self):
@@ -1175,7 +1204,7 @@ class ComponentReflectionTest(TestBase):
         meta = MetaData(testing.db)
         (users, addresses) = createTables(meta, schema)
         meta.create_all()
-        createViews(meta.bind, schema)
+        _create_views(meta.bind, schema)
         try:
             insp = Inspector(meta.bind)
             if table_type == 'view':
@@ -1192,7 +1221,7 @@ class ComponentReflectionTest(TestBase):
                     answer = ['email_addresses', 'users']
             eq_(table_names, answer)
         finally:
-            dropViews(meta.bind, schema)
+            _drop_views(meta.bind, schema)
             addresses.drop()
             users.drop()
 
@@ -1203,6 +1232,7 @@ class ComponentReflectionTest(TestBase):
     def test_get_table_names_with_schema(self):
         self._test_get_table_names('test_schema')
 
+    @testing.requires.views
     def test_get_view_names(self):
         self._test_get_table_names(table_type='view')
 
@@ -1216,7 +1246,7 @@ class ComponentReflectionTest(TestBase):
         table_names = ['users', 'email_addresses']
         meta.create_all()
         if table_type == 'view':
-            createViews(meta.bind, schema)
+            _create_views(meta.bind, schema)
             table_names = ['users_v', 'email_addresses_v']
         try:
             insp = Inspector(meta.bind)
@@ -1257,7 +1287,7 @@ class ComponentReflectionTest(TestBase):
                                 col.type, cols[i]['name'], ctype))
         finally:
             if table_type == 'view':
-                dropViews(meta.bind, schema)
+                _drop_views(meta.bind, schema)
             addresses.drop()
             users.drop()
 
@@ -1268,9 +1298,11 @@ class ComponentReflectionTest(TestBase):
     def test_get_columns_with_schema(self):
         self._test_get_columns(schema='test_schema')
 
+    @testing.requires.views
     def test_get_view_columns(self):
         self._test_get_columns(table_type='view')
 
+    @testing.requires.views
     @testing.requires.schemas
     def test_get_view_columns_with_schema(self):
         self._test_get_columns(schema='test_schema', table_type='view')
@@ -1378,7 +1410,7 @@ class ComponentReflectionTest(TestBase):
         meta = MetaData(testing.db)
         (users, addresses) = createTables(meta, schema)
         meta.create_all()
-        createViews(meta.bind, schema)
+        _create_views(meta.bind, schema)
         view_name1 = 'users_v'
         view_name2 = 'email_addresses_v'
         try:
@@ -1388,13 +1420,15 @@ class ComponentReflectionTest(TestBase):
             v2 = insp.get_view_definition(view_name2, schema=schema)
             self.assert_(v2)
         finally:
-            dropViews(meta.bind, schema)
+            _drop_views(meta.bind, schema)
             addresses.drop()
             users.drop()
 
+    @testing.requires.views
     def test_get_view_definition(self):
         self._test_get_view_definition()
 
+    @testing.requires.views
     @testing.requires.schemas
     def test_get_view_definition_with_schema(self):
         self._test_get_view_definition(schema='test_schema')
@@ -1419,4 +1453,60 @@ class ComponentReflectionTest(TestBase):
     def test_get_table_oid_with_schema(self):
         self._test_get_table_oid('users', schema='test_schema')
 
+class ColumnEventsTest(fixtures.TestBase):
+    @classmethod
+    def setup_class(cls):
+        cls.metadata = MetaData()
+        cls.to_reflect = Table(
+            'to_reflect',
+            cls.metadata,
+            Column('x', sa.Integer, primary_key=True),
+        )
+        cls.metadata.create_all(testing.db)
 
+    @classmethod
+    def teardown_class(cls):
+        cls.metadata.drop_all(testing.db)
+
+    def teardown(self):
+        events.SchemaEventTarget.dispatch._clear()
+
+    def _do_test(self, col, update, assert_):
+        # load the actual Table class, not the test
+        # wrapper
+        from sqlalchemy.schema import Table
+
+        m = MetaData(testing.db)
+        def column_reflect(table, column_info):
+            if column_info['name'] == col:
+                column_info.update(update)
+
+        t = Table('to_reflect', m, autoload=True, listeners=[
+            ('column_reflect', column_reflect),
+        ])
+        assert_(t)
+
+        m = MetaData(testing.db)
+        event.listen(Table, 'column_reflect', column_reflect)
+        t2 = Table('to_reflect', m, autoload=True)
+        assert_(t2)
+
+    def test_override_key(self):
+        self._do_test(
+            "x", {"key":"YXZ"},
+            lambda table: eq_(table.c.YXZ.name, "x")
+        )
+
+    def test_override_type(self):
+        def assert_(table):
+            assert isinstance(table.c.x.type, sa.String)
+        self._do_test(
+            "x", {"type":sa.String},
+            assert_
+        )
+
+    def test_override_info(self):
+        self._do_test(
+            "x", {"info":{"a":"b"}},
+            lambda table: eq_(table.c.x.info, {"a":"b"})
+        )
