@@ -1,17 +1,19 @@
-from sqlalchemy.test.testing import eq_, assert_raises, \
-    assert_raises_message
+from test.lib.testing import eq_, assert_raises, \
+    assert_raises_message, ne_
 import sys
 import time
 import threading
+from test.lib.engines import testing_engine
 from sqlalchemy import create_engine, MetaData, INT, VARCHAR, Sequence, \
     select, Integer, String, func, text, exc
-from sqlalchemy.test.schema import Table
-from sqlalchemy.test.schema import Column
-from sqlalchemy.test import TestBase, testing
+from test.lib.schema import Table
+from test.lib.schema import Column
+from test.lib import testing
+from test.lib import fixtures
 
 
 users, metadata = None, None
-class TransactionTest(TestBase):
+class TransactionTest(fixtures.TestBase):
     @classmethod
     def setup_class(cls):
         global users, metadata
@@ -49,6 +51,7 @@ class TransactionTest(TestBase):
 
     def test_rollback(self):
         """test a basic rollback"""
+
         connection = testing.db.connect()
         transaction = connection.begin()
         connection.execute(users.insert(), user_id=1, user_name='user1')
@@ -156,6 +159,28 @@ class TransactionTest(TestBase):
         assert len(result.fetchall()) == 0
         connection.close()
 
+    def test_with_interface(self):
+        connection = testing.db.connect()
+        trans = connection.begin()
+        connection.execute(users.insert(), user_id=1, user_name='user1')
+        connection.execute(users.insert(), user_id=2, user_name='user2')
+        try:
+            connection.execute(users.insert(), user_id=2, user_name='user2.5')
+        except Exception, e:
+            trans.__exit__(*sys.exc_info())
+
+        assert not trans.is_active
+        self.assert_(connection.scalar('select count(*) from '
+                     'query_users') == 0)
+
+        trans = connection.begin()
+        connection.execute(users.insert(), user_id=1, user_name='user1')
+        trans.__exit__(None, None, None)
+        assert not trans.is_active
+        self.assert_(connection.scalar('select count(*) from '
+                     'query_users') == 1)
+        connection.close()
+
     def test_close(self):
         connection = testing.db.connect()
         transaction = connection.begin()
@@ -270,6 +295,10 @@ class TransactionTest(TestBase):
             [(1, ), (2, )])
         connection.close()
 
+    # PG emergency shutdown:
+    # select * from pg_prepared_xacts
+    # ROLLBACK PREPARED '<xid>'
+    @testing.requires.skip_mysql_on_windows
     @testing.requires.two_phase_transactions
     @testing.requires.savepoints
     def test_mixed_two_phase_transaction(self):
@@ -348,7 +377,7 @@ class TransactionTest(TestBase):
         eq_(result.fetchall(), [('user1', ), ('user4', )])
         conn.close()
 
-class AutoRollbackTest(TestBase):
+class AutoRollbackTest(fixtures.TestBase):
 
     @classmethod
     def setup_class(cls):
@@ -380,7 +409,7 @@ class AutoRollbackTest(TestBase):
         users.drop(conn2)
         conn2.close()
 
-class ExplicitAutoCommitTest(TestBase):
+class ExplicitAutoCommitTest(fixtures.TestBase):
 
     """test the 'autocommit' flag on select() and text() objects.
 
@@ -512,12 +541,13 @@ class ExplicitAutoCommitTest(TestBase):
 tlengine = None
 
 
-class TLTransactionTest(TestBase):
+class TLTransactionTest(fixtures.TestBase):
+    __requires__ = ('ad_hoc_engines', )
 
     @classmethod
     def setup_class(cls):
         global users, metadata, tlengine
-        tlengine = create_engine(testing.db.url, strategy='threadlocal')
+        tlengine = testing_engine(options=dict(strategy='threadlocal'))
         metadata = MetaData()
         users = Table('query_users', metadata, Column('user_id', INT,
                       Sequence('query_users_id_seq', optional=True),
@@ -530,6 +560,7 @@ class TLTransactionTest(TestBase):
 
     @classmethod
     def teardown_class(cls):
+        tlengine.close()
         metadata.drop_all(tlengine)
         tlengine.dispose()
 
@@ -539,8 +570,9 @@ class TLTransactionTest(TestBase):
 
         tlengine.close()
 
+    @testing.crashes('oracle', 'TNS error of unknown origin occurs on the buildbot.')
     def test_rollback_no_trans(self):
-        tlengine = create_engine(testing.db.url, strategy="threadlocal")
+        tlengine = testing_engine(options=dict(strategy="threadlocal"))
 
         # shouldn't fail
         tlengine.rollback()
@@ -552,7 +584,7 @@ class TLTransactionTest(TestBase):
         tlengine.rollback()
 
     def test_commit_no_trans(self):
-        tlengine = create_engine(testing.db.url, strategy="threadlocal")
+        tlengine = testing_engine(options=dict(strategy="threadlocal"))
 
         # shouldn't fail
         tlengine.commit()
@@ -564,7 +596,7 @@ class TLTransactionTest(TestBase):
         tlengine.commit()
 
     def test_prepare_no_trans(self):
-        tlengine = create_engine(testing.db.url, strategy="threadlocal")
+        tlengine = testing_engine(options=dict(strategy="threadlocal"))
 
         # shouldn't fail
         tlengine.prepare()
@@ -927,7 +959,7 @@ class TLTransactionTest(TestBase):
 
     @testing.crashes('oracle+cx_oracle', 'intermittent failures on the buildbot')
     def test_dispose(self):
-        eng = create_engine(testing.db.url, strategy='threadlocal')
+        eng = testing_engine(options=dict(strategy='threadlocal'))
         result = eng.execute(select([1]))
         eng.dispose()
         eng.execute(select([1]))
@@ -955,7 +987,8 @@ class TLTransactionTest(TestBase):
 counters = None
 
 
-class ForUpdateTest(TestBase):
+class ForUpdateTest(fixtures.TestBase):
+    __requires__ = 'ad_hoc_engines',
 
     @classmethod
     def setup_class(cls):
@@ -1106,3 +1139,134 @@ class ForUpdateTest(TestBase):
                 update_style='nowait')
         self.assert_(len(errors) != 0)
 
+class IsolationLevelTest(fixtures.TestBase):
+    __requires__ = ('isolation_level', 'ad_hoc_engines')
+
+    def _default_isolation_level(self):
+        if testing.against('sqlite'):
+            return 'SERIALIZABLE'
+        elif testing.against('postgresql'):
+            return 'READ COMMITTED'
+        else:
+            assert False, "default isolation level not known"
+
+    def _non_default_isolation_level(self):
+        if testing.against('sqlite'):
+            return 'READ UNCOMMITTED'
+        elif testing.against('postgresql'):
+            return 'SERIALIZABLE'
+        else:
+            assert False, "non default isolation level not known"
+
+    def test_engine_param_stays(self):
+
+        eng = testing_engine()
+        isolation_level = eng.dialect.get_isolation_level(eng.connect().connection)
+        level = self._non_default_isolation_level()
+
+        ne_(isolation_level, level)
+
+        eng = testing_engine(options=dict(isolation_level=level))
+        eq_(
+            eng.dialect.get_isolation_level(eng.connect().connection),
+            level
+        )
+
+        # check that it stays
+        conn = eng.connect()
+        eq_(
+            eng.dialect.get_isolation_level(conn.connection),
+            level
+        )
+        conn.close()
+
+        conn = eng.connect()
+        eq_(
+            eng.dialect.get_isolation_level(conn.connection),
+            level
+        )
+        conn.close()
+
+    def test_default_level(self):
+        eng = testing_engine(options=dict())
+        isolation_level = eng.dialect.get_isolation_level(eng.connect().connection)
+        eq_(isolation_level, self._default_isolation_level())
+
+    def test_reset_level(self):
+        eng = testing_engine(options=dict())
+        conn = eng.connect()
+        eq_(eng.dialect.get_isolation_level(conn.connection), self._default_isolation_level())
+
+        eng.dialect.set_isolation_level(conn.connection, self._non_default_isolation_level())
+        eq_(eng.dialect.get_isolation_level(conn.connection), self._non_default_isolation_level())
+
+        eng.dialect.reset_isolation_level(conn.connection)
+        eq_(eng.dialect.get_isolation_level(conn.connection), self._default_isolation_level())
+
+        conn.close()
+
+    def test_reset_level_with_setting(self):
+        eng = testing_engine(options=dict(isolation_level=self._non_default_isolation_level()))
+        conn = eng.connect()
+        eq_(eng.dialect.get_isolation_level(conn.connection), self._non_default_isolation_level())
+
+        eng.dialect.set_isolation_level(conn.connection, self._default_isolation_level())
+        eq_(eng.dialect.get_isolation_level(conn.connection), self._default_isolation_level())
+
+        eng.dialect.reset_isolation_level(conn.connection)
+        eq_(eng.dialect.get_isolation_level(conn.connection), self._non_default_isolation_level())
+
+        conn.close()
+
+    def test_invalid_level(self):
+        eng = testing_engine(options=dict(isolation_level='FOO'))
+        assert_raises_message(
+            exc.ArgumentError, 
+                "Invalid value '%s' for isolation_level. "
+                "Valid isolation levels for %s are %s" % 
+                ("FOO", eng.dialect.name, ", ".join(eng.dialect._isolation_lookup)),
+            eng.connect)
+
+    def test_per_connection(self):
+        from sqlalchemy.pool import QueuePool
+        eng = testing_engine(options=dict(poolclass=QueuePool, pool_size=2, max_overflow=0))
+
+        c1 = eng.connect()
+        c1 = c1.execution_options(isolation_level=self._non_default_isolation_level())
+
+        c2 = eng.connect()
+        eq_(eng.dialect.get_isolation_level(c1.connection), self._non_default_isolation_level())
+        eq_(eng.dialect.get_isolation_level(c2.connection), self._default_isolation_level())
+
+        c1.close()
+        c2.close()
+        c3 = eng.connect()
+        eq_(eng.dialect.get_isolation_level(c3.connection), self._default_isolation_level())
+
+        c4 = eng.connect()
+        eq_(eng.dialect.get_isolation_level(c4.connection), self._default_isolation_level())
+
+        c3.close()
+        c4.close()
+
+    def test_per_statement_bzzt(self):
+        assert_raises_message(
+            exc.ArgumentError,
+            r"'isolation_level' execution option may only be specified "
+            r"on Connection.execution_options\(\), or "
+            r"per-engine using the isolation_level "
+            r"argument to create_engine\(\).",
+            select([1]).execution_options, isolation_level=self._non_default_isolation_level()
+        )
+
+
+    def test_per_engine_bzzt(self):
+        assert_raises_message(
+            exc.ArgumentError,
+            r"'isolation_level' execution option may "
+            r"only be specified on Connection.execution_options\(\). "
+            r"To set engine-wide isolation level, "
+            r"use the isolation_level argument to create_engine\(\).",
+            create_engine,
+            testing.db.url, execution_options={'isolation_level':self._non_default_isolation_level}
+        )
