@@ -1,4 +1,4 @@
-from sqlalchemy.test.testing import eq_, assert_raises
+from test.lib.testing import eq_, assert_raises
 import copy
 import pickle
 
@@ -7,10 +7,10 @@ from sqlalchemy.orm import *
 from sqlalchemy.orm.collections import collection
 from sqlalchemy.ext.associationproxy import *
 from sqlalchemy.ext.associationproxy import _AssociationList
-from sqlalchemy.test import *
-from sqlalchemy.test.util import gc_collect
+from test.lib import *
+from test.lib.util import gc_collect
 from sqlalchemy.sql import not_
-from test.orm import _base
+from test.lib import fixtures
 
 
 class DictCollection(dict):
@@ -43,21 +43,7 @@ class ObjectCollection(object):
         return iter(self.values)
 
 
-class Parent(object):
-    kids = association_proxy('children', 'name')
-    def __init__(self, name):
-        self.name = name
-
-class Child(object):
-    def __init__(self, name):
-        self.name = name
-
-class KVChild(object):
-    def __init__(self, name, value):
-        self.name = name
-        self.value = value
-
-class _CollectionOperations(TestBase):
+class _CollectionOperations(fixtures.TestBase):
     def setup(self):
         collection_class = self.collection_class
 
@@ -681,7 +667,7 @@ class ProxyFactoryTest(ListTest):
         self._test_sequence_ops()
 
 
-class ScalarTest(TestBase):
+class ScalarTest(fixtures.TestBase):
     def test_scalar_proxy(self):
         metadata = MetaData(testing.db)
 
@@ -800,7 +786,7 @@ class ScalarTest(TestBase):
         p2.bar = 'quux'
 
 
-class LazyLoadTest(TestBase):
+class LazyLoadTest(fixtures.TestBase):
     def setup(self):
         metadata = MetaData(testing.db)
 
@@ -909,7 +895,20 @@ class LazyLoadTest(TestBase):
         self.assert_(p._children is not None)
 
 
-class ReconstitutionTest(TestBase):
+class Parent(object):
+    def __init__(self, name):
+        self.name = name
+
+class Child(object):
+    def __init__(self, name):
+        self.name = name
+
+class KVChild(object):
+    def __init__(self, name, value):
+        self.name = name
+        self.value = value
+
+class ReconstitutionTest(fixtures.TestBase):
 
     def setup(self):
         metadata = MetaData(testing.db)
@@ -928,6 +927,7 @@ class ReconstitutionTest(TestBase):
         self.metadata = metadata
         self.parents = parents
         self.children = children
+        Parent.kids = association_proxy('children', 'name')
 
     def teardown(self):
         self.metadata.drop_all()
@@ -1006,7 +1006,7 @@ class PickleKeyFunc(object):
     def __call__(self, obj):
         return getattr(obj, self.name)
 
-class ComparatorTest(_base.MappedTest):
+class ComparatorTest(fixtures.MappedTest):
 
     run_inserts = 'once'
     run_deletes = None
@@ -1015,64 +1015,106 @@ class ComparatorTest(_base.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
-        Table('userkeywords', metadata, Column('keyword_id', Integer,
-              ForeignKey('keywords.id'), primary_key=True),
-              Column('user_id', Integer, ForeignKey('users.id')))
-        Table('users', metadata, Column('id', Integer,
+        Table('userkeywords', metadata, 
+          Column('keyword_id', Integer,ForeignKey('keywords.id'), primary_key=True),
+          Column('user_id', Integer, ForeignKey('users.id'))
+        )
+        Table('users', metadata, 
+            Column('id', Integer,
               primary_key=True, test_needs_autoincrement=True),
-              Column('name', String(64)))
-        Table('keywords', metadata, Column('id', Integer,
+            Column('name', String(64)),
+            Column('singular_id', Integer, ForeignKey('singular.id'))
+        )
+        Table('keywords', metadata, 
+            Column('id', Integer,
               primary_key=True, test_needs_autoincrement=True),
-              Column('keyword', String(64)))
+            Column('keyword', String(64)),
+            Column('singular_id', Integer, ForeignKey('singular.id'))
+        )
+        Table('singular', metadata,
+            Column('id', Integer,
+              primary_key=True, test_needs_autoincrement=True),
+        )
 
     @classmethod
     def setup_classes(cls):
-        class User(_base.ComparableEntity):
+        class User(cls.Comparable):
             def __init__(self, name):
                 self.name = name
 
+            # o2m -> m2o
+            # uselist -> nonuselist
             keywords = association_proxy('user_keywords', 'keyword',
                     creator=lambda k: UserKeyword(keyword=k))
 
-        class Keyword(_base.ComparableEntity):
+            # m2o -> o2m
+            # nonuselist -> uselist
+            singular_keywords = association_proxy('singular', 'keywords')
+
+        class Keyword(cls.Comparable):
             def __init__(self, keyword):
                 self.keyword = keyword
 
+            # o2o -> m2o
+            # nonuselist -> nonuselist
             user = association_proxy('user_keyword', 'user')
 
-        class UserKeyword(_base.ComparableEntity):
+        class UserKeyword(cls.Comparable):
             def __init__(self, user=None, keyword=None):
                 self.user = user
                 self.keyword = keyword
 
+        class Singular(cls.Comparable):
+            def __init__(self, value=None):
+                self.value = value
+
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
-        mapper(User, users)
-        mapper(Keyword, keywords, properties={'user_keyword'
-               : relationship(UserKeyword, uselist=False)})
-        mapper(UserKeyword, userkeywords, properties={'user'
-               : relationship(User, backref='user_keywords'), 'keyword'
-               : relationship(Keyword)})
+        users, Keyword, UserKeyword, singular, userkeywords, User, keywords, Singular = (cls.tables.users,
+                                cls.classes.Keyword,
+                                cls.classes.UserKeyword,
+                                cls.tables.singular,
+                                cls.tables.userkeywords,
+                                cls.classes.User,
+                                cls.tables.keywords,
+                                cls.classes.Singular)
+
+        mapper(User, users, properties={
+            'singular':relationship(Singular)
+        })
+        mapper(Keyword, keywords, properties={
+            'user_keyword':relationship(UserKeyword, uselist=False)
+        })
+
+        mapper(UserKeyword, userkeywords, properties={
+            'user' : relationship(User, backref='user_keywords'), 
+            'keyword' : relationship(Keyword)
+        })
+        mapper(Singular, singular, properties={
+            'keywords': relationship(Keyword)
+        })
 
     @classmethod
-    @testing.resolve_artifact_names
     def insert_data(cls):
+        UserKeyword, User, Keyword, Singular = (cls.classes.UserKeyword,
+                                cls.classes.User,
+                                cls.classes.Keyword,
+                                cls.classes.Singular)
+
         session = sessionmaker()()
         words = (
-            'quick',
-            'brown',
-            'fox',
-            'jumped',
-            'over',
-            'the',
-            'lazy',
+            'quick', 'brown',
+            'fox', 'jumped', 'over',
+            'the', 'lazy',
             )
         for ii in range(4):
             user = User('user%d' % ii)
+            user.singular = Singular()
             session.add(user)
             for jj in words[ii:ii + 3]:
-                user.keywords.append(Keyword(jj))
+                k = Keyword(jj)
+                user.keywords.append(k)
+                user.singular.keywords.append(k)
         orphan = Keyword('orphan')
         orphan.user_keyword = UserKeyword(keyword=orphan, user=None)
         session.add(orphan)
@@ -1084,8 +1126,9 @@ class ComparatorTest(_base.MappedTest):
     def _equivalent(self, q_proxy, q_direct):
         eq_(q_proxy.all(), q_direct.all())
 
-    @testing.resolve_artifact_names
-    def test_filter_any_kwarg(self):
+    def test_filter_any_kwarg_ul_nul(self):
+        UserKeyword, User = self.classes.UserKeyword, self.classes.User
+
         self._equivalent(self.session.query(User).
                     filter(User.keywords.any(keyword='jumped'
                          )),
@@ -1094,8 +1137,9 @@ class ComparatorTest(_base.MappedTest):
                             UserKeyword.keyword.has(keyword='jumped'
                          ))))
 
-    @testing.resolve_artifact_names
-    def test_filter_has_kwarg(self):
+    def test_filter_has_kwarg_nul_nul(self):
+        UserKeyword, Keyword = self.classes.UserKeyword, self.classes.Keyword
+
         self._equivalent(self.session.query(Keyword).
                     filter(Keyword.user.has(name='user2'
                          )),
@@ -1104,8 +1148,25 @@ class ComparatorTest(_base.MappedTest):
                             UserKeyword.user.has(name='user2'
                          ))))
 
-    @testing.resolve_artifact_names
-    def test_filter_any_criterion(self):
+    def test_filter_has_kwarg_nul_ul(self):
+        User, Singular = self.classes.User, self.classes.Singular
+
+        self._equivalent(
+            self.session.query(User).\
+                        filter(User.singular_keywords.any(keyword='jumped')),
+            self.session.query(User).\
+                        filter(
+                            User.singular.has(
+                                Singular.keywords.any(keyword='jumped')
+                            )
+                        )
+        )
+
+    def test_filter_any_criterion_ul_nul(self):
+        UserKeyword, User, Keyword = (self.classes.UserKeyword,
+                                self.classes.User,
+                                self.classes.Keyword)
+
         self._equivalent(self.session.query(User).
                     filter(User.keywords.any(Keyword.keyword
                          == 'jumped')),
@@ -1114,8 +1175,11 @@ class ComparatorTest(_base.MappedTest):
                             UserKeyword.keyword.has(Keyword.keyword
                          == 'jumped'))))
 
-    @testing.resolve_artifact_names
-    def test_filter_has_criterion(self):
+    def test_filter_has_criterion_nul_nul(self):
+        UserKeyword, User, Keyword = (self.classes.UserKeyword,
+                                self.classes.User,
+                                self.classes.Keyword)
+
         self._equivalent(self.session.query(Keyword).
                 filter(Keyword.user.has(User.name
                          == 'user2')),
@@ -1124,56 +1188,95 @@ class ComparatorTest(_base.MappedTest):
                                 UserKeyword.user.has(User.name
                          == 'user2'))))
 
-    @testing.resolve_artifact_names
-    def test_filter_contains(self):
+    def test_filter_any_criterion_nul_ul(self):
+        User, Keyword, Singular = (self.classes.User,
+                                self.classes.Keyword,
+                                self.classes.Singular)
+
+        self._equivalent(
+            self.session.query(User).\
+                        filter(User.singular_keywords.any(Keyword.keyword=='jumped')),
+            self.session.query(User).\
+                        filter(
+                            User.singular.has(
+                                Singular.keywords.any(Keyword.keyword=='jumped')
+                            )
+                        )
+        )
+
+    def test_filter_contains_ul_nul(self):
+        User = self.classes.User
+
         self._equivalent(self.session.query(User).
         filter(User.keywords.contains(self.kw)),
                          self.session.query(User).
                          filter(User.user_keywords.any(keyword=self.kw)))
 
-    @testing.resolve_artifact_names
-    def test_filter_eq(self):
+    def test_filter_contains_nul_ul(self):
+        User, Singular = self.classes.User, self.classes.Singular
+
+        self._equivalent(
+            self.session.query(User).filter(
+                            User.singular_keywords.contains(self.kw)
+            ),
+            self.session.query(User).filter(
+                            User.singular.has(
+                                Singular.keywords.contains(self.kw)
+                            )
+            ),
+        )
+
+    def test_filter_eq_nul_nul(self):
+        Keyword = self.classes.Keyword
+
         self._equivalent(self.session.query(Keyword).filter(Keyword.user
                          == self.u),
                          self.session.query(Keyword).
                          filter(Keyword.user_keyword.has(user=self.u)))
 
-    @testing.resolve_artifact_names
-    def test_filter_ne(self):
+    def test_filter_ne_nul_nul(self):
+        Keyword = self.classes.Keyword
+
         self._equivalent(self.session.query(Keyword).filter(Keyword.user
                          != self.u),
                          self.session.query(Keyword).
                          filter(not_(Keyword.user_keyword.has(user=self.u))))
 
-    @testing.resolve_artifact_names
-    def test_filter_eq_null(self):
+    def test_filter_eq_null_nul_nul(self):
+        UserKeyword, Keyword = self.classes.UserKeyword, self.classes.Keyword
+
         self._equivalent(self.session.query(Keyword).filter(Keyword.user
                          == None),
                          self.session.query(Keyword).
                             filter(Keyword.user_keyword.has(UserKeyword.user
                          == None)))
 
-    @testing.resolve_artifact_names
-    def test_filter_scalar_contains_fails(self):
+    def test_filter_scalar_contains_fails_nul_nul(self):
+        Keyword = self.classes.Keyword
+
         assert_raises(exceptions.InvalidRequestError, lambda : \
                       Keyword.user.contains(self.u))
 
-    @testing.resolve_artifact_names
-    def test_filter_scalar_any_fails(self):
+    def test_filter_scalar_any_fails_nul_nul(self):
+        Keyword = self.classes.Keyword
+
         assert_raises(exceptions.InvalidRequestError, lambda : \
                       Keyword.user.any(name='user2'))
 
-    @testing.resolve_artifact_names
-    def test_filter_collection_has_fails(self):
+    def test_filter_collection_has_fails_ul_nul(self):
+        User = self.classes.User
+
         assert_raises(exceptions.InvalidRequestError, lambda : \
                       User.keywords.has(keyword='quick'))
 
-    @testing.resolve_artifact_names
-    def test_filter_collection_eq_fails(self):
+    def test_filter_collection_eq_fails_ul_nul(self):
+        User = self.classes.User
+
         assert_raises(exceptions.InvalidRequestError, lambda : \
                       User.keywords == self.kw)
 
-    @testing.resolve_artifact_names
-    def test_filter_collection_ne_fails(self):
+    def test_filter_collection_ne_fails_ul_nul(self):
+        User = self.classes.User
+
         assert_raises(exceptions.InvalidRequestError, lambda : \
                       User.keywords != self.kw)
