@@ -1,28 +1,30 @@
-from sqlalchemy.test.testing import eq_, assert_raises, \
+from test.lib.testing import eq_, assert_raises, \
     assert_raises_message
-from sqlalchemy.test.util import gc_collect
+from test.lib.util import gc_collect
+from test.lib import pickleable
+from sqlalchemy.util import pickle
 import inspect
-import pickle
 from sqlalchemy.orm import create_session, sessionmaker, attributes, \
     make_transient, Session
-from sqlalchemy.orm.attributes import instance_state
 import sqlalchemy as sa
-from sqlalchemy.test import engines, testing, config
+from test.lib import engines, testing, config
 from sqlalchemy import Integer, String, Sequence
-from sqlalchemy.test.schema import Table, Column
+from test.lib.schema import Table, Column
 from sqlalchemy.orm import mapper, relationship, backref, joinedload, \
     exc as orm_exc, object_session
-from sqlalchemy.test.testing import eq_
-from test.engine import _base as engine_base
-from test.orm import _base, _fixtures
-
+from sqlalchemy.util import pypy
+from test.lib import fixtures
+from test.lib import fixtures
+from test.orm import _fixtures
 
 class SessionTest(_fixtures.FixtureTest):
     run_inserts = None
 
-    @testing.resolve_artifact_names
     def test_no_close_on_flush(self):
         """Flush() doesn't close a connection the session didn't open"""
+
+        User, users = self.classes.User, self.tables.users
+
         c = testing.db.connect()
         c.execute("select * from users")
 
@@ -32,9 +34,11 @@ class SessionTest(_fixtures.FixtureTest):
         s.flush()
         c.execute("select * from users")
 
-    @testing.resolve_artifact_names
     def test_close(self):
         """close() doesn't close a connection the session didn't open"""
+
+        User, users = self.classes.User, self.tables.users
+
         c = testing.db.connect()
         c.execute("select * from users")
 
@@ -46,8 +50,9 @@ class SessionTest(_fixtures.FixtureTest):
         s.close()
         c.execute("select * from users")
 
-    @testing.resolve_artifact_names
     def test_no_close_transaction_on_flulsh(self):
+        User, users = self.classes.User, self.tables.users
+
         c = testing.db.connect()
         try:
             mapper(User, users)
@@ -68,17 +73,18 @@ class SessionTest(_fixtures.FixtureTest):
         finally:
             c.close()
 
-    @testing.resolve_artifact_names
     def test_object_session_raises(self):
+        User = self.classes.User
+
         assert_raises(
             orm_exc.UnmappedInstanceError,
-            object_session, 
+            object_session,
             object()
         )
 
         assert_raises(
             orm_exc.UnmappedInstanceError,
-            object_session, 
+            object_session,
             User()
         )
 
@@ -93,35 +99,14 @@ class SessionTest(_fixtures.FixtureTest):
             seq.drop(testing.db)
 
 
-    @testing.resolve_artifact_names
-    def test_expunge_cascade(self):
-        mapper(Address, addresses)
-        mapper(User, users, properties={
-            'addresses':relationship(Address,
-                                 backref=backref("user", cascade="all"),
-                                 cascade="all")})
-
-        _fixtures.run_inserts_for(users)
-        _fixtures.run_inserts_for(addresses)
-
-        session = create_session()
-        u = session.query(User).filter_by(id=7).one()
-
-        # get everything to load in both directions
-        print [a.user for a in u.addresses]
-
-        # then see if expunge fails
-        session.expunge(u)
-
-        assert sa.orm.object_session(u) is None
-        assert sa.orm.attributes.instance_state(u).session_id is None
-        for a in u.addresses:
-            assert sa.orm.object_session(a) is None
-            assert sa.orm.attributes.instance_state(a).session_id is None
 
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_mapped_binds(self):
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
+
 
         # ensure tables are unbound
         m2 = sa.MetaData()
@@ -159,8 +144,12 @@ class SessionTest(_fixtures.FixtureTest):
         sess.close()
 
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_table_binds(self):
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
+
 
         # ensure tables are unbound
         m2 = sa.MetaData()
@@ -196,8 +185,9 @@ class SessionTest(_fixtures.FixtureTest):
         sess.close()
 
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_bind_from_metadata(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
 
         session = create_session()
@@ -212,8 +202,9 @@ class SessionTest(_fixtures.FixtureTest):
 
     @testing.requires.independent_connections
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_transaction(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         conn1 = testing.db.connect()
         conn2 = testing.db.connect()
@@ -233,8 +224,9 @@ class SessionTest(_fixtures.FixtureTest):
 
     @testing.requires.independent_connections
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_autoflush(self):
+        User, users = self.classes.User, self.tables.users
+
         bind = self.metadata.bind
         mapper(User, users)
         conn1 = bind.connect()
@@ -253,8 +245,9 @@ class SessionTest(_fixtures.FixtureTest):
         eq_(bind.connect().execute("select count(1) from users").scalar(), 1)
         sess.close()
 
-    @testing.resolve_artifact_names
     def test_make_transient(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         sess = create_session()
         sess.add(User(name='test'))
@@ -272,7 +265,7 @@ class SessionTest(_fixtures.FixtureTest):
         sess.add(u1)
         assert u1 in sess.new
 
-        # test expired attributes 
+        # test expired attributes
         # get unexpired
         u1 = sess.query(User).first()
         sess.expire(u1)
@@ -299,9 +292,10 @@ class SessionTest(_fixtures.FixtureTest):
         sess.flush()
         assert u1 in sess
 
-    @testing.resolve_artifact_names
     def test_make_transient_plus_rollback(self):
         # test for [ticket:2182]
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         sess = Session()
         u1 = User(name='test')
@@ -313,8 +307,9 @@ class SessionTest(_fixtures.FixtureTest):
         make_transient(u1)
         sess.rollback()
 
-    @testing.resolve_artifact_names
     def test_deleted_flag(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
 
         sess = sessionmaker()()
@@ -341,13 +336,18 @@ class SessionTest(_fixtures.FixtureTest):
 
         eq_(sess.query(User).count(), 1)
 
-    @testing.resolve_artifact_names
     def test_autoflush_expressions(self):
-        """test that an expression which is dependent on object state is 
+        """test that an expression which is dependent on object state is
         evaluated after the session autoflushes.   This is the lambda
         inside of strategies.py lazy_clause.
 
         """
+
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, backref="user")})
         mapper(Address, addresses)
@@ -371,8 +371,9 @@ class SessionTest(_fixtures.FixtureTest):
 
     @testing.requires.independent_connections
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_autoflush_unbound(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         try:
             sess = create_session(autocommit=False, autoflush=True)
@@ -396,8 +397,9 @@ class SessionTest(_fixtures.FixtureTest):
             raise
 
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_autoflush_2(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         conn1 = testing.db.connect()
         conn2 = testing.db.connect()
@@ -412,33 +414,10 @@ class SessionTest(_fixtures.FixtureTest):
                 ).scalar() == 1
         sess.commit()
 
-    @testing.resolve_artifact_names
-    def test_autoflush_rollback(self):
-        mapper(Address, addresses)
-        mapper(User, users, properties={
-            'addresses':relationship(Address)})
 
-        _fixtures.run_inserts_for(users)
-        _fixtures.run_inserts_for(addresses)
-
-        sess = create_session(autocommit=False, autoflush=True)
-        u = sess.query(User).get(8)
-        newad = Address(email_address='a new address')
-        u.addresses.append(newad)
-        u.name = 'some new name'
-        assert u.name == 'some new name'
-        assert len(u.addresses) == 4
-        assert newad in u.addresses
-        sess.rollback()
-        assert u.name == 'ed'
-        assert len(u.addresses) == 3
-
-        assert newad not in u.addresses
-        # pending objects dont get expired
-        assert newad.email_address == 'a new address'
-
-    @testing.resolve_artifact_names
     def test_autocommit_doesnt_raise_on_pending(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         session = create_session(autocommit=True)
 
@@ -456,10 +435,12 @@ class SessionTest(_fixtures.FixtureTest):
         sess.rollback()
         assert not sess.is_active
 
-    @testing.resolve_artifact_names
     def test_textual_execute(self):
         """test that Session.execute() converts to text()"""
 
+        users = self.tables.users
+
+
         sess = create_session(bind=self.metadata.bind)
         users.insert().execute(id=7, name='jack')
 
@@ -475,8 +456,9 @@ class SessionTest(_fixtures.FixtureTest):
             7)
 
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_subtransaction_on_external(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         conn = testing.db.connect()
         trans = conn.begin()
@@ -492,8 +474,9 @@ class SessionTest(_fixtures.FixtureTest):
 
     @testing.requires.savepoints
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_external_nested_transaction(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         try:
             conn = testing.db.connect()
@@ -516,8 +499,9 @@ class SessionTest(_fixtures.FixtureTest):
             raise
 
     @testing.requires.savepoints
-    @testing.resolve_artifact_names
     def test_heavy_nesting(self):
+        users = self.tables.users
+
         session = create_session(bind=testing.db)
         session.begin()
         session.connection().execute(users.insert().values(name='user1'
@@ -537,9 +521,10 @@ class SessionTest(_fixtures.FixtureTest):
         assert session.connection().execute('select count(1) from users'
                 ).scalar() == 2
 
-    @testing.fails_on('sqlite', 'FIXME: unknown')
-    @testing.resolve_artifact_names
+    @testing.requires.independent_connections
     def test_transactions_isolated(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         users.delete().execute()
 
@@ -552,8 +537,12 @@ class SessionTest(_fixtures.FixtureTest):
         assert s2.query(User).all() == []
 
     @testing.requires.two_phase_transactions
-    @testing.resolve_artifact_names
     def test_twophase(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         # TODO: mock up a failure condition here
         # to ensure a rollback succeeds
         mapper(User, users)
@@ -573,8 +562,9 @@ class SessionTest(_fixtures.FixtureTest):
         assert users.count().scalar() == 1
         assert addresses.count().scalar() == 1
 
-    @testing.resolve_artifact_names
     def test_subtransaction_on_noautocommit(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         sess = create_session(autocommit=False, autoflush=True)
         sess.begin(subtransactions=True)
@@ -587,8 +577,9 @@ class SessionTest(_fixtures.FixtureTest):
         sess.close()
 
     @testing.requires.savepoints
-    @testing.resolve_artifact_names
     def test_nested_transaction(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         sess = create_session()
         sess.begin()
@@ -610,8 +601,9 @@ class SessionTest(_fixtures.FixtureTest):
         sess.close()
 
     @testing.requires.savepoints
-    @testing.resolve_artifact_names
     def test_nested_autotrans(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         sess = create_session(autocommit=False)
         u = User(name='u1')
@@ -631,8 +623,9 @@ class SessionTest(_fixtures.FixtureTest):
         sess.close()
 
     @testing.requires.savepoints
-    @testing.resolve_artifact_names
     def test_nested_transaction_connection_add(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
 
         sess = create_session(autocommit=True)
@@ -666,8 +659,9 @@ class SessionTest(_fixtures.FixtureTest):
         sess.close()
 
     @testing.requires.savepoints
-    @testing.resolve_artifact_names
     def test_mixed_transaction_control(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
 
         sess = create_session(autocommit=True)
@@ -697,8 +691,9 @@ class SessionTest(_fixtures.FixtureTest):
         sess.close()
 
     @testing.requires.savepoints
-    @testing.resolve_artifact_names
     def test_mixed_transaction_close(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
 
         sess = create_session(autocommit=False)
@@ -717,8 +712,9 @@ class SessionTest(_fixtures.FixtureTest):
 
         eq_(len(sess.query(User).all()), 1)
 
-    @testing.resolve_artifact_names
     def test_error_on_using_inactive_session(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         sess = create_session(autocommit=True)
         sess.begin()
@@ -734,8 +730,9 @@ class SessionTest(_fixtures.FixtureTest):
                               sess.begin, subtransactions=True)
         sess.close()
 
-    @testing.resolve_artifact_names
     def test_preserve_flush_error(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         sess = Session()
 
@@ -759,8 +756,9 @@ class SessionTest(_fixtures.FixtureTest):
         sess.commit()
 
 
-    @testing.resolve_artifact_names
     def test_no_autocommit_with_explicit_commit(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         session = create_session(autocommit=False)
         session.add(User(name='ed'))
@@ -769,8 +767,9 @@ class SessionTest(_fixtures.FixtureTest):
             'autocommit=False should start a new transaction'
 
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_bound_connection(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         c = testing.db.connect()
         sess = create_session(bind=c)
@@ -791,8 +790,9 @@ class SessionTest(_fixtures.FixtureTest):
         assert len(sess.query(User).all()) == 0
         sess.close()
 
-    @testing.resolve_artifact_names
     def test_bound_connection_transactional(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         c = testing.db.connect()
 
@@ -826,10 +826,43 @@ class SessionTest(_fixtures.FixtureTest):
         assert not c.in_transaction()
         assert c.scalar("select count(1) from users") == 1
 
+    def test_bind_arguments(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+        mapper(User, users)
+        mapper(Address, addresses)
+
+        e1 = engines.testing_engine()
+        e2 = engines.testing_engine()
+        e3 = engines.testing_engine()
+
+        sess = Session(e3)
+        sess.bind_mapper(User, e1)
+        sess.bind_mapper(Address, e2)
+
+        assert sess.connection().engine is e3
+        assert sess.connection(bind=e1).engine is e1
+        assert sess.connection(mapper=Address, bind=e1).engine is e1
+        assert sess.connection(mapper=Address).engine is e2
+        assert sess.connection(clause=addresses.select()).engine is e2
+        assert sess.connection(mapper=User,
+                                clause=addresses.select()).engine is e1
+        assert sess.connection(mapper=User,
+                                clause=addresses.select(),
+                                bind=e2).engine is e2
+
+        sess.close()
 
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_add_delete(self):
+        User, Address, addresses, users = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users)
+
 
         s = create_session()
         mapper(User, users, properties={
@@ -883,8 +916,12 @@ class SessionTest(_fixtures.FixtureTest):
         assert user not in s
         assert s.query(User).count() == 0
 
-    @testing.resolve_artifact_names
     def test_is_modified(self):
+        User, Address, addresses, users = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users)
+
         s = create_session()
 
         mapper(User, users, properties={'addresses':relationship(Address)})
@@ -915,8 +952,9 @@ class SessionTest(_fixtures.FixtureTest):
         assert s.is_modified(user)
         assert not s.is_modified(user, include_collections=False)
 
-    @testing.resolve_artifact_names
     def test_is_modified_syn(self):
+        User, users = self.classes.User, self.tables.users
+
         s = sessionmaker()()
 
         mapper(User, users, properties={'uname':sa.orm.synonym('name')})
@@ -926,11 +964,13 @@ class SessionTest(_fixtures.FixtureTest):
         s.commit()
         assert not s.is_modified(u)
 
-    @testing.resolve_artifact_names
     def test_weak_ref(self):
         """test the weak-referencing identity map, which strongly-
         references modified items."""
 
+        users, User = self.tables.users, self.classes.User
+
+
         s = create_session()
         mapper(User, users)
 
@@ -959,8 +999,9 @@ class SessionTest(_fixtures.FixtureTest):
         assert user.name == 'fred'
         assert s.identity_map
 
-    @testing.resolve_artifact_names
     def test_weak_ref_pickled(self):
+        users, User = self.tables.users, pickleable.User
+
         s = create_session()
         mapper(User, users)
 
@@ -989,8 +1030,10 @@ class SessionTest(_fixtures.FixtureTest):
 
         assert not s.identity_map
 
-    @testing.resolve_artifact_names
+    @testing.uses_deprecated()
     def test_identity_conflict(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         for s in (
             create_session(),
@@ -1009,8 +1052,12 @@ class SessionTest(_fixtures.FixtureTest):
                           sa.orm.attributes.instance_state(u2))
 
 
-    @testing.resolve_artifact_names
     def test_weakref_with_cycles_o2m(self):
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
+
         s = sessionmaker()()
         mapper(User, users, properties={
             "addresses":relationship(Address, backref="user")
@@ -1038,8 +1085,12 @@ class SessionTest(_fixtures.FixtureTest):
         user = s.query(User).options(joinedload(User.addresses)).one()
         eq_(user, User(name="ed", addresses=[Address(email_address="ed2")]))
 
-    @testing.resolve_artifact_names
     def test_weakref_with_cycles_o2o(self):
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
+
         s = sessionmaker()()
         mapper(User, users, properties={
             "address":relationship(Address, backref="user", uselist=False)
@@ -1068,8 +1119,10 @@ class SessionTest(_fixtures.FixtureTest):
         user = s.query(User).options(joinedload(User.address)).one()
         eq_(user, User(name="ed", address=Address(email_address="ed2")))
 
-    @testing.resolve_artifact_names
+    @testing.uses_deprecated()
     def test_strong_ref(self):
+        users, User = self.tables.users, self.classes.User
+
         s = create_session(weak_identity_map=False)
         mapper(User, users)
 
@@ -1089,9 +1142,12 @@ class SessionTest(_fixtures.FixtureTest):
         s.flush()
         eq_(users.select().execute().fetchall(), [(user.id, 'u2')])
 
+    @testing.uses_deprecated()
+    @testing.fails_if(lambda: pypy, "pypy has a real GC")
     @testing.fails_on('+zxjdbc', 'http://www.sqlalchemy.org/trac/ticket/1473')
-    @testing.resolve_artifact_names
     def test_prune(self):
+        users, User = self.tables.users, self.classes.User
+
         s = create_session(weak_identity_map=False)
         mapper(User, users)
 
@@ -1139,223 +1195,10 @@ class SessionTest(_fixtures.FixtureTest):
         self.assert_(s.prune() == 0)
         self.assert_(len(s.identity_map) == 0)
 
-    @testing.resolve_artifact_names
-    def test_no_save_cascade_1(self):
-        mapper(Address, addresses)
-        mapper(User, users, properties=dict(
-            addresses=relationship(Address, cascade="none", backref="user")))
-        s = create_session()
 
-        u = User(name='u1')
-        s.add(u)
-        a = Address(email_address='u1@e')
-        u.addresses.append(a)
-        assert u in s
-        assert a not in s
-        s.flush()
-        print "\n".join([repr(x.__dict__) for x in s])
-        s.expunge_all()
-        assert s.query(User).one().id == u.id
-        assert s.query(Address).first() is None
-
-    @testing.resolve_artifact_names
-    def test_no_save_cascade_2(self):
-        mapper(Address, addresses)
-        mapper(User, users, properties=dict(
-            addresses=relationship(Address,
-                               cascade="all",
-                               backref=backref("user", cascade="none"))))
-
-        s = create_session()
-        u = User(name='u1')
-        a = Address(email_address='u1@e')
-        a.user = u
-        s.add(a)
-        assert u not in s
-        assert a in s
-        s.flush()
-        s.expunge_all()
-        assert s.query(Address).one().id == a.id
-        assert s.query(User).first() is None
-
-    @testing.resolve_artifact_names
-    def test_extension(self):
-        mapper(User, users)
-        log = []
-        class MyExt(sa.orm.session.SessionExtension):
-            def before_commit(self, session):
-                log.append('before_commit')
-            def after_commit(self, session):
-                log.append('after_commit')
-            def after_rollback(self, session):
-                log.append('after_rollback')
-            def before_flush(self, session, flush_context, objects):
-                log.append('before_flush')
-            def after_flush(self, session, flush_context):
-                log.append('after_flush')
-            def after_flush_postexec(self, session, flush_context):
-                log.append('after_flush_postexec')
-            def after_begin(self, session, transaction, connection):
-                log.append('after_begin')
-            def after_attach(self, session, instance):
-                log.append('after_attach')
-            def after_bulk_update(
-                self,
-                session,
-                query,
-                query_context,
-                result,
-                ):
-                log.append('after_bulk_update')
-
-            def after_bulk_delete(
-                self,
-                session,
-                query,
-                query_context,
-                result,
-                ):
-                log.append('after_bulk_delete')
-
-        sess = create_session(extension = MyExt())
-        u = User(name='u1')
-        sess.add(u)
-        sess.flush()
-        assert log == [
-            'after_attach',
-            'before_flush',
-            'after_begin',
-            'after_flush',
-            'before_commit',
-            'after_commit',
-            'after_flush_postexec',
-            ]
-        log = []
-        sess = create_session(autocommit=False, extension=MyExt())
-        u = User(name='u1')
-        sess.add(u)
-        sess.flush()
-        assert log == ['after_attach', 'before_flush', 'after_begin',
-                       'after_flush', 'after_flush_postexec']
-        log = []
-        u.name = 'ed'
-        sess.commit()
-        assert log == ['before_commit', 'before_flush', 'after_flush',
-                       'after_flush_postexec', 'after_commit']
-        log = []
-        sess.commit()
-        assert log == ['before_commit', 'after_commit']
-        log = []
-        sess.query(User).delete()
-        assert log == ['after_begin', 'after_bulk_delete']
-        log = []
-        sess.query(User).update({'name': 'foo'})
-        assert log == ['after_bulk_update']
-        log = []
-        sess = create_session(autocommit=False, extension=MyExt(),
-                              bind=testing.db)
-        conn = sess.connection()
-        assert log == ['after_begin']
-
-    @testing.resolve_artifact_names
-    def test_before_flush(self):
-        """test that the flush plan can be affected during before_flush()"""
-
-        mapper(User, users)
-
-        class MyExt(sa.orm.session.SessionExtension):
-            def before_flush(self, session, flush_context, objects):
-                for obj in list(session.new) + list(session.dirty):
-                    if isinstance(obj, User):
-                        session.add(User(name='another %s' % obj.name))
-                for obj in list(session.deleted):
-                    if isinstance(obj, User):
-                        x = session.query(User).filter(User.name
-                                == 'another %s' % obj.name).one()
-                        session.delete(x)
-
-        sess = create_session(extension = MyExt(), autoflush=True)
-        u = User(name='u1')
-        sess.add(u)
-        sess.flush()
-        eq_(sess.query(User).order_by(User.name).all(), 
-            [
-                User(name='another u1'),
-                User(name='u1')
-            ]
-        )
-
-        sess.flush()
-        eq_(sess.query(User).order_by(User.name).all(), 
-            [
-                User(name='another u1'),
-                User(name='u1')
-            ]
-        )
-
-        u.name='u2'
-        sess.flush()
-        eq_(sess.query(User).order_by(User.name).all(), 
-            [
-                User(name='another u1'),
-                User(name='another u2'),
-                User(name='u2')
-            ]
-        )
-
-        sess.delete(u)
-        sess.flush()
-        eq_(sess.query(User).order_by(User.name).all(), 
-            [
-                User(name='another u1'),
-            ]
-        )
-
-    @testing.resolve_artifact_names
-    def test_before_flush_affects_dirty(self):
-        mapper(User, users)
-
-        class MyExt(sa.orm.session.SessionExtension):
-            def before_flush(self, session, flush_context, objects):
-                for obj in list(session.identity_map.values()):
-                    obj.name += " modified"
-
-        sess = create_session(extension = MyExt(), autoflush=True)
-        u = User(name='u1')
-        sess.add(u)
-        sess.flush()
-        eq_(sess.query(User).order_by(User.name).all(), 
-            [
-                User(name='u1')
-            ]
-        )
-
-        sess.add(User(name='u2'))
-        sess.flush()
-        sess.expunge_all()
-        eq_(sess.query(User).order_by(User.name).all(), 
-            [
-                User(name='u1 modified'),
-                User(name='u2')
-            ]
-        )
-
-    @testing.resolve_artifact_names
-    def test_reentrant_flush(self):
-
-        mapper(User, users)
-
-        class MyExt(sa.orm.session.SessionExtension):
-            def before_flush(s, session, flush_context, objects):
-                session.flush()
-
-        sess = create_session(extension=MyExt())
-        sess.add(User(name='foo'))
-        assert_raises_message(sa.exc.InvalidRequestError,
-                              'already flushing', sess.flush)
-
-    @testing.resolve_artifact_names
     def test_pickled_update(self):
+        users, User = self.tables.users, pickleable.User
+
         mapper(User, users)
         sess1 = create_session()
         sess2 = create_session()
@@ -1367,8 +1210,9 @@ class SessionTest(_fixtures.FixtureTest):
         u2 = pickle.loads(pickle.dumps(u1))
         sess2.add(u2)
 
-    @testing.resolve_artifact_names
     def test_duplicate_update(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         Session = sessionmaker()
         sess = Session()
@@ -1407,8 +1251,9 @@ class SessionTest(_fixtures.FixtureTest):
         u3 = sess.query(User).get(u1.id)
         assert u3 is not u1 and u3 is not u2 and u3.name == u1.name
 
-    @testing.resolve_artifact_names
     def test_no_double_save(self):
+        users = self.tables.users
+
         sess = create_session()
         class Foo(object):
             def __init__(self):
@@ -1424,8 +1269,9 @@ class SessionTest(_fixtures.FixtureTest):
         assert b in sess
         assert len(list(sess)) == 1
 
-    @testing.resolve_artifact_names
     def test_identity_map_mutate(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
 
         sess = Session()
@@ -1439,8 +1285,62 @@ class SessionTest(_fixtures.FixtureTest):
                 del u3
                 gc_collect()
 
+class SessionDataTest(_fixtures.FixtureTest):
+    def test_expunge_cascade(self):
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
 
-class DisposedStates(_base.MappedTest):
+        mapper(Address, addresses)
+        mapper(User, users, properties={
+            'addresses':relationship(Address,
+                                 backref=backref("user", cascade="all"),
+                                 cascade="all")})
+
+        session = create_session()
+        u = session.query(User).filter_by(id=7).one()
+
+        # get everything to load in both directions
+        print [a.user for a in u.addresses]
+
+        # then see if expunge fails
+        session.expunge(u)
+
+        assert sa.orm.object_session(u) is None
+        assert sa.orm.attributes.instance_state(u).session_id is None
+        for a in u.addresses:
+            assert sa.orm.object_session(a) is None
+            assert sa.orm.attributes.instance_state(a).session_id is None
+
+    def test_autoflush_rollback(self):
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
+
+        mapper(Address, addresses)
+        mapper(User, users, properties={
+            'addresses':relationship(Address)})
+
+        sess = create_session(autocommit=False, autoflush=True)
+        u = sess.query(User).get(8)
+        newad = Address(email_address='a new address')
+        u.addresses.append(newad)
+        u.name = 'some new name'
+        assert u.name == 'some new name'
+        assert len(u.addresses) == 4
+        assert newad in u.addresses
+        sess.rollback()
+        assert u.name == 'ed'
+        assert len(u.addresses) == 3
+
+        assert newad not in u.addresses
+        # pending objects dont get expired
+        assert newad.email_address == 'a new address'
+
+
+class DisposedStates(fixtures.MappedTest):
     run_setup_mappers = 'once'
     run_inserts = 'once'
     run_deletes = None
@@ -1484,7 +1384,7 @@ class DisposedStates(_base.MappedTest):
         sess.identity_map.all_states = lambda : all_states
         for obj in objs:
             state = attributes.instance_state(obj)
-            sess.identity_map.remove(state)
+            sess.identity_map.discard(state)
             state.dispose()
 
     def _test_session(self, **kwargs):
@@ -1526,7 +1426,7 @@ class DisposedStates(_base.MappedTest):
         sess.rollback()
 
 
-class SessionInterface(testing.TestBase):
+class SessionInterface(fixtures.TestBase):
     """Bogus args to Session methods produce actionable exceptions."""
 
     # TODO: expand with message body assertions.
@@ -1658,9 +1558,11 @@ class SessionInterface(testing.TestBase):
         self._test_class_guards(early)
 
 
-class TLTransactionTest(engine_base.AltEngineTest, _base.MappedTest):
+class TLTransactionTest(fixtures.MappedTest):
+    run_dispose_bind = 'once'
+
     @classmethod
-    def create_engine(cls):
+    def setup_bind(cls):
         return engines.testing_engine(options=dict(strategy='threadlocal'))
 
     @classmethod
@@ -1671,22 +1573,23 @@ class TLTransactionTest(engine_base.AltEngineTest, _base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class User(_base.BasicEntity):
+        class User(cls.Basic):
             pass
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
+        users, User = cls.tables.users, cls.classes.User
+
         mapper(User, users)
 
     @testing.exclude('mysql', '<', (5, 0, 3), 'FIXME: unknown')
-    @testing.resolve_artifact_names
     def test_session_nesting(self):
-        sess = create_session(bind=self.engine)
-        self.engine.begin()
+        User = self.classes.User
+
+        sess = create_session(bind=self.bind)
+        self.bind.begin()
         u = User(name='ed')
         sess.add(u)
         sess.flush()
-        self.engine.commit()
-
+        self.bind.commit()
 
