@@ -1,18 +1,14 @@
-from sqlalchemy.test.testing import eq_, assert_raises, assert_raises_message
-from sqlalchemy.test import testing
-from sqlalchemy.test.schema import Table, Column
+from test.lib.testing import eq_, assert_raises, assert_raises_message
+from test.lib import testing
+from test.lib.schema import Table, Column
+from test.orm import _fixtures
+from test.lib import fixtures
 from sqlalchemy import Integer, String, ForeignKey, func
-from test.orm import _fixtures, _base
 from sqlalchemy.orm import mapper, relationship, backref, \
                             create_session, unitofwork, attributes,\
-                            Session
-from sqlalchemy.test.assertsql import AllOf, CompiledSQL
+                            Session, class_mapper, sync, exc as orm_exc
 
-from test.orm._fixtures import keywords, addresses, Base, Keyword,  \
-           Dingaling, item_keywords, dingalings, User, items,\
-           orders, Address, users, nodes, \
-            order_items, Item, Order, Node, \
-            composite_pk_table, CompositePk
+from test.lib.assertsql import AllOf, CompiledSQL
 
 class AssertsUOW(object):
     def _get_test_uow(self, session):
@@ -26,10 +22,7 @@ class AssertsUOW(object):
             uow.register_object(d, isdelete=True)
         return uow
 
-    def _assert_uow_size(self,
-        session, 
-        expected
-    ):
+    def _assert_uow_size(self, session, expected ):
         uow = self._get_test_uow(session)
         postsort_actions = uow._generate_actions()
         print postsort_actions
@@ -42,6 +35,11 @@ class UOWTest(_fixtures.FixtureTest,
 class RudimentaryFlushTest(UOWTest):
 
     def test_one_to_many_save(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address),
         })
@@ -72,6 +70,11 @@ class RudimentaryFlushTest(UOWTest):
             )
 
     def test_one_to_many_delete_all(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address),
         })
@@ -99,6 +102,11 @@ class RudimentaryFlushTest(UOWTest):
         )
 
     def test_one_to_many_delete_parent(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address),
         })
@@ -130,6 +138,11 @@ class RudimentaryFlushTest(UOWTest):
         )
 
     def test_many_to_one_save(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
 
         mapper(User, users)
         mapper(Address, addresses, properties={
@@ -162,6 +175,11 @@ class RudimentaryFlushTest(UOWTest):
             )
 
     def test_many_to_one_delete_all(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users)
         mapper(Address, addresses, properties={
             'user':relationship(User)
@@ -191,6 +209,11 @@ class RudimentaryFlushTest(UOWTest):
         )
 
     def test_many_to_one_delete_target(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users)
         mapper(Address, addresses, properties={
             'user':relationship(User)
@@ -225,6 +248,11 @@ class RudimentaryFlushTest(UOWTest):
         )
 
     def test_many_to_one_delete_unloaded(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users)
         mapper(Address, addresses, properties={
             'parent':relationship(User)
@@ -259,9 +287,11 @@ class RudimentaryFlushTest(UOWTest):
             testing.db,
             session.flush,
             AllOf(
-                # ensure all three m2os are loaded.
+                # [ticket:2002] - ensure the m2os are loaded.
                 # the selects here are in fact unexpiring
                 # each row - the m2o comes from the identity map.
+                # the User row might be handled before or the addresses
+                # are loaded so need to use AllOf
                 CompiledSQL(
                     "SELECT addresses.id AS addresses_id, addresses.user_id AS "
                     "addresses_user_id, addresses.email_address AS "
@@ -281,18 +311,220 @@ class RudimentaryFlushTest(UOWTest):
                     "FROM users WHERE users.id = :param_1",
                     lambda ctx: {'param_1': pid}
                 ),
+                CompiledSQL(
+                    "DELETE FROM addresses WHERE addresses.id = :id",
+                    lambda ctx: [{'id': c1id}, {'id': c2id}]
+                ),
+                CompiledSQL(
+                    "DELETE FROM users WHERE users.id = :id",
+                    lambda ctx: {'id': pid}
+                ),
+            ),
+        )
+
+    def test_many_to_one_delete_childonly_unloaded(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+        mapper(User, users)
+        mapper(Address, addresses, properties={
+            'parent':relationship(User)
+        })
+
+        parent = User(name='p1')
+        c1, c2 = Address(email_address='c1', parent=parent), \
+                    Address(email_address='c2', parent=parent)
+
+        session = Session()
+        session.add_all([c1, c2])
+        session.add(parent)
+
+        session.flush()
+
+        pid = parent.id
+        c1id = c1.id
+        c2id = c2.id
+
+        session.expire(c1)
+        session.expire(c2)
+
+        session.delete(c1)
+        session.delete(c2)
+
+        self.assert_sql_execution(
+            testing.db,
+            session.flush,
+            AllOf(
+                # [ticket:2049] - we aren't deleting User,
+                # relationship is simple m2o, no SELECT should be emitted for it.
+                CompiledSQL(
+                    "SELECT addresses.id AS addresses_id, addresses.user_id AS "
+                    "addresses_user_id, addresses.email_address AS "
+                    "addresses_email_address FROM addresses WHERE addresses.id = "
+                    ":param_1",
+                    lambda ctx: {'param_1': c1id}
+                ),
+                CompiledSQL(
+                    "SELECT addresses.id AS addresses_id, addresses.user_id AS "
+                    "addresses_user_id, addresses.email_address AS "
+                    "addresses_email_address FROM addresses WHERE addresses.id = "
+                    ":param_1",
+                    lambda ctx: {'param_1': c2id}
+                ),
             ),
             CompiledSQL(
                 "DELETE FROM addresses WHERE addresses.id = :id",
                 lambda ctx: [{'id': c1id}, {'id': c2id}]
             ),
+        )
+
+    def test_many_to_one_delete_childonly_unloaded_expired(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+        mapper(User, users)
+        mapper(Address, addresses, properties={
+            'parent':relationship(User)
+        })
+
+        parent = User(name='p1')
+        c1, c2 = Address(email_address='c1', parent=parent), \
+                    Address(email_address='c2', parent=parent)
+
+        session = Session()
+        session.add_all([c1, c2])
+        session.add(parent)
+
+        session.flush()
+
+        pid = parent.id
+        c1id = c1.id
+        c2id = c2.id
+
+        session.expire(parent)
+        session.expire(c1)
+        session.expire(c2)
+
+        session.delete(c1)
+        session.delete(c2)
+
+        self.assert_sql_execution(
+            testing.db,
+            session.flush,
+            AllOf(
+                # the parent User is expired, so it gets loaded here.
+                CompiledSQL(
+                    "SELECT addresses.id AS addresses_id, addresses.user_id AS "
+                    "addresses_user_id, addresses.email_address AS "
+                    "addresses_email_address FROM addresses WHERE addresses.id = "
+                    ":param_1",
+                    lambda ctx: {'param_1': c1id}
+                ),
+                CompiledSQL(
+                    "SELECT addresses.id AS addresses_id, addresses.user_id AS "
+                    "addresses_user_id, addresses.email_address AS "
+                    "addresses_email_address FROM addresses WHERE addresses.id = "
+                    ":param_1",
+                    lambda ctx: {'param_1': c2id}
+                ),
+            ),
             CompiledSQL(
-                "DELETE FROM users WHERE users.id = :id",
-                lambda ctx: {'id': pid}
+                "DELETE FROM addresses WHERE addresses.id = :id",
+                lambda ctx: [{'id': c1id}, {'id': c2id}]
             ),
         )
 
+    def test_natural_ordering(self):
+        """test that unconnected items take relationship() into account regardless."""
+
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+
+        mapper(User, users)
+        mapper(Address, addresses, properties={
+            'parent':relationship(User)
+        })
+
+        sess = create_session()
+
+        u1 = User(id=1, name='u1')
+        a1 = Address(id=1, user_id=1, email_address='a2')
+
+        sess.add_all([u1, a1])
+        self.assert_sql_execution(
+            testing.db,
+            sess.flush,
+            CompiledSQL(
+                "INSERT INTO users (id, name) VALUES (:id, :name)", 
+                {'id':1, 'name':'u1'}),
+            CompiledSQL(
+                "INSERT INTO addresses (id, user_id, email_address) "
+                "VALUES (:id, :user_id, :email_address)",
+                {'email_address': 'a2', 'user_id': 1, 'id': 1}
+            )
+        )
+
+        sess.delete(u1)
+        sess.delete(a1)
+        self.assert_sql_execution(
+            testing.db,
+            sess.flush,
+            CompiledSQL(
+                "DELETE FROM addresses WHERE addresses.id = :id",
+                [{'id': 1}]
+            ),
+            CompiledSQL(
+                "DELETE FROM users WHERE users.id = :id",
+                [{'id': 1}]
+            )
+        )
+
+    def test_natural_selfref(self):
+        """test that unconnected items take relationship() into account regardless."""
+
+        Node, nodes = self.classes.Node, self.tables.nodes
+
+
+        mapper(Node, nodes, properties={
+            'children':relationship(Node)
+        })
+
+        sess = create_session()
+
+        n1 = Node(id=1)
+        n2 = Node(id=2, parent_id=1)
+        n3 = Node(id=3, parent_id=2)
+
+        # insert order is determined from add order since they
+        # are the same class
+        sess.add_all([n1, n2, n3])
+
+        self.assert_sql_execution(
+            testing.db,
+            sess.flush,
+            CompiledSQL(
+                "INSERT INTO nodes (id, parent_id, data) VALUES "
+                "(:id, :parent_id, :data)", 
+                [{'parent_id': None, 'data': None, 'id': 1}, 
+                {'parent_id': 1, 'data': None, 'id': 2}, 
+                {'parent_id': 2, 'data': None, 'id': 3}]
+                ),
+        )
+
     def test_many_to_many(self):
+        keywords, items, item_keywords, Keyword, Item = (self.tables.keywords,
+                                self.tables.items,
+                                self.tables.item_keywords,
+                                self.classes.Keyword,
+                                self.classes.Item)
+
         mapper(Item, items, properties={
             'keywords':relationship(Keyword, secondary=item_keywords)
         })
@@ -334,6 +566,11 @@ class RudimentaryFlushTest(UOWTest):
         )
 
     def test_m2o_flush_size(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users)
         mapper(Address, addresses, properties={
             'user':relationship(User, passive_updates=True)
@@ -344,6 +581,11 @@ class RudimentaryFlushTest(UOWTest):
         self._assert_uow_size(sess, 2)
 
     def test_o2m_flush_size(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address),
         })
@@ -382,6 +624,8 @@ class RudimentaryFlushTest(UOWTest):
 
 class SingleCycleTest(UOWTest):
     def test_one_to_many_save(self):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
         mapper(Node, nodes, properties={
             'children':relationship(Node)
         })
@@ -416,6 +660,8 @@ class SingleCycleTest(UOWTest):
             )
 
     def test_one_to_many_delete_all(self):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
         mapper(Node, nodes, properties={
             'children':relationship(Node)
         })
@@ -440,6 +686,8 @@ class SingleCycleTest(UOWTest):
         )
 
     def test_one_to_many_delete_parent(self):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
         mapper(Node, nodes, properties={
             'children':relationship(Node)
         })
@@ -468,6 +716,8 @@ class SingleCycleTest(UOWTest):
         )
 
     def test_many_to_one_save(self):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
         mapper(Node, nodes, properties={
             'parent':relationship(Node, remote_side=nodes.c.id)
         })
@@ -502,6 +752,8 @@ class SingleCycleTest(UOWTest):
             )
 
     def test_many_to_one_delete_all(self):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
         mapper(Node, nodes, properties={
             'parent':relationship(Node, remote_side=nodes.c.id)
         })
@@ -526,6 +778,8 @@ class SingleCycleTest(UOWTest):
         )
 
     def test_cycle_rowswitch(self):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
         mapper(Node, nodes, properties={
             'children':relationship(Node)
         })
@@ -542,6 +796,8 @@ class SingleCycleTest(UOWTest):
         sess.flush()
 
     def test_bidirectional_mutations_one(self):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
         mapper(Node, nodes, properties={
             'children':relationship(Node, 
                                     backref=backref('parent',
@@ -562,6 +818,8 @@ class SingleCycleTest(UOWTest):
         sess.flush()
 
     def test_bidirectional_multilevel_save(self):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
         mapper(Node, nodes, properties={
             'children':relationship(Node, 
                 backref=backref('parent', remote_side=nodes.c.id)
@@ -618,6 +876,8 @@ class SingleCycleTest(UOWTest):
         )
 
     def test_singlecycle_flush_size(self):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
         mapper(Node, nodes, properties={
             'children':relationship(Node)
         })
@@ -652,6 +912,8 @@ class SingleCycleTest(UOWTest):
         self._assert_uow_size(sess, 2)
 
     def test_delete_unloaded_m2o(self):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
         mapper(Node, nodes, properties={
             'parent':relationship(Node, remote_side=nodes.c.id)
         })
@@ -708,20 +970,20 @@ class SingleCycleTest(UOWTest):
                     "WHERE nodes.id = :param_1",
                     lambda ctx: {'param_1': c2id}
                 ),
-            ),
-            CompiledSQL(
-                "DELETE FROM nodes WHERE nodes.id = :id",
-                lambda ctx: [{'id': c1id}, {'id': c2id}]
-            ),
-            CompiledSQL(
-                "DELETE FROM nodes WHERE nodes.id = :id",
-                lambda ctx: {'id': pid}
+                CompiledSQL(
+                    "DELETE FROM nodes WHERE nodes.id = :id",
+                    lambda ctx: [{'id': c1id}, {'id': c2id}]
+                ),
+                CompiledSQL(
+                    "DELETE FROM nodes WHERE nodes.id = :id",
+                    lambda ctx: {'id': pid}
+                ),
             ),
         )
 
 
 
-class SingleCyclePlusAttributeTest(_base.MappedTest,
+class SingleCyclePlusAttributeTest(fixtures.MappedTest,
                     testing.AssertsExecutionResults, AssertsUOW):
     @classmethod
     def define_tables(cls, metadata):
@@ -738,11 +1000,12 @@ class SingleCyclePlusAttributeTest(_base.MappedTest,
             Column('parent_id', Integer, ForeignKey('nodes.id')),
         )
 
-    @testing.resolve_artifact_names
     def test_flush_size(self):
-        class Node(Base):
+        foobars, nodes = self.tables.foobars, self.tables.nodes
+
+        class Node(fixtures.ComparableEntity):
             pass
-        class FooBar(Base):
+        class FooBar(fixtures.ComparableEntity):
             pass
 
         mapper(Node, nodes, properties={
@@ -767,7 +1030,7 @@ class SingleCyclePlusAttributeTest(_base.MappedTest,
 
         sess.flush()
 
-class SingleCycleM2MTest(_base.MappedTest, 
+class SingleCycleM2MTest(fixtures.MappedTest, 
                     testing.AssertsExecutionResults, AssertsUOW):
 
     @classmethod
@@ -787,9 +1050,10 @@ class SingleCycleM2MTest(_base.MappedTest,
                             ForeignKey('nodes.id'),primary_key=True),
             )
 
-    @testing.resolve_artifact_names
     def test_many_to_many_one(self):
-        class Node(Base):
+        nodes, node_to_nodes = self.tables.nodes, self.tables.node_to_nodes
+
+        class Node(fixtures.ComparableEntity):
             pass
 
         mapper(Node, nodes, properties={
@@ -898,7 +1162,7 @@ class SingleCycleM2MTest(_base.MappedTest,
             ),
         )
 
-class RowswitchAccountingTest(_base.MappedTest):
+class RowswitchAccountingTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('parent', metadata,
@@ -908,8 +1172,9 @@ class RowswitchAccountingTest(_base.MappedTest):
             Column('id', Integer, ForeignKey('parent.id'), primary_key=True)
         )
 
-    @testing.resolve_artifact_names
     def test_accounting_for_rowswitch(self):
+        parent, child = self.tables.parent, self.tables.child
+
         class Parent(object):
             def __init__(self, id):
                 self.id = id
@@ -950,5 +1215,74 @@ class RowswitchAccountingTest(_base.MappedTest):
 
         sess.flush()
 
+class BatchInsertsTest(fixtures.MappedTest, testing.AssertsExecutionResults):
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('t', metadata,
+            Column('id', Integer, primary_key=True, 
+                        test_needs_autoincrement=True),
+            Column('data', String(50)),
+            Column('def_', String(50), server_default='def1')
+        )
 
+    def test_batch_interaction(self):
+        """test batching groups same-structured, primary 
+        key present statements together.
+
+        """
+
+        t = self.tables.t
+
+        class T(fixtures.ComparableEntity):
+            pass
+        mapper(T, t)
+        sess = Session()
+        sess.add_all([
+            T(data='t1'),
+            T(data='t2'),
+            T(id=3, data='t3'),
+            T(id=4, data='t4'),
+            T(id=5, data='t5'),
+            T(id=6, data=func.lower('t6')),
+            T(id=7, data='t7'),
+            T(id=8, data='t8'),
+            T(id=9, data='t9', def_='def2'),
+            T(id=10, data='t10', def_='def3'),
+            T(id=11, data='t11'),
+        ])
+        self.assert_sql_execution(
+            testing.db,
+            sess.flush,
+            CompiledSQL(
+                "INSERT INTO t (data) VALUES (:data)",
+                {'data': 't1'}
+            ),
+            CompiledSQL(
+                "INSERT INTO t (data) VALUES (:data)",
+                {'data': 't2'}
+            ),
+            CompiledSQL(
+                "INSERT INTO t (id, data) VALUES (:id, :data)",
+                [{'data': 't3', 'id': 3}, 
+                    {'data': 't4', 'id': 4}, 
+                    {'data': 't5', 'id': 5}]
+            ),
+            CompiledSQL(
+                "INSERT INTO t (id, data) VALUES (:id, lower(:lower_1))",
+                {'lower_1': 't6', 'id': 6}
+            ),
+            CompiledSQL(
+                "INSERT INTO t (id, data) VALUES (:id, :data)",
+                [{'data': 't7', 'id': 7}, {'data': 't8', 'id': 8}]
+            ),
+            CompiledSQL(
+                "INSERT INTO t (id, data, def_) VALUES (:id, :data, :def_)",
+                [{'data': 't9', 'id': 9, 'def_':'def2'}, 
+                {'data': 't10', 'id': 10, 'def_':'def3'}]
+            ),
+            CompiledSQL(
+                "INSERT INTO t (id, data) VALUES (:id, :data)",
+                {'data': 't11', 'id': 11}
+            ),
+        )
 
