@@ -1,33 +1,34 @@
-from sqlalchemy.test.testing import eq_
+from test.lib.testing import eq_
 import datetime
 from sqlalchemy import *
 from sqlalchemy.sql import table, column
 from sqlalchemy import databases, sql, util
 from sqlalchemy.sql.compiler import BIND_TEMPLATES
 from sqlalchemy.engine import default
-from sqlalchemy.test.engines import all_dialects
+from test.lib.engines import all_dialects
 from sqlalchemy import types as sqltypes
-from sqlalchemy.test import *
+from test.lib import *
 from sqlalchemy.sql.functions import GenericFunction
-from sqlalchemy.test.testing import eq_
-from decimal import Decimal as _python_Decimal
-from sqlalchemy.test import testing
+from test.lib.testing import eq_
+from sqlalchemy.util.compat import decimal
+from test.lib import testing
 from sqlalchemy.databases import *
 
 
-class CompileTest(TestBase, AssertsCompiledSQL):
+class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
+    __dialect__ = 'default'
 
     def test_compile(self):
         for dialect in all_dialects(exclude=('sybase', 'access', 'informix', 'maxdb')):
             bindtemplate = BIND_TEMPLATES[dialect.paramstyle]
             self.assert_compile(func.current_timestamp(), "CURRENT_TIMESTAMP", dialect=dialect)
             self.assert_compile(func.localtime(), "LOCALTIME", dialect=dialect)
-            if isinstance(dialect, (firebird.dialect, maxdb.dialect, oracle.dialect)):
+            if isinstance(dialect, (firebird.dialect, maxdb.dialect)):
                 self.assert_compile(func.nosuchfunction(), "nosuchfunction", dialect=dialect)
             else:
                 self.assert_compile(func.nosuchfunction(), "nosuchfunction()", dialect=dialect)
-            
-            # test generic function compile    
+
+            # test generic function compile
             class fake_func(GenericFunction):
                 __return_type__ = sqltypes.Integer
 
@@ -39,14 +40,14 @@ class CompileTest(TestBase, AssertsCompiledSQL):
                             "fake_func(%s)" % 
                             bindtemplate % {'name':'param_1', 'position':1}, 
                             dialect=dialect)
-            
+
     def test_use_labels(self):
         self.assert_compile(select([func.foo()], use_labels=True), 
             "SELECT foo() AS foo_1"
         )
     def test_underscores(self):
         self.assert_compile(func.if_(), "if()")
-        
+
     def test_generic_now(self):
         assert isinstance(func.now().type, sqltypes.DateTime)
 
@@ -66,13 +67,13 @@ class CompileTest(TestBase, AssertsCompiledSQL):
             ('random()', sqlite.dialect()),
             ('random()', postgresql.dialect()),
             ('rand()', mysql.dialect()),
-            ('random', oracle.dialect())
+            ('random()', oracle.dialect())
         ]:
             self.assert_compile(func.random(), ret, dialect=dialect)
-    
+
     def test_namespacing_conflicts(self):
         self.assert_compile(func.text('foo'), 'text(:text_1)')
-        
+
     def test_generic_count(self):
         assert isinstance(func.count().type, sqltypes.Integer)
 
@@ -101,19 +102,19 @@ class CompileTest(TestBase, AssertsCompiledSQL):
             assert True
 
     def test_return_type_detection(self):
-        
+
         for fn in [func.coalesce, func.max, func.min, func.sum]:
             for args, type_ in [
                             ((datetime.date(2007, 10, 5), 
                                 datetime.date(2005, 10, 15)), sqltypes.Date),
                             ((3, 5), sqltypes.Integer),
-                            ((_python_Decimal(3), _python_Decimal(5)), sqltypes.Numeric),
+                            ((decimal.Decimal(3), decimal.Decimal(5)), sqltypes.Numeric),
                             (("foo", "bar"), sqltypes.String),
                             ((datetime.datetime(2007, 10, 5, 8, 3, 34), 
                                 datetime.datetime(2005, 10, 15, 14, 45, 33)), sqltypes.DateTime)
                         ]:
                 assert isinstance(fn(*args).type, type_), "%s / %s" % (fn(), type_)
-        
+
         assert isinstance(func.concat("foo", "bar").type, sqltypes.String)
 
 
@@ -189,11 +190,12 @@ class CompileTest(TestBase, AssertsCompiledSQL):
             , checkparams={'y_1': 45, 'x_1': 17, 'y_2': 12, 'x_2': 5})
 
 
-class ExecuteTest(TestBase):
+class ExecuteTest(fixtures.TestBase):
     @engines.close_first
     def tearDown(self):
         pass
-        
+
+    @testing.uses_deprecated
     def test_standalone_execute(self):
         x = testing.db.func.current_date().execute().scalar()
         y = testing.db.func.current_date().select().execute().scalar()
@@ -208,10 +210,10 @@ class ExecuteTest(TestBase):
     def test_conn_execute(self):
         from sqlalchemy.sql.expression import FunctionElement
         from sqlalchemy.ext.compiler import compiles
-        
+
         class myfunc(FunctionElement):
             type = Date()
-        
+
         @compiles(myfunc)
         def compile(elem, compiler, **kw):
             return compiler.process(func.current_date())
@@ -229,17 +231,17 @@ class ExecuteTest(TestBase):
     def test_exec_options(self):
         f = func.foo()
         eq_(f._execution_options, {})
-        
+
         f = f.execution_options(foo='bar')
         eq_(f._execution_options, {'foo':'bar'})
         s = f.select()
         eq_(s._execution_options, {'foo':'bar'})
-        
+
         ret = testing.db.execute(func.now().execution_options(foo='bar'))
         eq_(ret.context.execution_options, {'foo':'bar'})
         ret.close()
-        
-        
+
+
     @engines.close_first
     def test_update(self):
         """
@@ -299,13 +301,13 @@ class ExecuteTest(TestBase):
     @testing.fails_on_everything_except('postgresql')
     def test_as_from(self):
         # TODO: shouldnt this work on oracle too ?
-        x = testing.db.func.current_date().execute().scalar()
-        y = testing.db.func.current_date().select().execute().scalar()
-        z = testing.db.func.current_date().scalar()
-        w = select(['*'], from_obj=[testing.db.func.current_date()]).scalar()
+        x = func.current_date(bind=testing.db).execute().scalar()
+        y = func.current_date(bind=testing.db).select().execute().scalar()
+        z = func.current_date(bind=testing.db).scalar()
+        w = select(['*'], from_obj=[func.current_date(bind=testing.db)]).scalar()
 
         # construct a column-based FROM object out of a function, like in [ticket:172]
-        s = select([sql.column('date', type_=DateTime)], from_obj=[testing.db.func.current_date()])
+        s = select([sql.column('date', type_=DateTime)], from_obj=[func.current_date(bind=testing.db)])
         q = s.execute().first()[s.c.date]
         r = s.alias('datequery').select().scalar()
 
