@@ -1,13 +1,12 @@
-from sqlalchemy.test.testing import eq_
+from test.lib.testing import eq_, assert_raises_message, assert_raises
 import datetime
 from sqlalchemy import *
 from sqlalchemy import exc, sql, util
 from sqlalchemy.engine import default, base
-from sqlalchemy.test import *
-from sqlalchemy.test.testing import eq_, assert_raises_message, assert_raises
-from sqlalchemy.test.schema import Table, Column
+from test.lib import *
+from test.lib.schema import Table, Column
 
-class QueryTest(TestBase):
+class QueryTest(fixtures.TestBase):
 
     @classmethod
     def setup_class(cls):
@@ -49,8 +48,11 @@ class QueryTest(TestBase):
     def test_insert_heterogeneous_params(self):
         """test that executemany parameters are asserted to match the parameter set of the first."""
 
-        assert_raises_message(exc.InvalidRequestError, 
-            "A value is required for bind parameter 'user_name', in parameter group 2",
+        assert_raises_message(exc.StatementError, 
+            r"A value is required for bind parameter 'user_name', in "
+            "parameter group 2 \(original cause: (sqlalchemy.exc.)?InvalidRequestError: A "
+            "value is required for bind parameter 'user_name', in "
+            "parameter group 2\) 'INSERT INTO query_users",
             users.insert().execute,
             {'user_id':7, 'user_name':'jack'},
             {'user_id':8, 'user_name':'ed'},
@@ -252,6 +254,7 @@ class QueryTest(TestBase):
         ORDER BY.
 
         """
+
         users.insert().execute(
             {'user_id':7, 'user_name':'jack'},
             {'user_id':8, 'user_name':'ed'},
@@ -466,6 +469,7 @@ class QueryTest(TestBase):
         This should be run for DB-APIs with both positional and named
         paramstyles.
         """
+
         users.insert().execute(user_id = 7, user_name = 'jack')
         users.insert().execute(user_id = 8, user_name = 'fred')
 
@@ -505,6 +509,27 @@ class QueryTest(TestBase):
         a_eq(prep(r"(\:that$other)"), "(:that$other)")
         a_eq(prep(r".\:that$ :other."), ".:that$ ?.")
 
+    def test_select_from_bindparam(self):
+        """Test result row processing when selecting from a plain bind param."""
+
+        class MyInteger(TypeDecorator):
+            impl = Integer
+            def process_bind_param(self, value, dialect):
+                return int(value[4:])
+
+            def process_result_value(self, value, dialect):
+                return "INT_%d" % value
+
+        eq_(
+            testing.db.scalar(select([literal("INT_5", type_=MyInteger)])),
+            "INT_5"
+        )
+        eq_(
+            testing.db.scalar(select([literal("INT_5", type_=MyInteger).label('foo')])),
+            "INT_5"
+        )
+
+
     def test_delete(self):
         users.insert().execute(user_id = 7, user_name = 'jack')
         users.insert().execute(user_id = 8, user_name = 'fred')
@@ -519,6 +544,7 @@ class QueryTest(TestBase):
     @testing.exclude('mysql', '<', (5, 0, 37), 'database bug')
     def test_scalar_select(self):
         """test that scalar subqueries with labels get their type propagated to the result set."""
+
         # mysql and/or mysqldb has a bug here, type isn't propagated for scalar
         # subquery.
         datetable = Table('datetable', metadata,
@@ -596,6 +622,62 @@ class QueryTest(TestBase):
                         order_by=[users.c.user_id.desc()]),
                  [(3,), (2,), (1,)])
 
+    @testing.requires.nullsordering
+    def test_order_by_nulls(self):
+        """Exercises ORDER BY clause generation.
+
+        Tests simple, compound, aliased and DESC clauses.
+        """
+
+        users.insert().execute(user_id=1)
+        users.insert().execute(user_id=2, user_name='b')
+        users.insert().execute(user_id=3, user_name='a')
+
+        def a_eq(executable, wanted):
+            got = list(executable.execute())
+            eq_(got, wanted)
+
+        for labels in False, True:
+            a_eq(users.select(order_by=[users.c.user_name.nullsfirst()],
+                              use_labels=labels),
+                 [(1, None), (3, 'a'), (2, 'b')])
+
+            a_eq(users.select(order_by=[users.c.user_name.nullslast()],
+                              use_labels=labels),
+                 [(3, 'a'), (2, 'b'), (1, None)])
+
+            a_eq(users.select(order_by=[asc(users.c.user_name).nullsfirst()],
+                              use_labels=labels),
+                 [(1, None), (3, 'a'), (2, 'b')])
+
+            a_eq(users.select(order_by=[asc(users.c.user_name).nullslast()],
+                              use_labels=labels),
+                 [(3, 'a'), (2, 'b'), (1, None)])
+
+            a_eq(users.select(order_by=[users.c.user_name.desc().nullsfirst()],
+                              use_labels=labels),
+                 [(1, None), (2, 'b'), (3, 'a')])
+
+            a_eq(users.select(order_by=[users.c.user_name.desc().nullslast()],
+                              use_labels=labels),
+                 [(2, 'b'), (3, 'a'), (1, None)])
+
+            a_eq(users.select(order_by=[desc(users.c.user_name).nullsfirst()],
+                              use_labels=labels),
+                 [(1, None), (2, 'b'), (3, 'a')])
+
+            a_eq(users.select(order_by=[desc(users.c.user_name).nullslast()],
+                              use_labels=labels),
+                 [(2, 'b'), (3, 'a'), (1, None)])
+
+            a_eq(users.select(order_by=[users.c.user_name.nullsfirst(), users.c.user_id],
+                              use_labels=labels),
+                 [(1, None), (3, 'a'), (2, 'b')])
+
+            a_eq(users.select(order_by=[users.c.user_name.nullslast(), users.c.user_id],
+                              use_labels=labels),
+                 [(3, 'a'), (2, 'b'), (1, None)])
+
     @testing.fails_on("+pyodbc", "pyodbc row doesn't seem to accept slices")
     def test_column_slices(self):
         users.insert().execute(user_id=1, user_name='john')
@@ -704,6 +786,23 @@ class QueryTest(TestBase):
             )
             trans.rollback()
 
+    def test_no_inserted_pk_on_non_insert(self):
+        result = testing.db.execute("select * from query_users")
+        assert_raises_message(
+            exc.InvalidRequestError,
+            r"Statement is not an insert\(\) expression construct.",
+            getattr, result, 'inserted_primary_key'
+        )
+
+    @testing.requires.returning
+    def test_no_inserted_pk_on_returning(self):
+        result = testing.db.execute(users.insert().returning(users.c.user_id, users.c.user_name))
+        assert_raises_message(
+            exc.InvalidRequestError,
+            r"Can't call inserted_primary_key when returning\(\) is used.",
+            getattr, result, 'inserted_primary_key'
+        )
+
     def test_fetchone_til_end(self):
         result = testing.db.execute("select * from query_users")
         eq_(result.fetchone(), None)
@@ -808,8 +907,8 @@ class QueryTest(TestBase):
     def test_cant_execute_join(self):
         try:
             users.join(addresses).execute()
-        except exc.ArgumentError, e:
-            assert str(e).startswith('Not an executable clause: ')
+        except exc.StatementError, e:
+            assert str(e).startswith('Not an executable clause ')
 
 
 
@@ -906,6 +1005,7 @@ class QueryTest(TestBase):
         generate ? = ?.
 
         """
+
         users.insert().execute(user_id = 7, user_name = 'jack')
         users.insert().execute(user_id = 8, user_name = 'fred')
         users.insert().execute(user_id = 9, user_name = None)
@@ -955,7 +1055,7 @@ class QueryTest(TestBase):
         r = s.execute().fetchall()
         assert len(r) == 1
 
-class PercentSchemaNamesTest(TestBase):
+class PercentSchemaNamesTest(fixtures.TestBase):
     """tests using percent signs, spaces in table and column names.
 
     Doesn't pass for mysql, postgresql, but this is really a 
@@ -1055,7 +1155,7 @@ class PercentSchemaNamesTest(TestBase):
 
 
 
-class LimitTest(TestBase):
+class LimitTest(fixtures.TestBase):
 
     @classmethod
     def setup_class(cls):
@@ -1128,7 +1228,7 @@ class LimitTest(TestBase):
         self.assert_(len(r) == 3, repr(r))
         self.assert_(r[0] != r[1] and r[1] != r[2], repr(r))
 
-class CompoundTest(TestBase):
+class CompoundTest(fixtures.TestBase):
     """test compound statements like UNION, INTERSECT, particularly their ability to nest on
     different databases."""
     @classmethod
@@ -1255,6 +1355,7 @@ class CompoundTest(TestBase):
         more palatable to a wider variety of engines.
 
         """
+
         u = union(
             select([t1.c.col3]),
             select([t1.c.col3]),
@@ -1420,7 +1521,7 @@ class CompoundTest(TestBase):
         eq_(found, wanted)
 
 
-class JoinTest(TestBase):
+class JoinTest(fixtures.TestBase):
     """Tests join execution.
 
     The compiled SQL emitted by the dialect might be ANSI joins or
@@ -1692,7 +1793,7 @@ class JoinTest(TestBase):
             self.assertRows(expr, [(10, 20, 30)])
 
 
-class OperatorTest(TestBase):
+class OperatorTest(fixtures.TestBase):
     @classmethod
     def setup_class(cls):
         global metadata, flds
@@ -1722,5 +1823,14 @@ class OperatorTest(TestBase):
             [(2,),(1,)]
         )
 
+    @testing.requires.window_functions
+    def test_over(self):
+        eq_(
+            select([
+                flds.c.intcol, func.row_number().over(order_by=flds.c.strcol)
+            ]).execute().fetchall(),
+            [(13, 1L), (5, 2L)]
+        )
+
 
 
