@@ -1,13 +1,14 @@
 """Test various algorithmic properties of selectables."""
 
-from sqlalchemy.test.testing import eq_, assert_raises, \
+from test.lib.testing import eq_, assert_raises, \
     assert_raises_message
 from sqlalchemy import *
-from sqlalchemy.test import *
+from test.lib import *
 from sqlalchemy.sql import util as sql_util, visitors
 from sqlalchemy import exc
 from sqlalchemy.sql import table, column, null
 from sqlalchemy import util
+from test.lib import fixtures
 
 metadata = MetaData()
 table1 = Table('table1', metadata,
@@ -26,7 +27,8 @@ table2 = Table('table2', metadata,
 )
 
 
-class SelectableTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
+class SelectableTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
+    __dialect__ = 'default'
 
     def test_indirect_correspondence_on_labels(self):
         # this test depends upon 'distance' to
@@ -45,6 +47,33 @@ class SelectableTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         assert s.corresponding_column(s.c.col1) is s.c.col1
         assert s.corresponding_column(s.c.c1) is s.c.c1
 
+    def test_labeled_subquery_twice(self):
+        scalar_select = select([table1.c.col1]).label('foo')
+
+        s1 = select([scalar_select])
+        s2 = select([scalar_select, scalar_select])
+
+        eq_(
+            s1.c.foo.proxy_set,
+            set([s1.c.foo, scalar_select, scalar_select.element, table1.c.col1])
+        )
+        eq_(
+            s2.c.foo.proxy_set,
+            set([s2.c.foo, scalar_select, scalar_select.element, table1.c.col1])
+        )
+
+        assert s1.corresponding_column(scalar_select) is s1.c.foo
+        assert s2.corresponding_column(scalar_select) is s2.c.foo
+    
+    def test_label_grouped_still_corresponds(self):
+        label = select([table1.c.col1]).label('foo')
+        label2 = label.self_group()
+        
+        s1 = select([label])
+        s2 = select([label2])
+        assert s1.corresponding_column(label) is s1.c.foo
+        assert s2.corresponding_column(label) is s2.c.foo
+        
     def test_direct_correspondence_on_labels(self):
         # this test depends on labels being part
         # of the proxy set to get the right result
@@ -301,6 +330,85 @@ class SelectableTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
 
         assert_raises(exc.NoReferencedTableError, s.join, t1)
 
+    def test_multi_label_chain_naming_col(self):
+        # See [ticket:2167] for this one.
+        l1 = table1.c.col1.label('a')
+        l2 = select([l1]).label('b')
+        s = select([l2])
+        assert s.c.b is not None
+        self.assert_compile(
+            s.select(),
+            "SELECT b FROM (SELECT (SELECT table1.col1 AS a FROM table1) AS b)"
+        )
+
+        s2 = select([s.label('c')])
+        self.assert_compile(
+            s2.select(),
+            "SELECT c FROM (SELECT (SELECT (SELECT table1.col1 AS a FROM table1) AS b) AS c)"
+        )
+
+    def test_unusual_column_elements_text(self):
+        """test that .c excludes text()."""
+
+        s = select([table1.c.col1, text("foo")])
+        eq_(
+            list(s.c),
+            [s.c.col1]
+        )
+
+    def test_unusual_column_elements_clauselist(self):
+        """Test that raw ClauseList is expanded into .c."""
+
+        from sqlalchemy.sql.expression import ClauseList
+        s = select([table1.c.col1, ClauseList(table1.c.col2, table1.c.col3)])
+        eq_(
+            list(s.c),
+            [s.c.col1, s.c.col2, s.c.col3]
+        )
+
+    def test_unusual_column_elements_boolean_clauselist(self):
+        """test that BooleanClauseList is placed as single element in .c."""
+
+        c2 = and_(table1.c.col2==5, table1.c.col3==4)
+        s = select([table1.c.col1, c2])
+        eq_(
+            list(s.c),
+            [s.c.col1, s.corresponding_column(c2)]
+        )
+
+class AnonLabelTest(fixtures.TestBase):
+    """Test behaviors that we hope to change with [ticket:2168]."""
+
+    def test_anon_labels_named_column(self):
+        c1 = column('x')
+
+        # surprising
+        assert c1.label(None) is c1
+        eq_(str(select([c1.label(None)])), "SELECT x")
+
+    def test_anon_labels_literal_column(self):
+        c1 = literal_column('x')
+        assert c1.label(None) is c1
+        eq_(str(select([c1.label(None)])), "SELECT x")
+
+    def test_anon_labels_func(self):
+        c1 = func.count('*')
+        assert c1.label(None) is not c1
+
+        eq_(str(select([c1])), "SELECT count(:param_1) AS count_1")
+        c2 = select([c1]).compile()
+
+        eq_(str(select([c1.label(None)])), "SELECT count(:param_1) AS count_1")
+
+    def test_named_labels_named_column(self):
+        c1 = column('x')
+        eq_(str(select([c1.label('y')])), "SELECT x AS y")
+
+    def test_named_labels_literal_column(self):
+        c1 = literal_column('x')
+        eq_(str(select([c1.label('y')])), "SELECT x AS y")
+
+class JoinConditionTest(fixtures.TestBase, AssertsExecutionResults):
 
     def test_join_condition(self):
         m = MetaData()
@@ -371,56 +479,68 @@ class SelectableTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
                               "side to a subquery using alias\(\)\?",
                               t1t2.join, t2t3.select(use_labels=True))
 
-    def test_multi_label_chain_naming_col(self):
-        # See [ticket:2167] for this one.
-        l1 = table1.c.col1.label('a')
-        l2 = select([l1]).label('b')
-        s = select([l2])
-        assert s.c.b is not None
-        self.assert_compile(
-            s.select(),
-            "SELECT b FROM (SELECT (SELECT table1.col1 AS a FROM table1) AS b)"
+
+    def test_join_cond_no_such_unrelated_table(self):
+        m = MetaData()
+        # bounding the "good" column with two "bad" ones is so to 
+        # try to get coverage to get the "continue" statements
+        # in the loop...
+        t1 = Table('t1', m, 
+                            Column('y', Integer, ForeignKey('t22.id')),
+                            Column('x', Integer, ForeignKey('t2.id')), 
+                            Column('q', Integer, ForeignKey('t22.id')), 
+                            )
+        t2 = Table('t2', m, Column('id', Integer))
+        assert sql_util.join_condition(t1, t2).compare(t1.c.x==t2.c.id)
+        assert sql_util.join_condition(t2, t1).compare(t1.c.x==t2.c.id)
+
+    def test_join_cond_no_such_unrelated_column(self):
+        m = MetaData()
+        t1 = Table('t1', m, Column('x', Integer, ForeignKey('t2.id')), 
+                            Column('y', Integer, ForeignKey('t3.q')))
+        t2 = Table('t2', m, Column('id', Integer))
+        t3 = Table('t3', m, Column('id', Integer))
+        assert sql_util.join_condition(t1, t2).compare(t1.c.x==t2.c.id)
+        assert sql_util.join_condition(t2, t1).compare(t1.c.x==t2.c.id)
+
+    def test_join_cond_no_such_related_table(self):
+        m1 = MetaData()
+        m2 = MetaData()
+        t1 = Table('t1', m1, Column('x', Integer, ForeignKey('t2.id')))
+        t2 = Table('t2', m2, Column('id', Integer))
+        assert_raises_message(
+            exc.NoReferencedTableError,
+            "Foreign key associated with column 't1.x' could not find "
+            "table 't2' with which to generate a foreign key to "
+            "target column 'id'",
+            sql_util.join_condition, t1, t2
         )
 
-        s2 = select([s.label('c')])
-        self.assert_compile(
-            s2.select(),
-            "SELECT c FROM (SELECT (SELECT (SELECT table1.col1 AS a FROM table1) AS b) AS c)"
+        assert_raises_message(
+            exc.NoReferencedTableError,
+            "Foreign key associated with column 't1.x' could not find "
+            "table 't2' with which to generate a foreign key to "
+            "target column 'id'",
+            sql_util.join_condition, t2, t1
         )
 
-class AnonLabelTest(TestBase):
-    """Test behaviors that we hope to change with [ticket:2168]."""
+    def test_join_cond_no_such_related_column(self):
+        m = MetaData()
+        t1 = Table('t1', m, Column('x', Integer, ForeignKey('t2.q')))
+        t2 = Table('t2', m, Column('id', Integer))
+        assert_raises_message(
+            exc.NoReferencedColumnError,
+            "Could not create ForeignKey 't2.q' on table 't1': table 't2' has no column named 'q'",
+            sql_util.join_condition, t1, t2
+        )
 
-    def test_anon_labels_named_column(self):
-        c1 = column('x')
+        assert_raises_message(
+            exc.NoReferencedColumnError,
+            "Could not create ForeignKey 't2.q' on table 't1': table 't2' has no column named 'q'",
+            sql_util.join_condition, t2, t1
+        )
 
-        # surprising
-        assert c1.label(None) is c1
-        eq_(str(select([c1.label(None)])), "SELECT x")
-
-    def test_anon_labels_literal_column(self):
-        c1 = literal_column('x')
-        assert c1.label(None) is c1
-        eq_(str(select([c1.label(None)])), "SELECT x")
-
-    def test_anon_labels_func(self):
-        c1 = func.count('*')
-        assert c1.label(None) is not c1
-
-        eq_(str(select([c1])), "SELECT count(:param_1) AS count_1")
-        c2 = select([c1]).compile()
-
-        eq_(str(select([c1.label(None)])), "SELECT count(:param_1) AS count_1")
-
-    def test_named_labels_named_column(self):
-        c1 = column('x')
-        eq_(str(select([c1.label('y')])), "SELECT x AS y")
-
-    def test_named_labels_literal_column(self):
-        c1 = literal_column('x')
-        eq_(str(select([c1.label('y')])), "SELECT x AS y")
-
-class PrimaryKeyTest(TestBase, AssertsExecutionResults):
+class PrimaryKeyTest(fixtures.TestBase, AssertsExecutionResults):
 
     def test_join_pk_collapse_implicit(self):
         """test that redundant columns in a join get 'collapsed' into a
@@ -524,7 +644,7 @@ class PrimaryKeyTest(TestBase, AssertsExecutionResults):
             util.column_set([employee.c.id]))
 
 
-class ReduceTest(TestBase, AssertsExecutionResults):
+class ReduceTest(fixtures.TestBase, AssertsExecutionResults):
     def test_reduce(self):
         meta = MetaData()
         t1 = Table('t1', meta,
@@ -683,7 +803,7 @@ class ReduceTest(TestBase, AssertsExecutionResults):
             pjoin.c.page_id, pjoin.c.magazine_page_id])),
             util.column_set([pjoin.c.id]))
 
-class DerivedTest(TestBase, AssertsExecutionResults):
+class DerivedTest(fixtures.TestBase, AssertsExecutionResults):
     def test_table(self):
         meta = MetaData()
 
@@ -725,7 +845,25 @@ class DerivedTest(TestBase, AssertsExecutionResults):
         assert select([t1, t2]).alias('foo').is_derived_from(t1)
         assert not t2.select().alias('foo').is_derived_from(t1)
 
-class AnnotationsTest(TestBase):
+class AnnotationsTest(fixtures.TestBase):
+
+    def test_hashing(self):
+        t = table('t', column('x'))
+
+        a = t.alias()
+        s = t.select()
+        s2 = a.select()
+
+        for obj in [
+            t,
+            t.c.x,
+            a,
+            s,
+            s2
+        ]:
+            annot = obj._annotate({})
+            eq_(set([obj]), set([annot]))
+
     def test_custom_constructions(self):
         from sqlalchemy.schema import Column
         class MyColumn(Column):
@@ -794,6 +932,13 @@ class AnnotationsTest(TestBase):
         b5 = visitors.cloned_traverse(b3, {}, {'binary':visit_binary})
         assert str(b5) == ":bar = table1.col2"
 
+    def test_annotate_aliased(self):
+        t1 = table('t1', column('c1'))
+        s = select([(t1.c.c1 + 3).label('bat')])
+        a = s.alias()
+        a = sql_util._deep_annotate(a, {'foo': 'bar'})
+        eq_(a._annotations['foo'], 'bar')
+        eq_(a.element._annotations['foo'], 'bar')
 
     def test_annotate_expressions(self):
         table1 = table('table1', column('col1'), column('col2'))
