@@ -1,21 +1,48 @@
 # coding: utf-8
-from sqlalchemy.test.testing import eq_, assert_raises, assert_raises_message
+from test.lib.testing import eq_, assert_raises, assert_raises_message
 import decimal
 import datetime, os, re
 from sqlalchemy import *
-from sqlalchemy import exc, types, util, schema
+from sqlalchemy import exc, types, util, schema, dialects
+for name in dialects.__all__:
+    __import__("sqlalchemy.dialects.%s" % name)
 from sqlalchemy.sql import operators, column, table
-from sqlalchemy.test.testing import eq_
+from test.lib.testing import eq_
 import sqlalchemy.engine.url as url
-from sqlalchemy.databases import *
-from sqlalchemy.test.schema import Table, Column
-from sqlalchemy.test import *
-from sqlalchemy.test.util import picklers
-from decimal import Decimal
-from sqlalchemy.test.util import round_decimal
+from sqlalchemy.engine import default
+from test.lib.schema import Table, Column
+from test.lib import *
+from test.lib.util import picklers
+from sqlalchemy.util.compat import decimal
+from test.lib.util import round_decimal
+from test.lib import fixtures
 
+class AdaptTest(fixtures.TestBase):
+    def _all_dialect_modules(self):
+        return [
+            getattr(dialects, d)
+            for d in dialects.__all__
+            if not d.startswith('_')
+        ]
+
+    def _all_dialects(self):
+        return [d.base.dialect() for d in 
+                self._all_dialect_modules()]
+
+    def _all_types(self):
+        def types_for_mod(mod):
+            for key in dir(mod):
+                typ = getattr(mod, key)
+                if not isinstance(typ, type) or not issubclass(typ, types.TypeEngine):
+                    continue
+                yield typ
+
+        for typ in types_for_mod(types):
+            yield typ
+        for dialect in self._all_dialect_modules():
+            for typ in types_for_mod(dialect):
+                yield typ
 
-class AdaptTest(TestBase):
     def test_uppercase_rendering(self):
         """Test that uppercase types from types.py always render as their
         type.
@@ -27,13 +54,9 @@ class AdaptTest(TestBase):
 
         """
 
-        for dialect in [
-                oracle.dialect(), 
-                mysql.dialect(), 
-                postgresql.dialect(), 
-                sqlite.dialect(), 
-                mssql.dialect()]: 
+        for dialect in self._all_dialects():
             for type_, expected in (
+                (REAL, "REAL"),
                 (FLOAT, "FLOAT"),
                 (NUMERIC, "NUMERIC"),
                 (DECIMAL, "DECIMAL"),
@@ -49,14 +72,17 @@ class AdaptTest(TestBase):
                                     "NVARCHAR2(10)")),
                 (CHAR, "CHAR"),
                 (NCHAR, ("NCHAR", "NATIONAL CHAR")),
-                (BLOB, "BLOB"),
-                (BOOLEAN, ("BOOLEAN", "BOOL"))
+                (BLOB, ("BLOB", "BLOB SUB_TYPE 0")),
+                (BOOLEAN, ("BOOLEAN", "BOOL", "INTEGER"))
             ):
                 if isinstance(expected, str):
                     expected = (expected, )
 
-                compiled = types.to_instance(type_).\
+                try:
+                    compiled = types.to_instance(type_).\
                             compile(dialect=dialect)
+                except NotImplementedError:
+                    continue
 
                 assert compiled in expected, \
                     "%r matches none of %r for dialect %s" % \
@@ -66,7 +92,65 @@ class AdaptTest(TestBase):
                     "default str() of type %r not expected, %r" % \
                     (type_, expected)
 
-class TypeAffinityTest(TestBase):
+    @testing.uses_deprecated()
+    def test_adapt_method(self):
+        """ensure all types have a working adapt() method,
+        which creates a distinct copy.
+
+        The distinct copy ensures that when we cache
+        the adapted() form of a type against the original
+        in a weak key dictionary, a cycle is not formed.
+
+        This test doesn't test type-specific arguments of
+        adapt() beyond their defaults.
+
+        """
+
+        for typ in self._all_types():
+            if typ in (types.TypeDecorator, types.TypeEngine, types.Variant):
+                continue
+            elif typ is dialects.postgresql.ARRAY:
+                t1 = typ(String)
+            else:
+                t1 = typ()
+            for cls in [typ] + typ.__subclasses__():
+                if not issubclass(typ, types.Enum) and \
+                    issubclass(cls, types.Enum):
+                    continue
+                t2 = t1.adapt(cls)
+                assert t1 is not t2
+                for k in t1.__dict__:
+                    if k == 'impl':
+                        continue
+                    # assert each value was copied, or that
+                    # the adapted type has a more specific
+                    # value than the original (i.e. SQL Server
+                    # applies precision=24 for REAL)
+                    assert \
+                        getattr(t2, k) == t1.__dict__[k] or \
+                        t1.__dict__[k] is None
+
+    @testing.uses_deprecated()
+    def test_repr(self):
+        for typ in self._all_types():
+            if typ in (types.TypeDecorator, types.TypeEngine, types.Variant):
+                continue
+            elif typ is dialects.postgresql.ARRAY:
+                t1 = typ(String)
+            else:
+                t1 = typ()
+            repr(t1)
+
+    def test_plain_init_deprecation_warning(self):
+        for typ in (Integer, Date, SmallInteger):
+            assert_raises_message(
+                exc.SADeprecationWarning,
+                "Passing arguments to type object "
+                "constructor %s is deprecated" % typ,
+                typ, 11
+            )
+
+class TypeAffinityTest(fixtures.TestBase):
     def test_type_affinity(self):
         for type_, affin in [
             (String(), String),
@@ -89,7 +173,7 @@ class TypeAffinityTest(TestBase):
         ]:
             eq_(t1._compare_type_affinity(t2), comp, "%s %s" % (t1, t2))
 
-class PickleMetadataTest(TestBase):
+class PickleMetadataTest(fixtures.TestBase):
     def testmeta(self):
         for loads, dumps in picklers():
             column_types = [
@@ -118,12 +202,11 @@ class PickleMetadataTest(TestBase):
                 mt = loads(dumps(meta))
 
 
-class UserDefinedTest(TestBase, AssertsCompiledSQL):
+class UserDefinedTest(fixtures.TablesTest, AssertsCompiledSQL):
     """tests user-defined types."""
 
     def test_processing(self):
-
-        global users
+        users = self.tables.users
         users.insert().execute(
             user_id=2, goofy='jack', goofy2='jack', goofy4=u'jack',
             goofy7=u'jack', goofy8=12, goofy9=12)
@@ -155,7 +238,7 @@ class UserDefinedTest(TestBase, AssertsCompiledSQL):
             (Float(2), "FLOAT(2)", {'precision':4}),
             (Numeric(19, 2), "NUMERIC(19, 2)", {}),
         ]:
-            for dialect_ in (postgresql, mssql, mysql):
+            for dialect_ in (dialects.postgresql, dialects.mssql, dialects.mysql):
                 dialect_ = dialect_.dialect()
 
                 raw_impl = types.to_instance(impl_, **kw)
@@ -188,8 +271,8 @@ class UserDefinedTest(TestBase, AssertsCompiledSQL):
                 else:
                     return super(MyType, self).load_dialect_impl(dialect)
 
-        sl = sqlite.dialect()
-        pg = postgresql.dialect()
+        sl = dialects.sqlite.dialect()
+        pg = dialects.postgresql.dialect()
         t = MyType()
         self.assert_compile(t, "VARCHAR(50)", dialect=sl)
         self.assert_compile(t, "FLOAT", dialect=pg)
@@ -202,10 +285,47 @@ class UserDefinedTest(TestBase, AssertsCompiledSQL):
                 Float().dialect_impl(pg).__class__
         )
 
+    def test_user_defined_typedec_impl_bind(self):
+        class TypeOne(types.TypeEngine):
+            def bind_processor(self, dialect):
+                def go(value):
+                    return value + " ONE"
+                return go
+
+        class TypeTwo(types.TypeEngine):
+            def bind_processor(self, dialect):
+                def go(value):
+                    return value + " TWO"
+                return go
+
+        class MyType(types.TypeDecorator):
+            impl = TypeOne
+
+            def load_dialect_impl(self, dialect):
+                if dialect.name == 'sqlite':
+                    return TypeOne()
+                else:
+                    return TypeTwo()
+
+            def process_bind_param(self, value, dialect):
+                return "MYTYPE " + value
+        sl = dialects.sqlite.dialect()
+        pg = dialects.postgresql.dialect()
+        t = MyType()
+        eq_(
+            t._cached_bind_processor(sl)('foo'),
+            "MYTYPE foo ONE"
+        )
+        eq_(
+            t._cached_bind_processor(pg)('foo'),
+            "MYTYPE foo TWO"
+        )
+
     @testing.provide_metadata
     def test_type_coerce(self):
         """test ad-hoc usage of custom types with type_coerce()."""
 
+        metadata = self.metadata
         class MyType(types.TypeDecorator):
             impl = String
 
@@ -259,9 +379,7 @@ class UserDefinedTest(TestBase, AssertsCompiledSQL):
         )
 
     @classmethod
-    def setup_class(cls):
-        global users, metadata
-
+    def define_tables(cls, metadata):
         class MyType(types.UserDefinedType):
             def get_col_spec(self):
                 return "VARCHAR(100)"
@@ -345,8 +463,7 @@ class UserDefinedTest(TestBase, AssertsCompiledSQL):
             def copy(self):
                 return MyUnicodeType(self.impl.length)
 
-        metadata = MetaData(testing.db)
-        users = Table('type_users', metadata,
+        Table('users', metadata,
             Column('user_id', Integer, primary_key = True),
             # totall custom type
             Column('goofy', MyType, nullable = False),
@@ -360,13 +477,112 @@ class UserDefinedTest(TestBase, AssertsCompiledSQL):
             Column('goofy9', MyNewIntSubClass, nullable = False),
         )
 
-        metadata.create_all()
+class VariantTest(fixtures.TestBase, AssertsCompiledSQL):
+    def setup(self):
+        class UTypeOne(types.UserDefinedType):
+            def get_col_spec(self):
+                return "UTYPEONE"
+            def bind_processor(self, dialect):
+                def process(value):
+                    return value + "UONE"
+                return process
 
-    @classmethod
-    def teardown_class(cls):
-        metadata.drop_all()
+        class UTypeTwo(types.UserDefinedType):
+            def get_col_spec(self):
+                return "UTYPETWO"
+            def bind_processor(self, dialect):
+                def process(value):
+                    return value + "UTWO"
+                return process
 
-class UnicodeTest(TestBase, AssertsExecutionResults):
+        class UTypeThree(types.UserDefinedType):
+            def get_col_spec(self):
+                return "UTYPETHREE"
+
+        self.UTypeOne = UTypeOne
+        self.UTypeTwo = UTypeTwo
+        self.UTypeThree = UTypeThree
+        self.variant = self.UTypeOne().with_variant(
+                            self.UTypeTwo(), 'postgresql')
+        self.composite = self.variant.with_variant(
+                            self.UTypeThree(), 'mysql')
+
+    def test_illegal_dupe(self):
+        v = self.UTypeOne().with_variant(
+            self.UTypeTwo(), 'postgresql'
+        )
+        assert_raises_message(
+            exc.ArgumentError,
+            "Dialect 'postgresql' is already present "
+            "in the mapping for this Variant",
+            lambda: v.with_variant(self.UTypeThree(), 'postgresql')
+        )
+    def test_compile(self):
+        self.assert_compile(
+            self.variant,
+            "UTYPEONE",
+            use_default_dialect=True
+        )
+        self.assert_compile(
+            self.variant,
+            "UTYPEONE",
+            dialect=dialects.mysql.dialect()
+        )
+        self.assert_compile(
+            self.variant,
+            "UTYPETWO",
+            dialect=dialects.postgresql.dialect()
+        )
+
+    def test_compile_composite(self):
+        self.assert_compile(
+            self.composite,
+            "UTYPEONE",
+            use_default_dialect=True
+        )
+        self.assert_compile(
+            self.composite,
+            "UTYPETHREE",
+            dialect=dialects.mysql.dialect()
+        )
+        self.assert_compile(
+            self.composite,
+            "UTYPETWO",
+            dialect=dialects.postgresql.dialect()
+        )
+
+    def test_bind_process(self):
+        eq_(
+            self.variant._cached_bind_processor(
+                    dialects.mysql.dialect())('foo'),
+            'fooUONE'
+        )
+        eq_(
+            self.variant._cached_bind_processor(
+                    default.DefaultDialect())('foo'),
+            'fooUONE'
+        )
+        eq_(
+            self.variant._cached_bind_processor(
+                    dialects.postgresql.dialect())('foo'),
+            'fooUTWO'
+        )
+
+    def test_bind_process_composite(self):
+        assert self.composite._cached_bind_processor(
+                    dialects.mysql.dialect()) is None
+        eq_(
+            self.composite._cached_bind_processor(
+                    default.DefaultDialect())('foo'),
+            'fooUONE'
+        )
+        eq_(
+            self.composite._cached_bind_processor(
+                    dialects.postgresql.dialect())('foo'),
+            'fooUTWO'
+        )
+
+class UnicodeTest(fixtures.TestBase, AssertsExecutionResults):
     """tests the Unicode type.  also tests the TypeDecorator with instances in the types package."""
 
     @classmethod
@@ -421,7 +637,7 @@ class UnicodeTest(TestBase, AssertsExecutionResults):
 
     def test_round_trip(self):
         unicodedata = u"Alors vous imaginez ma surprise, au lever du jour, "\
-                    u"quand une drôle de petit voix m’a réveillé. Elle "\
+                    u"quand une drôle de petite voix m’a réveillé. Elle "\
                     u"disait: « S’il vous plaît… dessine-moi un mouton! »"
 
         unicode_table.insert().execute(unicode_varchar=unicodedata,unicode_text=unicodedata)
@@ -437,7 +653,7 @@ class UnicodeTest(TestBase, AssertsExecutionResults):
         # vs. cursor.execute()
 
         unicodedata = u"Alors vous imaginez ma surprise, au lever du jour, quand "\
-                        u"une drôle de petit voix m’a réveillé. "\
+                        u"une drôle de petite voix m’a réveillé. "\
                         u"Elle disait: « S’il vous plaît… dessine-moi un mouton! »"
 
         unicode_table.insert().execute(
@@ -455,7 +671,7 @@ class UnicodeTest(TestBase, AssertsExecutionResults):
         """ensure compiler processing works for UNIONs"""
 
         unicodedata = u"Alors vous imaginez ma surprise, au lever du jour, quand "\
-                        u"une drôle de petit voix m’a réveillé. "\
+                        u"une drôle de petite voix m’a réveillé. "\
                         u"Elle disait: « S’il vous plaît… dessine-moi un mouton! »"
 
         unicode_table.insert().execute(unicode_varchar=unicodedata,unicode_text=unicodedata)
@@ -480,7 +696,7 @@ class UnicodeTest(TestBase, AssertsExecutionResults):
         """
 
         unicodedata = u"Alors vous imaginez ma surprise, au lever du jour, quand "\
-                        u"une drôle de petit voix m’a réveillé. "\
+                        u"une drôle de petite voix m’a réveillé. "\
                         u"Elle disait: « S’il vous plaît… dessine-moi un mouton! »"
 
         # using Unicode explicly - warning should be emitted
@@ -533,7 +749,7 @@ class UnicodeTest(TestBase, AssertsExecutionResults):
         """checks String(unicode_error='ignore') is passed to underlying codec."""
 
         unicodedata = u"Alors vous imaginez ma surprise, au lever du jour, quand "\
-                        u"une drôle de petit voix m’a réveillé. "\
+                        u"une drôle de petite voix m’a réveillé. "\
                         u"Elle disait: « S’il vous plaît… dessine-moi un mouton! »"
 
         asciidata = unicodedata.encode('ascii', 'ignore')
@@ -571,7 +787,7 @@ class UnicodeTest(TestBase, AssertsExecutionResults):
             row = engine.execute(utf8_table.select()).first()
             x = row['plain_varchar_no_coding_error']
             connect_opts = engine.dialect.create_connect_args(testing.db.url)[1]
-            if connect_opts.get('use_unicode', False):
+            if isinstance(x, unicode):
                 x = x.encode('utf-8')
             a = hexlify(x)
             b = hexlify(asciidata)
@@ -610,7 +826,7 @@ class UnicodeTest(TestBase, AssertsExecutionResults):
                 eq_(
                       x,
                       u'Alors vous imaginez ma surprise, au lever du jour, quand une '
-                      u'drle de petit voix ma rveill. Elle disait:  Sil vous plat '
+                      u'drle de petite voix ma rveill. Elle disait:  Sil vous plat '
                       u'dessine-moi un mouton! '
                 )
             elif engine.dialect.returns_unicode_strings:
@@ -623,7 +839,7 @@ class UnicodeTest(TestBase, AssertsExecutionResults):
             m.drop_all(engine)
 
 
-class EnumTest(TestBase):
+class EnumTest(fixtures.TestBase):
     @classmethod
     def setup_class(cls):
         global enum_table, non_native_enum_table, metadata
@@ -697,7 +913,9 @@ class EnumTest(TestBase):
         eq_(e1.adapt(ENUM).name, 'foo')
         eq_(e1.adapt(ENUM).schema, 'bar')
 
-    @testing.fails_on('mysql+mysqldb', "MySQL seems to issue a 'data truncated' warning.")
+    @testing.crashes('mysql', 
+                    'Inconsistent behavior across various OS/drivers'
+                )
     def test_constraint(self):
         assert_raises(exc.DBAPIError, 
             enum_table.insert().execute,
@@ -712,7 +930,7 @@ class EnumTest(TestBase):
             {'id':4, 'someenum':'four'}
         )
 
-class BinaryTest(TestBase, AssertsExecutionResults):
+class BinaryTest(fixtures.TestBase, AssertsExecutionResults):
     __excluded_on__ = (
         ('mysql', '<', (4, 1, 1)),  # screwy varbinary types
         )
@@ -815,7 +1033,9 @@ class BinaryTest(TestBase, AssertsExecutionResults):
         f = os.path.join(os.path.dirname(__file__), "..", name)
         return open(f, mode='rb').read()
 
-class ExpressionTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
+class ExpressionTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
+    __dialect__ = 'default'
+
     @classmethod
     def setup_class(cls):
         global test_table, meta, MyCustomType, MyTypeDec
@@ -1080,25 +1300,27 @@ class ExpressionTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         assert distinct(test_table.c.data).type == test_table.c.data.type
         assert test_table.c.data.distinct().type == test_table.c.data.type
 
-class CompileTest(TestBase, AssertsCompiledSQL):
+class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
     def test_default_compile(self):
         """test that the base dialect of the type object is used
         for default compilation.
 
         """
+
         for type_, expected in (
             (String(), "VARCHAR"),
             (Integer(), "INTEGER"),
-            (postgresql.INET(), "INET"),
-            (postgresql.FLOAT(), "FLOAT"),
-            (mysql.REAL(precision=8, scale=2), "REAL(8, 2)"),
-            (postgresql.REAL(), "REAL"),
+            (dialects.postgresql.INET(), "INET"),
+            (dialects.postgresql.FLOAT(), "FLOAT"),
+            (dialects.mysql.REAL(precision=8, scale=2), "REAL(8, 2)"),
+            (dialects.postgresql.REAL(), "REAL"),
             (INTEGER(), "INTEGER"),
-            (mysql.INTEGER(display_width=5), "INTEGER(5)")
+            (dialects.mysql.INTEGER(display_width=5), "INTEGER(5)")
         ):
-            self.assert_compile(type_, expected)
+            self.assert_compile(type_, expected,
+                                allow_dialect_select=True)
 
-class DateTest(TestBase, AssertsExecutionResults):
+class DateTest(fixtures.TestBase, AssertsExecutionResults):
     @classmethod
     def setup_class(cls):
         global users_with_date, insert_data
@@ -1194,15 +1416,15 @@ class DateTest(TestBase, AssertsExecutionResults):
                      'DateTest mismatch: got:%s expected:%s' % (l, insert_data))
 
     def testtextdate(self):
-        x = testing.db.text(
+        x = testing.db.execute(text(
             "select user_datetime from query_users_with_date",
-            typemap={'user_datetime':DateTime}).execute().fetchall()
+            typemap={'user_datetime':DateTime})).fetchall()
 
         self.assert_(isinstance(x[0][0], datetime.datetime))
 
-        x = testing.db.text(
+        x = testing.db.execute(text(
             "select * from query_users_with_date where user_datetime=:somedate",
-            bindparams=[bindparam('somedate', type_=types.DateTime)]).execute(
+            bindparams=[bindparam('somedate', type_=types.DateTime)]),
             somedate=datetime.datetime(2005, 11, 10, 11, 52, 35)).fetchall()
 
     def testdate2(self):
@@ -1233,7 +1455,7 @@ class DateTest(TestBase, AssertsExecutionResults):
         finally:
             t.drop(checkfirst=True)
 
-class StringTest(TestBase):
+class StringTest(fixtures.TestBase):
 
     @testing.requires.unbounded_varchar
     def test_nolength_string(self):
@@ -1243,7 +1465,7 @@ class StringTest(TestBase):
         foo.create()
         foo.drop()
 
-class NumericTest(TestBase):
+class NumericTest(fixtures.TestBase):
     def setup(self):
         global metadata
         metadata = MetaData(testing.db)
@@ -1269,8 +1491,8 @@ class NumericTest(TestBase):
     def test_numeric_as_decimal(self):
         self._do_test(
             Numeric(precision=8, scale=4),
-            [15.7563, Decimal("15.7563"), None],
-            [Decimal("15.7563"), None], 
+            [15.7563, decimal.Decimal("15.7563"), None],
+            [decimal.Decimal("15.7563"), None], 
         )
 
     def test_numeric_as_float(self):
@@ -1281,7 +1503,7 @@ class NumericTest(TestBase):
 
         self._do_test(
             Numeric(precision=8, scale=4, asdecimal=False),
-            [15.7563, Decimal("15.7563"), None],
+            [15.7563, decimal.Decimal("15.7563"), None],
             [15.7563, None],
             filter_ = filter_
         )
@@ -1289,15 +1511,15 @@ class NumericTest(TestBase):
     def test_float_as_decimal(self):
         self._do_test(
             Float(precision=8, asdecimal=True),
-            [15.7563, Decimal("15.7563"), None],
-            [Decimal("15.7563"), None], 
+            [15.7563, decimal.Decimal("15.7563"), None],
+            [decimal.Decimal("15.7563"), None], 
             filter_ = lambda n:n is not None and round(n, 5) or None
         )
 
     def test_float_as_float(self):
         self._do_test(
             Float(precision=8),
-            [15.7563, Decimal("15.7563")],
+            [15.7563, decimal.Decimal("15.7563")],
             [15.7563],
             filter_ = lambda n:n is not None and round(n, 5) or None
         )
@@ -1366,7 +1588,6 @@ class NumericTest(TestBase):
         )
 
     @testing.fails_on('sqlite', 'TODO')
-    @testing.fails_on('postgresql+pg8000', 'TODO')
     @testing.fails_on("firebird", "Precision must be from 1 to 18")
     @testing.fails_on("sybase+pysybase", "TODO")
     @testing.fails_on('mssql+pymssql', 'FIXME: improve pymssql dec handling')
@@ -1382,9 +1603,61 @@ class NumericTest(TestBase):
             numbers
         )
 
+class NumericRawSQLTest(fixtures.TestBase):
+    """Test what DBAPIs and dialects return without any typing
+    information supplied at the SQLA level.
+
+    """
+    def _fixture(self, metadata, type, data):
+        t = Table('t', metadata,
+            Column("val", type)
+        )
+        metadata.create_all()
+        t.insert().execute(val=data)
+
+    @testing.fails_on('sqlite', "Doesn't provide Decimal results natively")
+    @testing.provide_metadata
+    def test_decimal_fp(self):
+        metadata = self.metadata
+        t = self._fixture(metadata, Numeric(10, 5), decimal.Decimal("45.5"))
+        val = testing.db.execute("select val from t").scalar()
+        assert isinstance(val, decimal.Decimal)
+        eq_(val, decimal.Decimal("45.5"))
+
+    @testing.fails_on('sqlite', "Doesn't provide Decimal results natively")
+    @testing.provide_metadata
+    def test_decimal_int(self):
+        metadata = self.metadata
+        t = self._fixture(metadata, Numeric(10, 5), decimal.Decimal("45"))
+        val = testing.db.execute("select val from t").scalar()
+        assert isinstance(val, decimal.Decimal)
+        eq_(val, decimal.Decimal("45"))
+
+    @testing.provide_metadata
+    def test_ints(self):
+        metadata = self.metadata
+        t = self._fixture(metadata, Integer, 45)
+        val = testing.db.execute("select val from t").scalar()
+        assert isinstance(val, (int, long))
+        eq_(val, 45)
+
+    @testing.provide_metadata
+    def test_float(self):
+        metadata = self.metadata
+        t = self._fixture(metadata, Float, 46.583)
+        val = testing.db.execute("select val from t").scalar()
+        assert isinstance(val, float)
+
+        # some DBAPIs have unusual float handling
+        if testing.against('oracle+cx_oracle', 'mysql+oursql'):
+            eq_(round_decimal(val, 3), 46.583)
+        else:
+            eq_(val, 46.583)
 
 
-class IntervalTest(TestBase, AssertsExecutionResults):
+
+
+class IntervalTest(fixtures.TestBase, AssertsExecutionResults):
     @classmethod
     def setup_class(cls):
         global interval_table, metadata
@@ -1405,6 +1678,13 @@ class IntervalTest(TestBase, AssertsExecutionResults):
     def teardown_class(cls):
         metadata.drop_all()
 
+    def test_non_native_adapt(self):
+        interval = Interval(native=False)
+        adapted = interval.dialect_impl(testing.db.dialect)
+        assert type(adapted) is Interval
+        assert adapted.native is False
+        eq_(str(adapted), "DATETIME")
+
     @testing.fails_on("+pg8000", "Not yet known how to pass values of the INTERVAL type")
     @testing.fails_on("postgresql+zxjdbc", "Not yet known how to pass values of the INTERVAL type")
     @testing.fails_on("oracle+zxjdbc", "Not yet known how to pass values of the INTERVAL type")
@@ -1430,7 +1710,7 @@ class IntervalTest(TestBase, AssertsExecutionResults):
         eq_(row['non_native_interval'], None)
 
 
-class BooleanTest(TestBase, AssertsExecutionResults):
+class BooleanTest(fixtures.TestBase, AssertsExecutionResults):
     @classmethod
     def setup_class(cls):
         global bool_table
@@ -1492,7 +1772,7 @@ class BooleanTest(TestBase, AssertsExecutionResults):
         testing.db.execute(
             "insert into booltest (id, unconstrained_value) values (1, 5)")
 
-class PickleTest(TestBase):
+class PickleTest(fixtures.TestBase):
     def test_eq_comparison(self):
         p1 = PickleType()
 
@@ -1518,7 +1798,7 @@ class PickleTest(TestBase):
         ):
             assert p1.compare_values(p1.copy_value(obj), obj)
 
-class CallableTest(TestBase):
+class CallableTest(fixtures.TestBase):
     @classmethod
     def setup_class(cls):
         global meta
